<?php

static $g_ctrldClient;

class PhonalisaCtrldConnector
{

    private $socket;
    private $lasterr;
    private $domain;

    public function connect()
    {
        $databaseServer = Yii::app()->params['databaseServer'];

        $service_port = 6858; //fixme
        $address = gethostbyname($databaseServer);

        $phonalisaDomain = getPhonalisaDomain();
        if ($phonalisaDomain == false) {
            return false;
        }
        $this->domain = getDBDomainForPhonalisaDomain($phonalisaDomain);

        $this->socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
        if ($this->socket === false) {
            $this->lasterr = "socket_create() failed, Reason: " . socket_strerror(socket_last_error()) . "\n";
            return false;
        }
        $result = @socket_connect($this->socket, $address, $service_port);
        if ($result === false) {
            $this->lasterr = "socket_connect() failed.\nReason: ($result) " . socket_strerror(socket_last_error($this->socket)) . "\n";
            return false;
        }
        return true;
    }

    private static function staticExec($action, $data)
    {
        $client = PhonalisaCtrldConnector::instance();
        $result = $client->execute($action, $data);
        $jObject = json_decode($result);
        return $jObject;
    }

    public function execute($action, $data)
    {
        $data_string = json_encode($data);

        $msg = new TcpMsg();
        $msg->component = "ctrld";
        $msg->name = $action;
        $msg->processType = MsgProcessType::Command;
        $msg->dataType = MsgDataType::Text;
        $msg->msgItems["data"] = $data_string;
        $msg->send($this->socket);

        //get the answer
        $answer = new TcpMsg();
        $answer->recieve($this->socket);
        return $answer->msgItems["data"];
    }

    private static function instance()
    {
        global $g_ctrldClient;
        if ($g_ctrldClient == NULL) {

            $g_ctrldClient = new PhonalisaCtrldConnector();
            if (!$g_ctrldClient->connect()) {
                throw new CDbException(__('GLOBAL_COULD_NOT_CONNECT_TO_CTRLD'));
            }
            Log::d("Connected to Ctrld.");
        }
        return $g_ctrldClient;
    }

    public static function hasConnection()
    {
        try {
            PhonalisaCtrldConnector::getProcesses();
        } catch (Exception $exc) {
            app()->user->setErrorFlashMessage($exc->getMessage());
            return false;
        }
        return true;
    }

    public static function getProcesses()
    {
        $data = array();
        return PhonalisaCtrldConnector::staticExec("getprocesses", $data);
    }

    public static function startProcess($processName)
    {
        $data = array();
        $data['process'] = $processName;
        return PhonalisaCtrldConnector::staticExec("startprocess", $data);
    }

    public static function stopProcess($processName)
    {
        $data = array();
        $data['process'] = $processName;
        return PhonalisaCtrldConnector::staticExec("stopprocess", $data);
    }

    public static function restartProcess($processName)
    {
        $data = array();
        $data['process'] = $processName;
        return PhonalisaCtrldConnector::staticExec("restartprocess", $data);
    }

    public static function shutdownSystem()
    {
        $data = array();
        return PhonalisaCtrldConnector::staticExec("shutdownSystem", $data);
    }

    public static function rebootSystem()
    {
        $data = array();
        return PhonalisaCtrldConnector::staticExec("rebootSystem", $data);
    }

    public static function writeDhcpConfig($data)
    {
        return PhonalisaCtrldConnector::staticExec("writedhcpconfig", $data);
    }

    public static function writeNetworkConfig($data)
    {
        return PhonalisaCtrldConnector::staticExec("writenetworkconfig", $data);
    }

    public static function getDHCPLeases()
    {
        $data = array();
        return PhonalisaCtrldConnector::staticExec("getdhcpleases", $data);
    }
    
    public static function getSystemInfo() 
    {
        return PhonalisaCtrldConnector::staticExec("getsysteminfo", array());
    }

    public static function getSystemLoadInfo() 
    {
        return PhonalisaCtrldConnector::staticExec("systemloadinfo", array());
    }
    
    public static function getMemoryInfo()
    {
        $data = array();
        return PhonalisaCtrldConnector::staticExec("getmeminfo", $data);
    }

    public static function getThermalInfo()
    {
        $data = array();
        return PhonalisaCtrldConnector::staticExec("getthermalinfo", $data);
    }

    public static function writeDnsConfig($data)
    {
        return PhonalisaCtrldConnector::staticExec("writednsconfig", $data);
    }

    public static function writeFirewallConfig($data)
    {
        return PhonalisaCtrldConnector::staticExec("writefirewallconfig", $data);
    }

    public static function writeSSHPublicKey($data)
    {
        return PhonalisaCtrldConnector::staticExec("writesshpubkey", $data);
    }

    public static function firewallBlockIP($data)
    {
        return PhonalisaCtrldConnector::staticExec("firewallblockip", $data);
    }

    public static function firewallUnblockIP($data)
    {
        return PhonalisaCtrldConnector::staticExec("firewallunblockip", $data);
    }

    public static function createNewCA($data)
    {
        return PhonalisaCtrldConnector::staticExec("createnewca", $data);
    }

    public static function createNewServerCert($data)
    {
        return PhonalisaCtrldConnector::staticExec("createnewservercert", $data);
    }

    public static function storePemForService($data)
    {
        return PhonalisaCtrldConnector::staticExec("storepemforservice", $data);
    }

    public static function runSystemUpgrade($autoUpdate = '0')
    {
        $data = array();
        $data['autoupdate'] = $autoUpdate;
        return PhonalisaCtrldConnector::staticExec("runsystemupgrade", $data);
    }

    public static function getSystemUpgradeStatus()
    {
        $data = array();
        return PhonalisaCtrldConnector::staticExec("getsystemupgradestatus", $data);
    }

    public static function getSystemVersion()
    {
        $data = array();
        return PhonalisaCtrldConnector::staticExec("getsystemversion", $data);
    }

    public static function runFactoryReset()
    {
        $data = array();
        return PhonalisaCtrldConnector::staticExec("factoryreset", $data);
    }


}
