<?php

class TargetHelper
{

    public $separator = '_';

    public function getTargetList($targetTypes = array())
    {
        $targetList = array();

        foreach ($targetTypes as $targetType) {
            $targetsForType = $this->getTargetsForType($targetType);
            if ($targetsForType) {
                $title = $this->getTranslationForTargetType($targetType);
                $targetList[$title] = $targetsForType;
            }
        }

        return $targetList;
    }

    public function getModelForTargetType($targetType)
    {
        switch ($targetType) {
            case TargetType::User:
            case TargetType::UserVoiceMail:
                return User::model();
                break;
            case TargetType::Group:
            case TargetType::GroupVoiceMail:
                return Groups::model();
                break;
            case TargetType::Ivr:
                require_once(__DIR__ . '/../modules/routingIVR/models/IVR.php');
                return IVR::model();
                break;
            case TargetType::Queue:
                return Queue::model();
                break;
            case TargetType::CallGroup:
                return CallGroup::model();
            case TargetType::Conference:
                return Conference::model();
            default:
                return null;
                break;
        }
    }

    public function getTargetsForType($type, $searchAttribut = 'extension')
    {
        $model = $this->getModelForTargetType($type);

        // Invalide or unsupported target type
        if ($model == null)
            return null;

        $modelTargets = $model->find(array(), $searchAttribut);

        // No targets in model found
        if ($modelTargets == null)
            return null;

        $targetList = array();
        foreach ($modelTargets as $target) {


            if ($target->extension !== '') {    // becouse we just need Targets with extension
                // FIXME
                // use $this->getTitlePatternForTargetType() as value...
                $title = $this->getTitleForModelAndTargetType($target, $type);
                if ($type == TargetType::UserVoiceMail || $type == TargetType::GroupVoiceMail) {
                    $targetList[(int) $target->extension][$type . $this->separator . $target->id] = $title . __('VOICE_MAIL_SUFFIX');
                } else {
                    $targetList[(int) $target->extension][$type . $this->separator . $target->id] = $title;
                }
            }
        }

        $targetList = $this->sortTargetList($targetList);
        if (empty($targetList)) {
            return null;
        }

        return $targetList;
    }

    public function sortTargetList($listData)
    {
        $sortedListData = array();
        ksort($listData);
        foreach ($listData as $data) {
            foreach ($data as $key => $value) {

                $sortedListData[$key] = $value;
            }
        }

        return $sortedListData;
    }

    public function getTranslationForTargetType($targetType)
    {
        switch ($targetType) {
            case TargetType::User:
                return __('USERS');
                break;
            case TargetType::UserVoiceMail:
                return __('USER_VOICE_MAILS');
                break;
            case TargetType::Group:
                return __('GROUPS');
                break;
            case TargetType::GroupVoiceMail:
                return __('GROUP_VOICE_MAILS');
                break;
            case TargetType::Ivr:
                return __('IVRS');
                break;
            case TargetType::Queue:
                return __('QUEUES');
                break;
            case TargetType::CallGroup:
                return __('CALL_GROUP');
                break;
            case TargetType::CallThrough:
                return __('CALL_TROUGHT');
                break;
            case TargetType::UserFax:
                return __('USER_FAX');
                break;
            case TargetType::Conference:
                return __('CONFERENCE');
                break;
            case TargetType::Manual:
                return __('MANUAL');
            default:
                return 'n/a';
                break;
        }
    }

    public function getTitleForModelAndTargetType($model, $targetType)
    {
        switch ($targetType) {
            case TargetType::User:
            case TargetType::UserVoiceMail:
                return getFormatedUsername($model);
                break;
            case TargetType::Group:
            case TargetType::GroupVoiceMail:
            case TargetType::CallGroup:
                return '<' . $model->extension . '> ' . $model->name;
                break;
            case TargetType::Ivr:
                return '<' . $model->extension . '> ' . $model->title;
                break;
            case TargetType::Queue:
                return '<' . $model->extension . '> ' . $model->name;
                break;
            default:
                return 'n/a';
                break;
        }
    }

    public function getSelectedValueForAttributes($typeAttribute, $targetAttribute, $model)
    {
        return $model->$typeAttribute . $this->separator . $model->$targetAttribute;
    }

    public static function setTargetAttributes($typeAttribute, $targetAttribute, &$model, $separator = '_')
    {
        $attributes = explode($separator, $model->$targetAttribute);
        $model->$typeAttribute = $attributes[0];
        $model->$targetAttribute = $attributes[1];
    }
}
