<?php

final class CallForwardType {
	const None     = 0x00000000;
	const Simple     = 0x00000001;
	const Advanced     = 0x00000010;
	private function __construct(){
	}

	public static function toMySQLEnum($type) {
		if($type == CallForwardType::None) {
			return "none";
		} else if ($type == CallForwardType::Simple) {
			return "simple";
		} else if ($type == CallForwardType::Advanced) {
			return "advanced";
		} else {
			return "none";
		}
	}

	public static function fromMySQLEnum($type) {
		if($type == "none" ) {
			return CallForwardType::None;
		} else if ($type == "simple") {
			return CallForwardType::Simple;
		} else if ($type == "advanced") {
			return CallForwardType::Advanced;
		} else {
			return CallForwardType::None;
		}
	}

}

class CallForward {

	private $mCriterias = array();
	private $mType = CallForwardType::Simple;
	private $callForwardTarget = null;
	private $mId;

	private $mPosition; //sort

	private $mParent = null;

	public function __construct($parent){
		$this->mParent = $parent;
	}

	public function getPosition() {
		return $this->mPosition;
	}

	public function setPosition($pos) {
		$this->mPosition = $pos;
	}

	public function getId() {
		return $this->mId;
	}

	public function setId($id) {
		$this->mId = $id;
	}

	public function setParent($parent) {
		$this->mParent = $parent;
	}

	public function getParent() {
		$this->mParent;
	}

	public function isBrandNew() {
		if ($this->mId == "") {
			return true;
		} else {
			return false;
		}
	}

	public function getForwardCriterias() {
		//get all Criterias
		return $this->mCriterias;
			
	}

	public function setType($type) {
		$this->mType = $type;
	}

	public function getType() {
		return $this->mType;
	}

	public function setCallForwardTarget(AbstractCallForwardTarget $target) {
		$this->callForwardTarget = $target;
		$this->callForwardTarget->setParent($this);
	}

	public function getCallForwardTarget() {

		if($this->callForwardTarget == null) {
			$this->callForwardTarget = CallForwardingFactory::getCallNewCallForwardTargetByName("number", $this);
		}

		return $this->callForwardTarget;
	}

	public function addCriteria(AbstractCallForwardCriteria $criteria) {
		$criteria->setParent($this);
		$this->mCriterias[] = $criteria;
	}

	public function getCriteriaDescription() {
		$description = "";
		foreach ($this->mCriterias as $criteria) {
			if($description == "") {
				$description = __("Rufumleitung ist Aktiv ");
			} else {
				$description .= __(" und <br>");
			}
			$description .= $criteria->getCriteriaDescription();
		}
		return $description;
	}

	public function getTargetDescription() {
		$target = $this->getCallForwardTarget();
		if ($target == null) {
			return __("Es ist kein Rufumleitungsziel definiert!");
		} else {
			//return $target->getDescribtion();
			return "FIXME (class.callforward.php) Add description!";
		}
	}

	public function getAvialableSystemRecordings() {
		return $this->mParent->getAvialableSystemRecordings();
	}
	
	public function getAvialableAddressBooks() {
		return $this->mParent->getAvialableAddressBooks();
	}
	
	public function getAvialableGroups() {
		return $this->mParent->getAvialableGroups();
	}
		
	public function getAvialableCalendars() {
		return $this->mParent->getAvialableCalendars();
	}
	
		
	public function getDialedExtension() {
		 return $this->mParent->getDialedExtension();
	}

	public function toJSON() {

		$json_Data = array();
		$json_Data['id'] = $this->mId;
		$json_Data['type'] = $this->mType;
		$json_Data['position'] = $this->mPosition;

		$json_Data['target'] = json_decode($this->getCallForwardTarget()->toJSON());

		$json_Data['criterias'] = array();
		$i = 0;
		foreach ($this->mCriterias as $criteria ) {
			++$i;
			$json_Data['criterias'][$i] = json_decode($criteria->toJSON());
		}

		return json_encode($json_Data);
	}
	
	public function doesYourCriteriasNeedFreePhoneStatus() {
		
		foreach ($this->mCriterias as $criteria) {
			if ($criteria->getCriteriaName() == "PhoneStatus") {
				if ($criteria->getProperty("phonestatus") == "noanswer") {
					return true;
				}
			}
		}
		return false;
		
	}
	
	public function checkIfYourCriteriasMatch($variables) {

		$matchedArray = array();
		
		//print_r($this->mCriterias);
		
		foreach ($this->mCriterias as $criteria) {
			$matchedArray[$criteria->getCriteriaName()][] = $criteria->checkIfMatched($variables);
		}
		//check each group of states if one in the group matches (cause criterias of the same group name are handelt as OR, like phonestatus is busy OR offline)
		foreach ($matchedArray as $name =>  $criteriasWithTheSameName) {
				
			$groupMatched = false; //OR Flag

			foreach ($criteriasWithTheSameName as $criteria) {
				if ($criteria) {
					$groupMatched = true;
					break;
				}
			}

			if (!$groupMatched) { //this rule does not match
				return false;
			}
		}

		return true;
	}


	public function getExtraDialplanVars() {
		$ret = array();
		foreach ($this->mCriterias as $criteria) {
			foreach ($criteria->getExtraDialplanVars() as $key => $value) {
				$ret[$key] = $value;
			}
		}

		/*$vars = $this->getCallForwardTarget()->getExtraDialplanVars();
	        foreach ($vars as $key => $value) {
                      $ret[$key] = $value;
                }*/

		
		return $ret;
	}

























}
