<?php
require_once 'class.abstractcallforwardcriteria.php';

class CallForwardCriteriaCalendar extends AbstractCallForwardCriteria {

	public function __construct($parent){
		$this->setCriteriaName("Calendar");
		$this->setCriteriaTitle(__("PHONESETTINGSCALLFORWARD_CALENDAR_STATE_GENERAL"));
		
		$property = new CallForwardCriteriaProperty();
		$property->setPropertyName("is_free_or_not");
		$property->setPropertyTitle("Ist");
		$property->setPropertyTypeName("select");
	
		$cap = new CallForwardCriteriaPropertyCapability();
		$cap->setName("is_free");
		$cap->setTitle(__("PHONESETTINGSCALLFORWARD_ADDRESSBOOK_CALENDAR_USER_IS_FREE"));
		$property->addPropertyCapability($cap);

		$cap = new CallForwardCriteriaPropertyCapability();
		$cap->setName("is_not_free");
		$cap->setTitle(__("PHONESETTINGSCALLFORWARD_ADDRESSBOOK_CALENDAR_USER_IS_NOT_FREE"));
		$property->addPropertyCapability($cap);
		$this->addPropertyType($property);
		
		//Currently there is only one calendar!
		$property = new CallForwardCriteriaProperty();
		$property->setPropertyName("calendar");
		$property->setPropertyTitle("laut Kalender");
		$property->setPropertyTypeName("select");
		
		$calendars = $parent->getAvialableCalendars();
		
		foreach ($calendars as $calendar) {
			$cap = new CallForwardCriteriaPropertyCapability();
			$cap->setName($calendar->id);
			$cap->setTitle($calendar->title);
			$property->addPropertyCapability($cap);
		}

		$this->addPropertyType($property);
		
	}
	
	public function getCriteriaDescription() {
		
		$string = "";
		$isTrueIfIsFree = $this->getProperty("is_free_or_not","is_free");
		if ($isTrueIfIsFree == "is_free") {
			$string =  __("PHONESETTINGSCALLFORWARD_FREE_IN_CALENDAR_GENERAL").": ";
		} else {
			$string = __("PHONESETTINGSCALLFORWARD_NOT_FREE_IN_CALENDAR_GENERAL").": ";
		}
		
		$id = $this->getProperty("calendar");
		$cal = Calendar::model()->findByPk($id);
		if ($cal == NULL) {
			$string .= "?";
		} else {
			$string .= $cal->title;
		}
		
		return CHtml::encode($string);		
	}
	
	public function checkIfMatched($variables) {
		
		$state = @$variables['calenderstatus'];
		if ($state == "") {
			return false;
		}
		
		$isTrueIfIsFree = $this->getProperty("is_free_or_not","is_free");
		
		if ($isTrueIfIsFree == "is_free") {
			$isTrueIfIsFree = true;
		} else {
			$isTrueIfIsFree = false;
		}

		if ($state == "busy") {
			if ($isTrueIfIsFree) {
				return true;
			} else {
				return false;
			}
		} else {
			if ($isTrueIfIsFree) {
				return false;
			} else {
				return true;
			}
		}
				
		return false;
	}
}