<?php
require_once 'class.abstractcallforwardcriteria.php';

final class CallForwardCriteriaQueueStatusType {
	const None         = 0x00000000;
	const MaxCallers   = 0x00000001;
	const NoAgents     = 0x00000010;
	const NoAnswer     = 0x00000100;
	private function __construct(){
	}
	
	public static function toMySQLEnum($type) {
		if($type == CallForwardCriteriaQueueStatusType::MaxCallers) {
			return "maxcallers";
		} else if ($type == CallForwardCriteriaQueueStatusType::NoAgents) {
			return "noagents";
		} else if ($type == CallForwardCriteriaQueueStatusType::NoAnswer) {
			return "noanswer";
		} else {
			return "none";
		}
	}
	
	public static function fromMySQLEnum($type) {
		if($type == "maxcallers" ) {
			return CallForwardCriteriaQueueStatusType::MaxCallers;
		} else if ($type == "noagents") {
			return CallForwardCriteriaQueueStatusType::NoAgents;
		} else if ($type == "noanswer") {
			return CallForwardCriteriaQueueStatusType::NoAnswer;
		} else {
			return CallForwardCriteriaQueueStatusType::None;
		}
	}
	
}

class CallForwardCriteriaQueueStatus extends AbstractCallForwardCriteria {
	
	private $mCallForwardCriteriaPhoneStatusType = CallForwardCriteriaQueueStatusType::MaxCallers;
	private $mCallTimeout = 15;
	
	public function __construct($parent){
		$this->setCriteriaName("QueueStatus");
		$this->setCriteriaTitle(__("Warteschlangenstatus"));
		$property = new CallForwardCriteriaProperty();
		$property->setPropertyName("queuestatus");
		$property->setPropertyTitle(__("Warteschlangenstatus ist: "));
		$property->setPropertyTypeName("select");
		
		$cap = new CallForwardCriteriaPropertyCapability();
		$cap->setName(CallForwardCriteriaQueueStatusType::toMySQLEnum(CallForwardCriteriaQueueStatusType::MaxCallers));
		$cap->setTitle(__("Maximale Anruferzahl erreicht (voll)"));
		$property->addPropertyCapability($cap);
		
		$cap = new CallForwardCriteriaPropertyCapability();
		$cap->setName(CallForwardCriteriaQueueStatusType::toMySQLEnum(CallForwardCriteriaQueueStatusType::NoAgents));
		$cap->setTitle(__("Keine Agenten angemeldet (leer)"));
		$property->addPropertyCapability($cap);
				
		$cap = new CallForwardCriteriaPropertyCapability();
		$cap->setName(CallForwardCriteriaQueueStatusType::toMySQLEnum(CallForwardCriteriaQueueStatusType::NoAnswer));
		$cap->setTitle(__("Keine Antwort nach"));
		$calltimeoutproperty = new CallForwardCriteriaProperty();
		$calltimeoutproperty->setPropertyName("calltimeout");
		$calltimeoutproperty->setPropertyTypeName("numeric");
		$cap->addSubCallForwardCriteriaProperty($calltimeoutproperty);
		$property->addPropertyCapability($cap);
		
		
		
		//$property->addPropertyCapability(CallForwardCriteriaPhoneStatusType::toMySQLEnum(CallForwardCriteriaPhoneStatusType::Busy));
		//$property->addPropertyCapability(CallForwardCriteriaPhoneStatusType::toMySQLEnum(CallForwardCriteriaPhoneStatusType::Offline));
		
		
		
		$this->addPropertyType($property);
		
		
		
	}
	
	public function setMatchedPhoneStatus($type) {
		$this->setProperty("queuestatus", CallForwardCriteriaQueueStatusType::toMySQLEnum($type));
		
		if ($type == CallForwardCriteriaQueueStatusType::NoAnswer && $this->getCallTimeOutForNoAnswer() == "") {
			$this->setCallTimeOutForNoAnswer("15");
		}
	}
		
	public function getMatchedQueueStatus() {
		return CallForwardCriteriaQueueStatusType::fromMySQLEnum($this->getProperty("queuestatus"));
	}
	
	public function setCallTimeOutForNoAnswer($timeout) {
		$this->setProperty("calltimeout", $timeout);
	}
	
	public function getCallTimeOutForNoAnswer() {
		return $this->getProperty("calltimeout");
	}
	
	public function getCriteriaDescription() {
		
		if ($this->isInverse()) {
			return __("Invers.");
		} else {
			$type = $this->getMatchedQueueStatus();
			switch ($type) {
				case CallForwardCriteriaQueueStatusType::MaxCallers :
					return __("Warteschlangenstatus: Maximale Anruferzahl erreicht (voll)"); 
					break;
				case CallForwardCriteriaQueueStatusType::NoAgents :
					return __("Warteschlangenstatus: Keine Agenten angemeldet (leer)");
					break;
				case CallForwardCriteriaQueueStatusType::NoAnswer :
					return __("Warteschlangenstatus: keine Antwort nach ").$this->getCallTimeOutForNoAnswer().__(" Sekunden");
					break;
			}
		}
	}

	public function checkIfMatched($variables) {
		
		//is the phonestatus set?
		$currentState = @$variables['queuestatus'];
		if ($currentState == "") {		
			return false;
		}
		//transform into enum
		$currentState = CallForwardCriteriaQueueStatusType::fromMySQLEnum($currentState);
		
		if ($this->getMatchedQueueStatus() == $currentState) {
			//current state matches.
			return true;
		}
		return false;
	}

	
}