$(document).ready(function () {

// Modal window constructor
PlModalWindow = function(title, content, id, autoDestroy, customStyle, hAlign, vAlign) {
  this.id = id ? id : '' + new Date().getTime();
  this.title = title ? title : '';
  this.content = content ? content : '';
  this.hAlign = hAlign ? hAlign : 'center';
  this.vAlign = vAlign ? vAlign : 'center';
  this.autoDestroy = autoDestroy ? autoDestroy : true;
  this.opacity = 0.7;
  this.customStyle = customStyle ? customStyle : false;
  
  this.yOffset = 10;
  
  this.xPos = 0;
  this.yPos = 0;

  
  this.speed = 500;
  this.hidden = true;
  this.init(id, title, content);
  

}

$.extend(PlModalWindow.prototype, {
   init: function(id, title, content) {	   
	   // create lightBox overlay if not exist
	   if(!$("#lightBoxOverlay").length > 0){
		   $(document.createElement('div')).attr('id','lightBoxOverlay').appendTo('body');
	   }
	   
	   // create new window
	   var windowTemplate = '';
	   if(this.customStyle){
	   	console.log('custom');
		   windowTemplate += '<div class="modalWindowWraper" id="'+this.id+'">'; 
		   windowTemplate += '<div><div style="display:none"><h1>'+this.title+'</h1><a id="close_'+this.id+'" class="modalWindowClose" href="#">&times;</a></div>';
		   windowTemplate += '<div class="modalWindowContent" style="border:none;padding:0">'+this.content+'</div></div></div>';
	   }
	   else{
		   windowTemplate += '<div class="modalWindowWraper" id="'+this.id+'">'; 
		   windowTemplate += '<div class="modalWindow"><div class="modalWindowHeader"><h1>'+this.title+'</h1><a id="close_'+this.id+'" class="modalWindowClose" href="#">&times;</a></div>';
		   windowTemplate += '<div class="modalWindowContent">'+this.content+'</div></div></div>';
	   }
	   $(windowTemplate).appendTo('body');
	   
	   $('#close_'+this.id).click($.proxy(this.close, this));
	   
	   this.setTransitions();
	   
	   
	   $(window).keyup($.proxy(this.observeKeyPress, this));	   
	   
	   $(window).resize($.proxy(this.updateOverlaySize, this));
	   $(window).resize($.proxy(this.updateWindowPosition, this)); 
	   //$(window).scroll($.proxy(this.updateWindowPosition, this));             
           
   },
   
   observeKeyPress: function(e){
   		if(!this.hidden && e.keyCode == 27){
	   		this.close();
   		}
   },
   
   close: function() {
	   this.hide(this.speed);
	   if(this.autoDestroy){
		   this.destroy();
	   }
   },
   
   setTransitions: function() {
   		$('#'+this.id+',#lightBoxOverlay').css({
	   		'-webkit-transition':'opacity '+this.speed+'ms ease-in-out',
	   		'-moz-transition':'opacity '+this.speed+'ms ease-in-out',
	   		'-o-transition':'opacity '+this.speed+'ms ease-in-out',
	   		'transition':'opacity '+this.speed+'ms ease-in-out',
	   	});	   
   },
   
   updateWindowPosition: function(event) {
    // Without proxy, `this` would refer to the event target
    // use event.target to reference that element.

    var el = $('#'+this.id);
    // calculate window Y position
    switch(this.vAlign){
		case 'top': this.yPos = this.yOffset;
		   				 break;
			case 'center': this.yPos = window.innerHeight/2 - $(el).outerHeight()/2;
		   					this.yPos = this.yPos < this.yOffset ? this.yOffset : this.yPos;
		   				 break;
		   	case 'bottom': this.yPos = window.innerHeight - $(el).outerHeight();
		   				 break;
		   	default: this.yPos = 0;
	   	}
	   	
	   	// calculate window X position
	   	switch(this.hAlign){
		   	case 'left': this.xPos = 0;
		   				 break;
		   	case 'center': this.xPos = window.innerWidth/2 - $(el).outerWidth()/2;
		   				 break;
		   	case 'right': this.xPos = window.innerWidth - $(el).outerWidth();
		   				 break;
		   	default: this.xPos = 0;
	   	}
	   	
	   	$(el).css({
	   		'top':this.yPos,
	   		'left':this.xPos
	   	});    
  },
     
   updateOverlaySize: function(){
	   $('#lightBoxOverlay').css({height: '100%'});
   },
   
   
   display: function(speed){
   		// animation speed
	   speed = speed ? speed : this.speed;   

	   this.updateWindowPosition();
	   
	   var win = $('#'+this.id);
	   	$('#lightBoxOverlay').css({display:'block'});
	   	$(win).css({
	   		'display':'block',
	   		'top':this.yPos,
	   		'left':this.xPos
	   	});
	   	
	   	
	   	setTimeout((function(o) {
        	return function() {
            		$(win).css('opacity', 1);
            		$('#lightBoxOverlay').css({opacity:o});
            	}
        })(this.opacity), 1);
	   this.hidden = false;
   },
   
   
   hide: function(speed){
	   // animation speed
	   var win = $('#'+this.id);	   
	   

	   $(win).css({opacity:0});
	   $('#lightBoxOverlay').css({opacity:0});
	   
	   setTimeout(function(){
		   $(win).css({display:'none'});
		   $('#lightBoxOverlay').css({display:'none'});
		}, this.speed);
		this.hidden = true;
	},
	setContent: function(content){
		this.content = content;
		$('#'+this.id+' .modalWindowContent').append(content);
	},
	setTitle: function(title){
		this.title = title;
		$('#'+this.id+' .modalWindowHeader h1').html(title);
	},
   destroy: function(){
   		$('#'+this.id).remove();
   		$(window).unbind('keyup', this.observeKeyPress);
   		$(window).unbind('resize', this.updateOverlaySize);
   		$(window).unbind('resize', this.updateWindowPosition);   		
   		//$('#close_'+this.id+',#lightBoxOverlay').unbind('click', this.close);
	   delete this;
   },
   
});	
});