<?php

class DebugController extends Controller
{

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(
            array('allow', // allow all users to perform actions
                'actions' => array('less', 'test', 'wizardLess', 'uiLess', 'index', 'renderFile', 'oauth', 'login'),
                'expression' => AccessManager::checkDevMode(),
            ),
            array('deny', // deny all users
                'users' => array('*'),
            ),
        );
    }
    
    /**
     * recompile less files
     */
    public function actionLess(){
        $less = Yii::app()->less;
        $basePath = str_replace('\'', '/', Yii::getPathOfAlias('webroot') . '/');
        foreach ($less->paths as $from=>$to) {
            $fromPath = $basePath . $from;
            $toPath = $basePath . $to;
            $less->compile($fromPath, $toPath);
        }
        $this->redirect(Yii::app()->request->urlReferrer);
    }


    public function actionTest(){
        $less = Yii::app()->less;
        $basePath = str_replace('\'', '/', Yii::getPathOfAlias('webroot') . '/');

        $fromPath = $basePath . 'less/colors/red.less';
        $toPath = $basePath . 'css/wizard_red.css';
        $less->compileFile($fromPath, $toPath);
        dump($fromPath);
        dump($toPath);

        $fromPath = $basePath . 'less/colors/default.less';
        $toPath = $basePath . 'css/wizard_default.css';
        $less->compileFile($fromPath, $toPath);
        dump($fromPath);
        dump($toPath);

        $fromPath = $basePath . 'less/colors/green.less';
        $toPath = $basePath . 'css/wizard_green.css';
        $less->compileFile($fromPath, $toPath);
        dump($fromPath);
        dump($toPath);
        /*
foreach ($less->paths as $from=>$to) {
            $fromPath = $basePath . $from;
            $toPath = $basePath . $to;
            dump($fromPath);
            dump($toPath);
            $less->compile($fromPath, $toPath);
        }
*/
    }


    public function actionAllLess(){

    }


    public function actionWizardLess(){
        $basePath = str_replace('\'', '/', Yii::getPathOfAlias('webroot') . '/');
        $files = glob('less/presets/wizard/*.{less}', GLOB_BRACE);

        foreach($files as $file) {
            $path_parts = pathinfo($file);
            $fromPath = $basePath . $file;
            $toPath = $basePath . 'css/wizard_' . $path_parts['filename'].'.css';
            Yii::app()->less->compileFile($file, $toPath);

            // Debug
            dump($fromPath);
            dump($toPath);
        }
    }


    public function actionUiLess(){
        $basePath = str_replace('\'', '/', Yii::getPathOfAlias('webroot') . '/');
        $files = glob('less/presets/theme/*.{less}', GLOB_BRACE);

        foreach($files as $file) {
            $path_parts = pathinfo($file);
            $fromPath = $basePath . $file;
            $toPath = $basePath . 'css/theme_' . $path_parts['filename'].'.css';
            Yii::app()->less->compileFile($file, $toPath);

            // Debug
            dump($fromPath);
            dump($toPath);
        }
    }


    public function actionIndex(){
        $this->render('less');
    }


    public function actionRenderFile(){
        $color = isset($_POST['color']) ? $_POST['color'] : 'default';
        $type = isset($_POST['type']) ? $_POST['type'] : 'wizard';
        $prefix = '';

        $basePath = str_replace('\'', '/', Yii::getPathOfAlias('webroot') . '/');

        $fromPath = $basePath . 'less/presets/'.$type.'/'.$color.'.less';
        $toPath = $basePath . 'css/'.$type.'_' . $color.'.css';

        Yii::app()->less->compileFile($fromPath, $toPath);
    }


    public function actionOauth(){

        try{
            $hybridauth = Yii::app()->hybridauth;
            $twitter = $hybridauth->authenticate( "Twitter" );
            
            // get the user profile 
          	$twitter_user_profile = $twitter->getUserProfile();
          
          	echo "Ohai there! U are connected with: <b>{$twitter->id}</b><br />"; 
          	echo "As: <b>{$twitter_user_profile->displayName}</b><br />"; 
          	echo "And your provider user identifier is: <b>{$twitter_user_profile->identifier}</b><br />"; 
         
          	// debug the user profile
          	print_r( $twitter_user_profile );
         
          	// exp of using the twitter social api: Returns settings for the authenticating user.
          	$account_settings = $twitter->api()->get( 'account/settings.json' );
         
          	// print recived settings 
          	echo "Your account settings on Twitter: " . print_r( $account_settings, true );
         
          	// disconnect the user ONLY form twitter
          	// this will not disconnect the user from others providers if any used nor from your application
          	echo "Logging out.."; 
          	$twitter->logout();
        }


        catch( Exception $e ){
            // Display the recived error,
            // to know more please refer to Exceptions handling section on the userguide
            switch( $e->getCode() ){
            case 0 : echo "Unspecified error."; break;
            case 1 : echo "Hybriauth configuration error."; break;
            case 2 : echo "Provider not properly configured."; break;
            case 3 : echo "Unknown or disabled provider."; break;
            case 4 : echo "Missing provider application credentials."; break;
            case 5 : echo "Authentification failed. "
                    . "The user has canceled the authentication or the provider refused the connection.";
                break;
            case 6 : echo "User profile request failed. Most likely the user is not connected "
                    . "to the provider and he should authenticate again.";
                $twitter->logout();
                break;
            case 7 : echo "User not connected to the provider.";
                $twitter->logout();
                break;
            case 8 : echo "Provider does not support this feature."; break;
            }

            // well, basically your should not display this to the end user, just give him a hint and move on..
            echo "<br /><br /><b>Original error message:</b> " . $e->getMessage();
        }


    }

    public function actionLogin(){
        $this->render('login');
    }
}