<?php

class passwordStrength extends CValidator
{

    public $regexNumber = "#[0-9]+#";
    public $regexUperCase = "#[A-ZÄÜÖ]+#";
    public $regexNoSpecialChar = "/^[a-zA-Z0-9ÄäÖöÜü]*$/";

    protected function validateAttribute($object, $attribute)
    {
        // extract the attribute value from it's model object
        $value = $object->$attribute;

        if (is_subclass_of($object, "PhonalisaRecord") && !$object->isNewRecord()) {
            if (!$this->passwordChanged(get_class($object), $object->id, $value)) {
                return true;
            }
        }

        if ($value == null) {
            return true;
        }

        if ($value === '') {
            return true;
        }

        $itemName = $this->getItemName($object);
        $strength = cfg('PASSWORD_STRENGTH');

        $message = "";
        if ($strength == "NONE") {
            $message = $this->checkPasswordNone($value);
        } else if ($strength == "MODERATE") {
            $message = $this->checkPasswordModerate($value, $itemName);
        } else if ($strength == "STRONG") {
            $message = $this->checkPasswordStrong($value, $itemName);
        } else if ($strength == "PARANOID") {
            $message = $this->checkPasswordParanoid($value, $itemName);
        }

        if ($message != "") {
            $object->addError($attribute, $message);
            return false;
        }

        return true;
    }

    private function passwordChanged($modelName, $id, $value)
    {
        $model = $modelName::model()->findByPk($id);
        if ($model) {
            if ($modelName === "User") {
                return $this->userPasswordChanged($model, $value);
            }
        }
        return true;
    }

    private function userPasswordChanged($user, $value)
    {
        if ($user->password == md5($value . $user->id)) {
            return false;
        }
        return true;
    }

    private function getItemName($object)
    {
        $username = "";
        if (is_subclass_of($object, "PhonalisaRecord")) {
            $username = $object->name; // for User model
        } else {
            $username = $object->userName; // for WizardForm.php
        }
        return $username;
    }

    public function checkPasswordNone($password)
    {
        if (mb_strlen($password, 'UTF-8') < 3) {
            return __('PHONALISA_PASSWORD_SECURITY_EXCEPTION_PASS_TO_SHORT_3');
        }

        return "";
    }

    public function checkPasswordModerate($password, $username)
    {
        if (mb_strlen($password, 'UTF-8') < 6) {
            return __('PHONALISA_PASSWORD_SECURITY_EXCEPTION_PASS_TO_SHORT_6');
        }

        if (!preg_match($this->regexNumber, $password)) {
            return __('PHONALISA_PASSWORD_SECURITY_EXCEPTION_PASS_NO_NUMBER');
        }

        if (!preg_match($this->regexUperCase, $password)) {
            return __('PHONALISA_PASSWORD_SECURITY_EXCEPTION_PASS_NO_UPPERCASE_LETTER');
        }

        if ($username == "") { // no username, skip check.
            return "";
        }

        if ($password == $username) {
            return __('PHONALISA_PASSWORD_SECURITY_EXCEPTION_PASS_IS_USERNAME');
        }

        return "";
    }

    public function checkPasswordStrong($password, $username)
    {
        if (mb_strlen($password, 'UTF-8') < 8) {
            return __('PHONALISA_PASSWORD_SECURITY_EXCEPTION_PASS_TO_SHORT_8');
        }

        if (!preg_match($this->regexNumber, $password)) {
            return __('PHONALISA_PASSWORD_SECURITY_EXCEPTION_PASS_NO_NUMBER');
        }

        if (!preg_match($this->regexUperCase, $password)) {
            return __('PHONALISA_PASSWORD_SECURITY_EXCEPTION_PASS_NO_UPPERCASE_LETTER');
        }

        if (preg_match($this->regexNoSpecialChar, $password)) {
            return __('PHONALISA_PASSWORD_SECURITY_EXCEPTION_PASS_NO_SPECIAL_CHARACTER');
        }

        if ($username == "") { // no username, skip check.
            return "";
        }

        if (strpos($password, $username) !== false) {
            return __('PHONALISA_PASSWORD_SECURITY_EXCEPTION_PASS_CONTAINS_USERNAME');
        }

        return "";
    }

    public function checkPasswordParanoid($password, $username)
    {
        if (mb_strlen($password, 'UTF-8') < 12) {
            return __('PHONALISA_PASSWORD_SECURITY_EXCEPTION_PASS_TO_SHORT_12');
        }

        if (!preg_match($this->regexNumber, $password)) {
            return __('PHONALISA_PASSWORD_SECURITY_EXCEPTION_PASS_NO_NUMBER');
        }

        if (!preg_match($this->regexUperCase, $password)) {
            return __('PHONALISA_PASSWORD_SECURITY_EXCEPTION_PASS_NO_UPPERCASE_LETTER');
        }

        if (preg_match($this->regexNoSpecialChar, $password)) {
            return __('PHONALISA_PASSWORD_SECURITY_EXCEPTION_PASS_NO_SPECIAL_CHARACTER');
        }

        if ($username == "") { // no username, skip check.
            return "";
        }

        if (strpos(strtoupper($password), strtoupper($username)) !== false) {
            return __('PHONALISA_PASSWORD_SECURITY_EXCEPTION_PASS_CONTAINS_USERNAME');
        }

        return "";
    }
}
