<?php

Yii::import('bootstrap.widgets.TbActiveForm');
Yii::import('ext.PlGrid.input.PlInput');

class PlActiveForm extends TbActiveForm {

    // Input classes.
	const INPUT_HORIZONTAL = 'ext.PlGrid.input.PlInputHorizontal';

    public $gridId;
    public $displayProgressBar = false;

    public $enableAjaxValidation = true;
    public $enableClientValidation = true;
    public $type = 'horizontal';
    public $clientOptions = array('validateOnSubmit'=>true);
    public $htmlOptions = array('enctype' => 'multipart/form-data');
    public $autoRefreshGrid = true;
    public $afterSaveScript;
    public $onValidationError;
    public $onSubmitError;
    public $onSubmitSuccess;
    public $renderContextHelp = false;
    public $loadCoreScripts = false;
    
    public function init()
    {
        $this->setupAjaxFormScripts();
        parent::init();
        if ($this->renderContextHelp) {
            echo CHtml::openTag('div', array('class' => 'formWithContextHelp'));
            echo CHtml::openTag('div', array('class' => 'formContent-container'));
        }
        $this->setupSubmitButton();
        if (!$this->loadCoreScripts) { 
            Yii::app()->clientScript->scriptMap=array(
                'jquery.js'=>false,
                'jquery.ba-bbq.js'=>false,
                'bootstrap.js'=>false,
                'jquery.yiigridview.js'=>false
            );
        }
    }

    public function run()
    {
        if ($this->renderContextHelp) {
            $html = CHtml::closeTag('div');
            $contextHelpContent = app()->getController()->renderPartial('application.views.site.contextHelp', array(), true, false);
            $html.= CHtml::tag('div', array('class' => 'contextHelp-container contextHelp-siteBar', 'style' => "display:none"), $contextHelpContent);
            $html.= CHtml::closeTag('div');
            echo $html;
        }
        parent::run();
        if ($this->displayProgressBar) {
            $this->setUpProgressBar();
        }
    }


    private function setUpProgressBar(){
        $this->widget('bootstrap.widgets.TbProgress', array(
                'percent'=>0, // the progress
                'striped'=>true,
                'animated'=>true,
                'htmlOptions' => array(
                    'id'=>$this->gridId . '_Modal_progressBar',
                    'style' => 'margin-bottom:0'
                )
            ));
    }


    private function setupSubmitButton(){
        echo CHtml::submitButton('save', array('id'=>$this->gridId.'_hiddenSaveBtn', 'style'=>'display:none'));
    }


    private function setupAjaxFormScripts(){  
        if($this->gridId){
            if($this->onSubmitError == null) {
                $this->onSubmitError = $this->getOnSumbitErrorScript();
            }
            
            if($this->onSubmitSuccess == null) {
                $this->onSubmitSuccess = $this->getOnSumbitSuccessScript();
            }
            
            if ($this->onValidationError == null) {
                $this->onValidationError = $this->getOnValidationErrorScript();
            }

            
            $refreshGridJs = $this->autoRefreshGrid ? '$.fn.yiiGridView.update("'.$this->gridId.'");' : '';
            $afterSaveJs = isset($this->afterSaveScript) ? $this->afterSaveScript : '';
        
            $aftValidateScript = new CJavaScriptExpression('function sumbitModalForm(form, data, hasError){

                    // allow to post invalid form if deleteRequest
                    var isDeleteRequset = $(form[0]).find("input[name=\'deleteRequest\']").length; 
                    
                    if(!hasError || isDeleteRequset != 0) {
                    var formData = new FormData(form[0]);
                        $.ajax({
                            url: form.attr("action"),  //server script to process data
                            type: "POST",
                            xhr: function() {  // custom xhr
                                myXhr = $.ajaxSettings.xhr();
                                if(myXhr.upload){ // check if upload property exists
                                    myXhr.upload.addEventListener("progress",progressHandlingFunction, false); // for handling the progress of the upload
                                }
                                return myXhr;
                            },
                            //Ajax events
                            beforeSend: function(jqXHR, settings){
                                $("#'.$this->gridId.'_Modal").unbind("keyup");
                            },
                            success: function(data, textStatus, jqXHR){
                                var errorsFound = false;
                                var e;
                                
                                if(data.length != ""){
                                    if(typeof data === "string"){
                                        try{
                                            e = jQuery.parseJSON(data);
                                        }
                                        catch(e)
                                        {
                                            //console.log("not json");
                                        }

                                    }
                                    else{
                                        e = data;
                                    }
                                    
                                    
                                    if(e != undefined && e.errors != undefined){
                                        errorsFound = true;
                                    }
                                }
                                

                                
                                
                                if(errorsFound){                                 
                                        jQuery.each(e.errors, function(key, value) {
                                            jQuery("#"+key+"_em_").show().html(value.toString());
                                            jQuery("#"+key).addClass("clsError");
                                            jQuery("label[for="+key+"]").addClass("clsError");
                                        });
                                        $("#'.$this->gridId.'_Modal_progressBar > div").width(0+"%");
                                        $("#'.$this->gridId.'_Modal_saveBtn").button("reset");


                                    return false;
                                }
                                
                                
                                '.$refreshGridJs.'
                                this.submitSuccess(data, textStatus, jqXHR);
                                '.$afterSaveJs.'
                            },
                            error: '.$this->onSubmitError.',
                            submitSuccess: '.$this->onSubmitSuccess.',
                            //error: errorHandler,
                            // Form data
                            data: formData,
                            //Options to tell JQuery not to process data or worry about content-type
                            cache: false,
                            contentType: false,
                            processData: false
                        });
                    }
                    else{
                        '.$this->onValidationError.'
                    }
                    return false;
            }

            function progressHandlingFunction(e){
                if(e.lengthComputable){
                    var currentProgress = Math.round(e.loaded/(e.total/100));
                    $("#'.$this->gridId.'_Modal_progressBar > div").width(currentProgress+"%");
                }
            }


            ');

            $befValidateScript = new CJavaScriptExpression('function beforeValidateModalForm(form){
                $("#'.$this->gridId.'_Modal_saveBtn").button("loading");
                return true;
            }');
            
            Yii::app()->clientScript->registerScript('beforeValidateModalForm', $befValidateScript, CClientScript::POS_BEGIN);
            $this->clientOptions['beforeValidate'] = 'js:beforeValidateModalForm';
            
            Yii::app()->clientScript->registerScript('sumbitModalForm', $aftValidateScript, CClientScript::POS_BEGIN);
            $this->clientOptions['afterValidate'] = 'js:sumbitModalForm';
        }
    }


    public function colorFieldRow($model, $attribute, $htmlOptions = array()){
		return $this->inputRow(PlInput::TYPE_COLOR, $model, $attribute, null, $htmlOptions);
	}

    public function datePickerRow($model, $attribute, $htmlOptions = array()){
        return $this->inputRow(PlInput::TYPE_DATE, $model, $attribute, null, $htmlOptions);
    }
	
	
	protected function getInputClassName()
	{
		if (isset($this->input))
			return $this->input;
		else
		{
			switch ($this->type)
			{
				case self::TYPE_HORIZONTAL:
					return self::INPUT_HORIZONTAL;
					break;

				case self::TYPE_INLINE:
					return self::INPUT_INLINE;
					break;

				case self::TYPE_SEARCH:
					return self::INPUT_SEARCH;
					break;

				case self::TYPE_VERTICAL:
				default:
					return self::INPUT_VERTICAL;
					break;
			}
		}
	}
	
	
	private function getOnSumbitErrorScript() {
    	return 'function(jqXHR, textStatus, errorThrown){
    	    console.log(jqXHR);
    	    console.log(textStatus);
    	    console.log(errorThrown);
    	    
    	    alert(jqXHR.responseText);
    	    
            $("#'.$this->gridId.'_Modal_progressBar > div").width(0+"%");
            $("#'.$this->gridId.'_Modal_saveBtn").button("reset");
        }';
	}
	
	private function getOnSumbitSuccessScript() {
    	return 'function(data, textStatus, jqXHR){
            var modalWindow = $(form).closest(".modal");
            var gridId = $(modalWindow).prev(".grid-view").attr("id");
            $(modalWindow).modal("hide");

            $("#'.$this->gridId.'_Modal_saveBtn").button("reset");
            $("#'.$this->gridId.'_Modal_deleteBtn").button("reset");
        }';
	}
        
        private function getOnValidationErrorScript()
        {
            return '$("#'.$this->gridId.'_Modal_saveBtn").button("reset");';
        }

}
