<?php

class CDRType {

    const CDRTypeSuccess = 0; // green  
    const CDRTypeNoAnswer = 1; // grey 
    const CDRTypeForwarded = 2; // blue
    const CDRTypeFailure = 3; // red 
}

class CDR extends PhonalisaRecord {

    public  $id;
    public $source;
    public $target;
    public $previous_cdr_id;
    public $start_timestamp;
    public $answered_timestamp;
    public $duration;
    public $did;
    public $type;
    public $forwarded_target;
    public $error_code;
    public $logmessages;
    public $meta;

    public static function myAttributes() {
    
        return array(
            'start_timestamp' => array('key' => 'start_timestamp', 'type' => 'dateTime'),
            'source'=>'source',
            'target' => 'target',
            'forwarded_target' => 'forwarded_target',       // target => forwarded_target
            'type'=> 'type',
            'duration'=> 'duration',
            'previous_cdr_id' => 'previous_cdr_id',         // not render 
            'answered_timestamp' => 'answered_timestamp',   // not render
            'did' => 'did',
            'error_code'=>'error_code',                     // popover with error text
            'logmessages' =>'logmessages',
            'meta' => 'meta'
        );
    }
    
    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return Calendar the static model class
     */
    public static function model($className=__CLASS__)
    {
        return parent::model($className);
    }
    
    /**
     * @return string the associated database table name
     */
    public function tableName()
    {
        return strtolower('CDR');
    }
    
    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
            array(
                'id,
                source,
                target,
                previous_cdr_id,
                start_timestamp,
                answered_timestamp,
                duration,
                did,
                type,
                forwarded_target,
                error_code,
                logmessages',
                
                'save'
            )
        );
    }
    
    /**
     * @return array relational rules.
     */
    public function relations()
    {
        // NOTE: you may need to adjust the relation name and the related
        // class name for the relations automatically generated below.
        return array(
        );
    }
    
    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'id' => __('CDR_MODEL_ID'),
             'source'=> __('CDR_MODEL_SOURCE'),
            'target' => __('CDR_MODEL_TARGET'),
            'previous_cdr_id' => __('CDR_MODEL_PREVIOUS_CDR_ID'),
            'start_timestamp' => __('CDR_MODEL_START_TIMESTAMP'),
            'answered_timestamp' => __('CDR_MODEL_ANSWERED_TIMESTAMP'),
            'duration'=> __('CDR_MODEL_DURATION'),
            'did' => __('CDR_MODEL_DID'),
            'type'=> __('CDR_MODEL_TYPE'),
            'forwarded_target' => __('CDR_MODEL_FORWARD_TARGET'),
            'error_code'=>__('CDR_MODEL_ERROR_CODE'),
            'logmessages' => __('CDR_MODEL_LOGMESSAGES'),
            'meta' => __('CDR_MODEL_META')
        );
    }

    public function typeToString($type = null)
    {
        if (!isset($type)) {
            $type = $this->type;
        }

        switch ($type) {
            case CDRType::CDRTypeSuccess :
                return __('CDR_TYPE_SUCCESS');
                break;
            case CDRType::CDRTypeNoAnswer :
                return __('CDR_TYPE_NO_ANSWER');
                break;
            case CDRType::CDRTypeForwarded :
                return __('CDR_TYPE_FORWARDED');
                break;
            case CDRType::CDRTypeFailure :
                return __('CDR_TYPE_FAILURE');
                break;
        }
    }

    public function getLabelType() {
        switch ($this->type) {
            case CDRType::CDRTypeSuccess :
                return 'success';
            break;
            case CDRType::CDRTypeNoAnswer :
                return '';
            break;
            case CDRType::CDRTypeForwarded :
                return 'info';
            break;
            case CDRType::CDRTypeFailure :
                return 'important';
            break;
        }
    }
    
    public function humanReadableTypeSearchFilter()
    {
        return array(
            CDRType::CDRTypeSuccess => $this->typeToString(CDRType::CDRTypeSuccess),
            CDRType::CDRTypeNoAnswer => $this->typeToString(CDRType::CDRTypeNoAnswer),
            CDRType::CDRTypeForwarded => $this->typeToString(CDRType::CDRTypeForwarded),
            CDRType::CDRTypeFailure => $this->typeToString(CDRType::CDRTypeFailure)
        );
    }

    /**
     * Retrieves a list of models based on the current search/filter conditions.
     * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
     */
    public function search($order = 'start_timestamp DESC')
    {
        // Warning: Please modify the following code to remove attributes that
        // should not be searched.
        $criteria = new CDbCriteria;
        $filter = $this->getFilter($criteria, $order);

        $filter['pagination'] = array(
            'pageSize' => getGridPageSize('cdr-grid'),
        );

        if ($filter != null) {
            return new CPhonalisaDataProvider($this, array(
                'criteria' => $criteria,
                'pagination' => array(
                    'pageSize' => getGridPageSize('cdr-grid'),
                )
            ));
        }
    }
}
