<?php

class CalendarEvent extends PhonalisaRecord
{

    public $id;
    public $owner;
    public $start;
    public $duration = '';
    public $end;
    public $members;
    public $title = '';
    public $url = '';
    public $location;
    public $type;
    public $calendar;
    public $busy;
    public $description = '';
    public $allDay;
    public $customType;
    public $created;
    public $changed;

    public static function myAttributes()
    {
        return array(
            'id' => 'id',
            'owner' => 'owner',
            'start' => 'start',
            'end' => 'end',
            'duration' => 'duration',
            'members' => 'members',
            'title' => 'title',
            'url' => 'url',
            'location' => 'location',
            'type' => 'type',
            'calendar' => 'calendar',
            'busy' => 'busy',
            'description' => 'description',
            'allDay' => 'allDay',
            'created' => 'created',
            'changed' => 'changed'
        );
    }

    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return Calendar the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * @return string the associated database table name
     */
    public function tableName()
    {
        return 'calendarEvents';
    }

    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
            array('title, calendar', 'required'),
            array('type', 'reuiredIfCustomTypeIsEmpty'),
            array('customType', 'reuiredIfTypeIsEmpty'),
            //array('allDay', 'boolean', 'default'=> false),
            //     array('type','length','min'=>1),
            array('id, start, owner, allDay, busy, duration, end, title, url, members, calendar, location, description, created, changed', 'safe'),
            // The following rule is used by search().
            // Please remove those attributes that should not be searched.
            array('id, start, duration, end, title, location, url, members, description', 'safe', 'on' => 'search'),
        );
    }

    public function reuiredIfTypeIsEmpty($attribute, $params)
    {
        if ($this->type == '') {
            $validator = CValidator::createValidator('required', $this, $attribute, $params);
            $validator->validate($this);
        }
        return true;
    }

    public function reuiredIfCustomTypeIsEmpty($attribute, $params)
    {
        if ($this->customType == '') {
            $validator = CValidator::createValidator('required', $this, $attribute, $params);
            $validator->validate($this);
        }
        return true;
    }

    /**
     * @return array relational rules.
     */
    public function relations()
    {
        // NOTE: you may need to adjust the relation name and the related
        // class name for the relations automatically generated below.
        return array();
    }

    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'id' => __('CALENDER_EVENTS_MODEL_ID'),
            'start' => __('CALENDER_EVENTS_MODEL_START'),
            'end' => __('CALENDER_EVENTS_MODEL_END'),
            'duration' => __('CALENDER_EVENTS_MODEL_DURATION'),
            'members' => __('CALENDER_EVENTS_MODEL_MEMBERS'),
            'title' => __('CALENDER_EVENTS_MODEL_TITLE'),
            'url' => __('CALENDER_EVENTS_MODEL_URL'),
            'location' => __('CALENDER_EVENTS_MODEL_LOCATION'),
            'type' => __('CALENDER_EVENTS_MODEL_TYPE'),
            'customType' => __('CALENDER_EVENTS_MODEL_TYPE'),
            'calendar' => __('CALENDER_EVENTS_MODEL_CALENDAR'),
            'busy' => __('CALENDER_EVENTS_MODEL_BUSY'),
            'description' => __('CALENDER_EVENTS_MODEL_DESCRIPTION'),
            'allDay' => __('CALENDER_EVENTS_MODEL_ALL_DAY'),
            'owner' => __('CALENDER_EVENTS_MODEL_OWNER'),
        );
    }

    /**
     * Retrieves a list of models based on the current search/filter conditions.
     * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
     */
    public function search()
    {
        // Warning: Please modify the following code to remove attributes that
        // should not be searched.

        $criteria = new CDbCriteria();

        $criteria->compare('id', $this->id);
        return new CPhonalisaDataProvider($this, array(
            'criteria' => $criteria,
        ));
    }

    public function convertDateToServerFormat()
    {
        $pattern = PhonalisaDate::userDatePattern(true) . " " . PhonalisaDate::userTimePattern(false, true);

        $time = $this->start;
        $this->start = PhonalisaDate::newServerDate($time, $pattern);

        $time = $this->end;
        $this->end = PhonalisaDate::newServerDate($time, $pattern);
    }

    public function convertDateToUserFormat()
    {
        $format = PhonalisaDate::userDatePattern(true) . ' ' . PhonalisaDate::userTimePattern(false, true);
        $this->start = PhonalisaDate::convertServerDateTo($format, $this->start, true);
        $this->end = PhonalisaDate::convertServerDateTo($format, $this->end, true);
    }

    public function formatJsonEvent()
    {
        $eventData = array();
        $eventData = $this->attributes;
        $eventData['title'] = CHtml::encode($this->title);
        $eventData['location'] = CHtml::encode($this->location);
        $eventData['url'] = false;
        $eventData['allDay'] = ($this->allDay == '1' || $this->allDay == 'true') ? true : false;
        $eventData['busy'] = $this->busy == '1' ? true : false;
        return $eventData;
    }

    public function formatJsonEventWithColor()
    {
        $eventData = $this->formatJsonEvent();

        $calendarModel = Calendar::model()->findByPk($this->calendar);
        $eventData['color'] = $calendarModel->color;

        $user = User::model()->findByPk(user()->sudoUser);
        if ($user) {
            $eventData['editable'] = $user->hasPermission($calendarModel->id, 'write_calendar');
        } else {
            throw new CHttpException(403, 'No permissions to update this resource');
        }
        return $eventData;
    }

    public function isWriteable()
    {
        $user = User::model()->findByPk(user()->sudoUser);
        if ($user) {
            return $user->hasPermission($this->calendar, 'write_calendar');
        }
        return false;
    }

    public function beforeSave()
    {
        if (!isset($this->id) || empty($this->created)) {
            $this->created = PhonalisaDate::newServerTimestamp();
        }

        if (empty($this->owner)) {
            $this->owner = Yii::app()->user->sudoUser;
        }

        $this->changed = PhonalisaDate::newServerTimestamp();

        return parent::beforeSave();
    }

    public function afterSave()
    {
        if (($this->scenario != 'iCloud')) {
            $iCloudData = iCloudData::model()->findByAttributes(array('fileId' => $this->id));
            if ($iCloudData && $iCloudData->directoryId != $this->calendar) {
                $this->deleteDuplicatedICloudEvent($iCloudData);
            }
            $this->updateICloudContactLists();
        }
        parent::afterSave();
    }

    private function deleteDuplicatedICloudEvent($iCloudData)
    {
        $iCloudContactsLists = iCloudContactsList::model()->findAllByAttributes(array('plDirectoryId' => $iCloudData->directoryId));

        if (!empty($iCloudContactsLists)) {
            foreach ($iCloudContactsLists as $iCloudContactsList) {
                $iCloudContactsList->deleteShiftedEventInOtherDirectory($iCloudData, $this->id);
            }
        }

        $iCloudData->directoryId = $this->calendar;
        $iCloudData->save();
    }

    private function updateICloudContactLists()
    {
        $iCloudContactsLists = iCloudContactsList::model()->findAllByAttributes(array('plDirectoryId' => $this->calendar));

        if (!empty($iCloudContactsLists)) {
            foreach ($iCloudContactsLists as $iCloudContactsList) {
                $iCloudContactsList->updateICloudContactsList('uploadList', $this->id);
            }
        }
    }

    public function afterDelete()
    {
        $eventTypeOwner = CalendarEvent::model()->findAllByAttributes(array('type' => $this->type), 2);
        $eventTypeOwner = count($eventTypeOwner);
        if ($eventTypeOwner == 0) {
            $typeToDelete = EventType::model()->findByPk($this->type);
            if ($typeToDelete) {
                $typeToDelete->delete();
            }
        }

        $this->deleteICloudData();

        return parent::afterDelete();
    }

    private function deleteICloudData()
    {
        $iCloudData = iCloudData::model()->findByAttributes(array('fileId' => $this->id));

        if (!empty($iCloudData)) {
            $iCloudData->delete();
        }
    }
}
