<?php
class Conference extends PhonalisaRecord
{
    public $id;
    public $name;
    public $extension;    
    public $pin;
    public $announce_file;
    public $listen_to_music = true; // bool
    public $mohId;
    public $useConfereceMemberName;
    public $scriptExtension;
    

    public static function myAttributes() {

        return array( 
            'name'=>'name',
            'extension'=>'extension',
            'announce_file'=>'announce_file',
            'pin'=>'pin',
            'useConfereceMemberName'=>'useConfereceMemberName',
            'listen_to_music'=>'listen_to_music',
            'mohId'=>'mohId',
            'scriptExtension' => 'scriptExtension'
        );
    }
	/**
	 * Returns the static model of the specified AR class.
	 * @param string $className active record class name.
	 * @return RoutingInbound the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'conference';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
            array('name,extension','required'),
			array('pin ,useConfereceMemberName , extension', 'numerical', 'integerOnly'=>true),
            array('extension', 'length', 'min'=>2, 'max'=>10),
            array('extension','ext.PhonalisaValidators.uniqueExtension'),
            array('mohId, announce_file, listen_to_music, scriptExtension','safe'), // FIXME
            array('name, extension, pin', 'safe','on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
            'name'=>__('CONFERENCE_MODEL_NAME'),
            'extension'=>__('CONFERENCE_MODEL_EXTENSION'),
            'pin'=>__('CONFERENCE_MODEL_PIN'),
            'announce_file'=>__('CONFERENCE_MODEL_ANNOUNCE_FILE'),
            'listen_to_music'=>__('CONFERENCE_MODEL_LISTEN_TO_MUSIC'),
            'mohId'=>__('CONFERENCE_MODEL_MOH_ID'),
            'useConfereceMemberName'=>__('CONFERENCE_MODEL_USECONFERECEMEMBERNAME'),
            'scriptExtension' => __('USER_MODEL_SCRIPT_EXTENSION')
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * @return CPhonalisaDataProvider the data provider that can return the models based on the search/filter conditions.
	 */

	public function search($order = 'name') {
        $criteria=new CDbCriteria;
        $filter = $this->getFilter($criteria, $order);
        $filter['pagination'] = array(
            'pageSize'=> getGridPageSize('conference-grid'),
        );
        
        if ($filter != null) {
            $dataprovider = new CPhonalisaDataProvider($this, $filter);
            $routingConferenceMax = PhonalisaLicenseManager::routingConferenceMax();
            if ($routingConferenceMax >=0) {
                $dataprovider->pagination->pageSize = $dataprovider->pagination->pageSize > $routingConferenceMax ? $routingConferenceMax : $dataprovider->pagination->pageSize;
                $dataprovider->totalItemCount = $dataprovider->totalItemCount > $routingConferenceMax ? $routingConferenceMax : $dataprovider->totalItemCount;
            }
            
            return $dataprovider;
        }
	}

    public function getAnnounceFileListData() {

        $listData = array();
        
        
        $audiofiles = AudioFile::model()->findAllByAttributes(array('type'=>AudioFileType::systemType));
        if (!empty($audiofiles)) {
            foreach ($audiofiles as $audiofile) {
                if (AudioFileUseCase::compare($audiofile->usecase,AudioFileUseCase::CONFERENCE)) {
            
                    $listData[$audiofile->id] = $audiofile->name;
                }
            }
        }
        asort($listData);
        $listData =array(''=> __('CONFERENCE_MODEL_SELECT_ANNOUNCE_FILE_STANDARD')) + $listData;
        return $listData;
    }

    public function getMohData() {
        $listData = array();
        $playlists = MusicOnHold::model()->find(array(),'name');
        $listData= CHtml::listData($playlists,'id','name');
        $ldata['default'] = __("DEFAULT_MOH_PLAYLIST_NAME");
        $listData = array_merge($ldata,$listData);
        //FIXME ! add standard MOH !
        //dump($listData);
        return $listData;
    }
    
    public function afterDelete() {
        $oldModeratorMOdels = ConferenceModerator::model()->findAllByAttributes(array('conferenceId'=>$this->id));
        foreach ($oldModeratorMOdels as $model ) {
            $model->delete();
        }
        return parent::afterDelete();
    }
}