<?php

/**
 * This is the model class for table "addressbook".
 *
 * The followings are the available columns in table 'addressbook':
 * @property integer $id
 * @property string $title
 * @property integer $type
 */
class DateTimeRuleType {
    const dateRule=1;
    const weekRule=2;
}
class DateTimeRule extends PhonalisaRecord
{

//     public  $id;
    public  $parentId;
    public  $type;
    public  $date_from;
    public  $date_to;
    public  $time_from;
    public  $time_to;
    public  $mon;
    public  $tue;
    public  $wed;
    public  $thur;
    public  $fri;
    public  $sat;
    public  $sun;
    public  $sort;
    public $daylightSavingTime;

    public static function myAttributes() {

        return array(
//                     'id'=>'id',
                    'parentId'=>'parentId',
                    'type'=>'type',
                    'date_from'=>'date_from',
                    'date_to'=>'date_to',
                    'time_from'=>'time_from',
                    'time_to'=>'time_to',
                    'mon'=>'mon',
                    'tue'=>'tue',
                    'wed'=>'wed',
                    'thur'=>'thur',
                    'fri'=>'fri',
                    'sat'=>'sat',
                    'sun'=>'sun',
                    'sort'=>'sort',
                    'daylightSavingTime' => 'daylightSavingTime'
                );
    }
    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return Addressbook the static model class
     */
    public static function model($className=__CLASS__)
    {
            return parent::model($className);
    }

    /**
     * @return string the associated database table name
     */
    public function tableName()
    {
            return 'dateTimeProfile';
    }



    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
            // NOTE: you should only define rules for those attributes that
            // will receive user inputs.
            return array(
                    array('type, parentId', 'required'),
                    array('type, sort', 'numerical', 'integerOnly'=>true),
                    array('id, type, date_from, date_to,time_to,time_from mon, tue, wed, thur, fri, sat, sun, daylightSavingTime', 'safe'),
                    // The following rule is used by search().
                    // Please remove those attributes that should not be searched.
                    array('type, date_from, date_to, mon, tue, wed, thur, fri, sat, sun, daylightSavingTime', 'safe', 'on'=>'search'),
            );
    }

    /**
     * @return array relational rules.
     */
    public function relations()
    {

            // NOTE: you may need to adjust the relation name and the related
            // class name for the relations automatically generated below.
            return array(
            );
    }

    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
            return array(
                'parentId'=>__('DATE_TIME_RULE_MODEL_PARENT_ID'),
                'date_from'=>__('DATE_TIME_RULE_MODEL_DATE_FROM'),
                'date_to'=>__('DATE_TIME_RULE_MODEL_DATE_TO'),
                'time_from'=>__('DATE_TIME_RULE_MODEL_TIME_FROM'),
                'time_to'=>__('DATE_TIME_RULE_MODEL_TIME_TO'),
                'mon'=>__('DATE_TIME_RULE_MODEL_MON'),
                'tue'=>__('DATE_TIME_RULE_MODEL_TUE'),
                'wed'=>__('DATE_TIME_RULE_MODEL_WED'),
                'thur'=>__('DATE_TIME_RULE_MODEL_THUR'),
                'fri'=>__('DATE_TIME_RULE_MODEL_FRI'),
                'sat'=>__('DATE_TIME_RULE_MODEL_SAT'),
                'sun'=>__('DATE_TIME_RULE_MODEL_SUN'),
                'title' => __('DATE_TIME_RULE_MODEL_TITLE'),
                'type' => __('DATE_TIME_RULE_MODEL_TYPE'),
            );
    }

    /**
     * Retrieves a list of models based on the current search/filter conditions.
     * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
     */
    public function search($type,$parentId,$order = 'title' )
    {        
        $models = $this->getRuleModelsForParentByType($parentId, $type);
        
        //default values for a new DateTimeRule:
        if (empty($models)) {
            $models[] = $this->getDefaultModel($type);
        }
                
        $dataProvider=new CArrayDataProvider($models, array(
                'id'=>'key',
                'pagination'=>false,
        ));


        return $dataProvider;

    }

    private function getRuleModelsForParentByType($parentId, $type) {
        $sortBy = $this->getOrderByAttribute($type);
        $models = $this->find(array('type'=>$type, 'parentId'=>$parentId),$sortBy);
        return $models;
    }
    
    private function getOrderByAttribute($type) {
        if ($type == DateTimeRuleType::weekRule) {
            $orderByAttribute = 'sort';
        } else {
            $orderByAttribute  = 'date_from';
        }
        return $orderByAttribute ;
    }
    
    private function getDefaultModel($type) {
        
        $defaultModel = new DateTimeRule();
        $defaultModel->id= 'new';
        $defaultModel->type= $type;
        return $defaultModel;
    }
    
    public function beforeSave() {
        if (trim($this->date_from)) {
            $this->toServerTime("date_from");
        }

        if (trim($this->date_to)) {
            $this->toServerTime("date_to");
        }

        if (!empty($this->time_from)) {
            if (cfg('USER_TIME_FORMAT') == '12') {
                $this->time_from   = date('H:i', strtotime($this->time_from));
            }
            $this->toServerTime("time_from");
        }
        
        if (!empty($this->time_to))
        {   
            if (cfg('USER_TIME_FORMAT') == '12') {
                $this->time_to   = date('H:i', strtotime($this->time_to));
            }
            $this->toServerTime("time_to");
        }
        $this->daylightSavingTime = PhonalisaDate::convertServerDateTo('I', PhonalisaDate::newServerDate(), true);
        return parent::beforeSave();
    }
    
    private function toServerTime($attribute)
    {
        if (substr($attribute, 0, 4) === "date") {
            $pattern = PhonalisaDate::userDatePattern() . " " . PhonalisaDate::userTimePattern();
            $this->$attribute = PhonalisaDate::newServerDate($this->$attribute, $pattern);
        } else if (substr($attribute, 0, 4) === "time") {
            $currentDate = PhonalisaDate::convertServerDateTo("Y-m-d", PhonalisaDate::newServerDate());
            $date = PhonalisaDate::newServerDate($currentDate . " " . $this->$attribute . ":00");
            $this->$attribute = PhonalisaDate::convertServerDateTo("H:i", $date);
        }
    }
}