<?php

class FirewallBannedHost extends PhonalisaRecord
{

    public $ip;
    public $datetime;
    public $reason;

    const UNKNOWN = '0';
    const TO_MANY_FAILED_SIP_REGISTER_ATTEMPS = '1';

    public static function myAttributes()
    {
        return array(
            'ip' => 'ip',
            'datetime' => 'datetime',
            'reason' => 'reason'
        );
    }

    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return Job the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
            // The following rule is used by search().
            // Please remove those attributes that should not be searched.
            array('ip, datetime, reason', 'safe'),
        );
    }

    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'ip' => __('NETWORK_FIREWALL_BANNED_HOST_MODEL_IP'),
            'datetime' => __('NETWORK_FIREWALL_BANNED_HOST_MODEL_DATETIME'),
            'reason' => __('NETWORK_FIREWALL_BANNED_HOST_MODEL_REASON')
        );
    }

    /**
     * Retrieves a list of models based on the current search/filter conditions.
     * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
     */
    public function search($order = 'ip')
    {
        // Warning: Please modify the following code to remove attributes that
        // should not be searched.

        $criteria = new CDbCriteria;
        $filter = $this->getFilter($criteria, $order);

        $filter['pagination'] = array(
            'pageSize' => getGridPageSize('banned-host-grid'),
        );

        if ($filter != null) {
            return new CPhonalisaDataProvider($this, $filter);
        }
    }

    public function beforeDelete()
    {
        $data = array('ip' => $this->ip);
        $result = PhonalisaCtrldConnector::firewallUnblockIP($data);

        if (!isset($result->status)) {
            app()->user->getFlashes(); // remove flash-messages
            app()->user->setErrorFlashMessage();
            return false;
        }
        if ($result->status == 'error') {
            app()->user->getFlashes(); // remove flash-messages
            app()->user->setErrorFlashMessage(@$result->errorText);
            return false;
        }

        $this->deleteBannedHost();
        return parent::beforeDelete();
    }

    protected function deleteBannedHost()
    {
        $model = BannedHost::model()->findByAttributes(array('host' => $this->ip));
        if ($model) {
            Log::w('Remove "' . $model->host . '" from bannedHost-List.', 'PhonalisaFirewall');
            $model->delete();
        }
    }
}
