<?php

/**
 * LoginForm class.
 * LoginForm is the data structure for keeping
 * user login form data. It is used by the 'login' action of 'SiteController'.
 */
class LoginForm extends CFormModel
{
	public $username;
	public $password;
	public $rememberMe = true;

	private $_identity;

	/**
	 * Declares the validation rules.
	 * The rules state that username and password are required,
	 * and password needs to be authenticated.
	 */
	public function rules()
	{
		return array(
			// username and password are required
			array('username, password', 'required'),
			// rememberMe needs to be a boolean
			array('rememberMe', 'boolean'),
			// password needs to be authenticated
			array('password', 'authenticate'),
		);
	}

	/**
	 * Declares attribute labels.
	 */
	public function attributeLabels()
	{
		return array(
			'rememberMe'=>__('LOGIN_REMEMBER_ME'),
                        'username'=>__("LOGIN_USERNAME"),
                        'password'=>__("LOGIN_PASSWORD")
		);
	}

	/**
	 * Authenticates the password.
	 * This is the 'authenticate' validator as declared in rules().
	 */
	public function authenticate($attribute,$params)
	{
		if(!$this->hasErrors())
		{
			$this->_identity = new UserIdentity($this->username,$this->password);
			if(!$this->_identity->authenticate())
				$this->addError('password',__('Incorrect username or password!'));
		}
	}

	/**
	 * Logs in the user using the given username and password in the model.
	 * @return boolean whether login is successful
	 */
	public function login()
	{
		if($this->_identity===null)
		{
			$this->_identity=new UserIdentity($this->username,$this->password);
			$this->_identity->authenticate();
		}
		if($this->_identity->errorCode===UserIdentity::ERROR_NONE)
		{
			$duration = $this->rememberMe ? 3600*24*30 : 0; // 30 days
			
			#$duration = 3600*24*30; // 30 days
			Yii::app()->user->login($this->_identity, $duration);
			return true;
		}
		else
			return false;
	}
    
//     public function swichUser(){
//     	
//         if($this->_identity===null)
//         {
//             $this->_identity=new UserIdentity($this->username,$this->password);
//             
//             $this->_identity->authenticate();
//             
//         }
//         if($this->_identity->errorCode===UserIdentity::ERROR_PASSWORD_INVALID)
//         {
//             $duration=$this->rememberMe ? 3600*24*30 : 0; // 30 days
//             $this->_identity->errorCode = 0;
//             
//             Yii::app()->user->login($this->_identity,$duration);            
//             //print_r(Yii::app()->user);
//             
//             return true;
//         }
//         else
//             return false;
//     }
}
