<?php

class NetworkSettingsValueType
{

    const text = 'textField';
    const bool = 'Checkbox';

}

class NetworkSettings extends PhonalisaRecord
{

    public $label;
    public $value;
    public $key;
    public $type;
    private $m_ConfigSections = NULL;
    private $m_ConfigSectionsHumanReadableNames = NULL;
    private $m_LastConfigSection = NULL;
    private $m_DefaultValues = NULL;

    public static function myAttributes()
    {
        return array(
            'label' => 'label',
            'value' => 'value',
            'key' => 'key',
            'type' => 'type'
        );
    }

    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return Job the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * @return string the associated database table name
     */
    public function tableName()
    {
        return 'networkSettings';
    }

    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
            // The following rule is used by search().
            // Please remove those attributes that should not be searched.
            array(
                'label,'
                . 'value,'
                . 'key,'
                . 'type',
                'safe'
            ),
        );
    }

    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'label' => __('NETWORK_ADAPTER_SETTINGS_LABEL'),
            'value' => __('NETWORK_ADAPTER_SETTINGS_VALUE'),
            'key' => __('NETWORK_ADAPTER_SETTINGS_KEY'),
            'type' => __('NETWORK_ADAPTER_SETTINGS_TYPE')
        );
    }

    public function getConfigItemsArray()
    {
        if ($this->m_ConfigSections == NULL) {
            $this->buildConfigSections();
        }
        return $this->m_ConfigSections;
    }

    private function buildConfigSections()
    {
        $this->m_ConfigSections = array();
        $this->m_ConfigSectionsHumanReadableNames = array();

        $this->addSectionDHCP();
    }

    public static function getConfigOptionsArray()
    {
        return array(
            'ENABLE_DCHPD' => NetworkSettingsValueType::bool,
            'GLOBAL_SUBNET_MASK' => NetworkSettingsValueType::text,
            'GLOBAL_SUBNET' => NetworkSettingsValueType::text,
            'GLOBAL_BROADCAST_ADDRESS' => NetworkSettingsValueType::text,
            'GLOBAL_ROUTER' => NetworkSettingsValueType::text,
            'GLOBAL_DNS' => NetworkSettingsValueType::text,
            'GLOBAL_NTP' => NetworkSettingsValueType::text,
            'RANGE_FROM' => NetworkSettingsValueType::text,
            'RANGE_TO' => NetworkSettingsValueType::text
        );
    }

    private function addSectionDHCP()
    {
        $this->addConfigSection("DHCP", __("NETWORK_ADAPTER_SETTINGS_SECTION_TITLE_DHCP"));
        $configOptions = NetworkSettings::getConfigOptionsArray();

        foreach ($configOptions as $key => $valueType) {
            $this->addConfigOption($key, $this->getDefaultValue($key), $valueType);
        }
    }

    private function addConfigSection($sectionName, $humanName)
    {
        $this->m_ConfigSectionsHumanReadableNames[$sectionName] = $humanName;
        $this->m_ConfigSections[$sectionName] = array();
        $this->m_LastConfigSection = $sectionName;
    }

    private function addConfigOption($key, $defaultValue, $type)
    {
        if ($this->m_LastConfigSection == NULL) {
            Log::e("Last networkSettings section not set!", "phonalisa-webinterface-config");
            return;
        }

        $section = $this->m_LastConfigSection;
        $devModeKey = '';

        if (Yii::app()->params->devMode) {
            $devModeKey = 'KEY: ' . $key;
        }

        $label = "NETWORK_SETTINGS_CONFIG_KEY_" . strtoupper($key);

        $this->m_DefaultValues[$key] = $defaultValue;
        $this->m_ConfigSections[$section][$key] = array(
            "key" => $key,
            "label" => '<section data-translation-id="' . $label . '">' . __($label) . '</section>' . $devModeKey,
            "type" => $type,
        );
    }

    public function getSectionItems($section)
    {
        $configSections = $this->getConfigItemsArray();
        $configs = $configSections[$section];
        $arrayDataProviderConfigs = array();

        foreach ($configs as $config) {
            $model = new NetworkSettings();
            $model->key = $config["key"];
            $model->label = $config["label"];
            $model->type = $config["type"];
            $model->value = NetworkSettings::getValueByKey($model->key);

            $arrayDataProviderConfigs[] = $model;
        }

        $dataProvider = new CArrayDataProvider($arrayDataProviderConfigs, array(
            'id' => 'key',
            'pagination' => false,
            'sort' => array('attributes' => array('key'),
            ),
        ));
        return $dataProvider;
    }

    public function getNetworkAdapters()
    {
        $model = NetworkInterface::model();
        return $model->search();
    }

    public function getConfigSectionHumanReadableName()
    {
        if ($this->m_ConfigSectionsHumanReadableNames == NULL) {
            $this->buildConfigSections();
        }
        return $this->m_ConfigSectionsHumanReadableNames;
    }

    public static function getValueByKey($key)
    {
        $model = NetworkSettings::model()->findByAttributes(array('key' => $key));
        $result = '';
        if ($model) {
            $result = $model->value;
            if ($result == '') {
                $result = $model->getDefaultValue($key);
            }
        }
        return $result;
    }

    private function getDefaultValue($key)
    {
        $defaultValue = $this->getDefaultValuesArray();
        $result = '';

        if (isset($defaultValue[$key]) && $defaultValue[$key] !== '') {
            $result = $defaultValue[$key];
        }

        return $result;
    }

    private function getDefaultValuesArray()
    {
        return array(
            'PROV_HOST' => cfg('PROV_HOST'),
            'PROV_PORT' => cfg('PROV_PORT'),
        );
    }

    public function getDHCPLeases()
    {
        $leases = PhonalisaCtrldConnector::getDHCPLeases();
        $data = array();
        $key = 0;
        if (isset($leases->leases)) {
            foreach ($leases->leases as $lease) {
                $data[$key]['id'] = $key;
                $data[$key]['ip'] = isset($lease->ip) ? $lease->ip : '';
                $data[$key]['hostname'] = isset($lease->hostname) ? $lease->hostname : '';
                $data[$key]['mac'] = isset($lease->mac) ? $lease->mac : '';
                $data[$key]['starts'] = isset($lease->starts) ? $lease->starts : '';
                $key++;
            }
        }
        $dataProvider = new CArrayDataProvider($data, array(
            'id' => 'key',
            'pagination' => false,
        ));
        return $dataProvider;
    }

    public static function writeDhcpConfigsToCtrld()
    {
        $result = false;
        if (PhonalisaCtrldConnector::hasConnection()) {
            $writeDhcpConfig = PhonalisaCtrldConnector::writeDhcpConfig(NetworkSettings::getConvertedSettings());
            if (isset($writeDhcpConfig->status) && $writeDhcpConfig->status == 'ok') {
                $result = true;
            }
        }

        if ($result == false) {
            app()->user->setErrorFlashMessage('NETWORK_SETTINGS_WRITE_TO_CTRLD_ERROR');
        }
        return $result;
    }

    public static function getConvertedSettings()
    {
        $settings = NetworkSettings::model()->findAll();
        $convertedSettings = array();
        if ($settings) {
            foreach ($settings as $setting) {
                $convertedKey = "%" . strtoupper($setting->key) . "%";
                $convertedSettings[$convertedKey] = $setting->value;
            }
        }
        $convertedSettings["%PROV_PROTOCOL%"] = cfg("PROV_PROTOCOL");
        $convertedSettings["%PROV_BASEPATH%"] = cfg("PROV_BASEPATH");
        $convertedSettings["%PROV_HOST%"] = cfg("PROV_HOST");
        $convertedSettings["%PROV_PORT%"] = cfg("PROV_PORT");
        $convertedSettings["%PROV_PORT_SECURE%"] = cfg("PROV_PORT_SECURE");
        $convertedSettings["%NTP_HOST%"] = cfg("NTP_HOST");
        return $convertedSettings;
    }
}
