<?php

class ProvisioningProfile extends PhonalisaRecord
{

    public $id;
    public $name;
    public $descr;
    public $lastEditedPhoneType;

    public static function myAttributes()
    {
        return array(
            'name' => 'name',
            'descr' => 'descr',
            'lastEditedPhoneType' => 'lastEditedPhoneType'
        );
    }

    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return Calendar the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * @return string the associated database table name
     */
    public function tableName()
    {
        return strtolower('ProvisioningProfile');
    }

    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
            array('name', 'required'),
            array('id, name, descr, lastEditedPhoneType', 'safe')
                //FIXME 
        );
    }

    /**
     * @return array relational rules.
     */
    public function relations()
    {
        // NOTE: you may need to adjust the relation name and the related
        // class name for the relations automatically generated below.
        return array(
        );
    }

    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'name' => __('PROVISIONING_PROFILE_MODEL_NAME'),
            'descr' => __('PROVISIONING_PROFILE_MODEL_DESCR'),
        );
    }

    /**
     * Retrieves a list of models based on the current search/filter conditions.
     * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
     */
    public function search($order = 'name')
    {
        // Warning: Please modify the following code to remove attributes that
        // should not be searched.

        $criteria = new CDbCriteria;
        $filter = $this->getFilter($criteria, $order);
        if ($filter != null) {
            $dataprovider = new CPhonalisaDataProvider($this, array('criteria' => $criteria));
            $provisioningProfileMax = PhonalisaLicenseManager::provisioningProfileMax();
            if ($provisioningProfileMax >= 0) {
                $dataprovider->pagination->pageSize = $dataprovider->pagination->pageSize > $provisioningProfileMax ? $provisioningProfileMax : $dataprovider->pagination->pageSize;
                $dataprovider->totalItemCount = $dataprovider->totalItemCount > $provisioningProfileMax ? $provisioningProfileMax : $dataprovider->totalItemCount;
            }
            return $dataprovider;
        }
    }

    public function afterDelete()
    {
        $oldParams = PhoneParameters::model()->findAllByAttributes(array('profileID' => $this->id));
        foreach ($oldParams as $oldParam) {
            $oldParam->delete();
        }
    }

    public function getParamsForPhone($phoneModel)
    {
        return PhoneParameters::model()->findAllByAttributes(array('profileID' => $this->id, 'phoneModel' => $phoneModel));
    }
}
