<?php

class SystemUpdate extends PhonalisaRecord
{

    public $descr;
    public $value;
    public $sort;

    public static function myAttributes()
    {
        return array(
            'descr' => 'descr',
            'value' => 'value',
            'sort' => 'sort'
        );
    }

    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return Job the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * @return string the associated database table name
     */
    public function tableName()
    {
        return 'systemUpdate';
    }

    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
// NOTE: you should only define rules for those attributes that
// will receive user inputs.
        return array(
// The following rule is used by search().
// Please remove those attributes that should not be searched.
            array('descr, value, sort', 'safe')
        );
    }

    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'descr' => __('SYSTEM_UPDATE_DESCRIPTION'),
            'value' => __('SYSTEM_UPDATE_VALUE'),
            'phonalisaVersion' => __('SYSTEM_UPDATE_PHONALISA_VERSION'),
            'applianceVersion' => __('SYSTEM_UPDATE_APPLIANCE_VERSION'),
            'releaseNotes' => __('SYSTEM_UPDATE_RELEASE_NOTES'),
            'timestamp' => __('SYSTEM_UPDATE_TIMESTAMP'),
            'releaseDatetime' => __('SYSTEM_UPDATE_RELEASE_DATETIME'),
            'currentPhonalisaVersion' => __('SYSTEM_UPDATE_CURRENT_PHONALISA_VERSION'),
            'currentApplianceVersion' => __('SYSTEM_UPDATE_CURRENT_APPLIANCE_VERSION')
        );
    }

    public function getSystemUpdateOverview($order = 'sort')
    {
        $models = $this->findAll();
        if (empty($models)) {
            $models[] = $this;
        }

        $data = array();
        foreach ($models as $model) {
            if ($model->descr == 'timestamp') {
                continue;
            }
            $data[] = $model;
        }

        $sort = new CSort;
        $sort->defaultOrder = $order;
        return new CArrayDataProvider($data, array('pagination' => false, 'sort' => $sort));
    }

    public function getInstalledVersionOverview()
    {
        $overviewData = self::getInstalledVersionData();
        $data = array();
        $id = 0;
        foreach ($overviewData as $key => $value) {
            $data[$id]['id'] = $id;
            $data[$id]['descr'] = $key;
            $data[$id]['value'] = $value;
            $id++;
        }
        return new CArrayDataProvider($data, array('pagination' => false));
    }

    public static function getInstalledVersionData()
    {
        $timestampModel = SystemUpdate::model()->findByAttributes(array('descr' => 'timestamp'));
        $timestamp = '';
        if ($timestampModel) {
            $timestamp = $timestampModel->value;
        }

        $overviewData = array();
        $overviewData['currentPhonalisaVersion'] = Yii::app()->params->phonalisaVersion;
        $overviewData['currentApplianceVersion'] = self::getSystemVersion();
        $overviewData['timestamp'] = $timestamp;

        return $overviewData;
    }

    public static function getSystemVersion()
    {
        $result = '';
        $ctrld = PhonalisaCtrldConnector::getSystemVersion();
        if (isset($ctrld->status) && isset($ctrld->version) && $ctrld->status === 'ok') {
            $result = $ctrld->version;
        }
        return $result;
    }

    public static function getUpdateFileDataFromServer()
    {
        $applianceModel = app()->params->applianceModel;
        $releaseBranch = app()->params->phonalisaReleaseBranch;

        $url = 'http://winupdates.phonalisa.de/appliance/' . $applianceModel . '/' . $releaseBranch . '/update.json';
        $response = CurlHelper::sendGetRequest($url);
        $data = str_replace(array(PHP_EOL, ' '), '', $response['content']);
        $data = str_replace(',}', '}', $data);
        $data = json_decode($data);

        return $data;
    }

    public static function serverVersionNewerThanSystem($version)
    {
        $updateVersion = SystemUpdate::model()->findByAttributes(array('descr' => 'applianceVersion'));
        if ($updateVersion) {
            return SystemUpdate::compareVersions($version, $updateVersion->value);
        }
        return false;
    }

    protected static function compareVersions($currentVersion, $newVersion)
    {
        $currentVersionParts = explode(".", $currentVersion);
        $newVersionParts = explode(".", $newVersion);

        foreach ($newVersionParts as $key => $value) {
            if (!isset($currentVersionParts[$key])) {
                return true;
            } else if ((int) $currentVersionParts[$key] < (int) $value) {
                return true;
            } else if ((int) $currentVersionParts[$key] > (int) $value) {
                return false;
            }
        }
        return false;
    }

    public static function createUpdateDoneAlert()
    {
        $phonalisaVersion = SystemUpdate::model()->findByAttributes(array('descr' => 'phonalisaVersion'));
        $releaseNotes = SystemUpdate::model()->findByAttributes(array('descr' => 'releaseNotes'));
        $changeLog = "Release Notes:\n" . base64_decode($releaseNotes->value) . "\n\n";
        $changeLog.= "Upgrade-Log:\n" . SystemUpdate::getSystemUpgradeStatusAsString();

        $alert = new Alert();
        $alert->created = PhonalisaDate::newServerDate();
        $alert->level = Alert::LEVEL_INFO;
        $alert->component = Alert::COMPONENT_SYSTEM;
        $alert->shortDescr = __('SYSTEM_UPDATE_CRON_JOB_UPDATE_SYSTEM_TO_VERSION') . " " . $phonalisaVersion->value;
        $alert->descr = $changeLog;
        if (!$alert->save()) {
            Log::e('Could not save Alert-Model!', 'SystemUpdate');
        }
    }

    protected static function getSystemUpgradeStatusAsString()
    {
        $upgradeStatus = PhonalisaCtrldConnector::getSystemUpgradeStatus();
        $result = '';
        foreach ($upgradeStatus->output as $value) {
            if ($value != '') {
                $result.= $value . "\n";
            }
        }
        return $result;
    }
}
