<?php

class iCloudData extends PhonalisaRecord
{

    public $id;
    public $owner;
    public $listId;
    public $fileName;
    public $fileId;
    public $directoryId;
    public $data;
    public $eTag;
    public $changed;
    public $provider;

    public static function myAttributes()
    {

        return array(
            'owner' => 'owner',
            'listId' => 'listId',
            'fileName' => 'fileName',
            'fileId' => 'fileId',
            'data' => 'data',
            'eTag' => 'eTag',
            'directoryId' => 'directoryId',
            'provider' => 'provider'
        );
    }

    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    public function rules()
    {
        return array(
            array('owner, listId, fileName, fileId, data, eTag', 'required'),
            array(
                'owner, '
                . 'listId, '
                . 'fileName, '
                . 'fileId, '
                . 'directoryId, '
                . 'data, '
                . 'eTag, '
                . 'provider',
                'safe',
                'on' => 'update'
            )
        );
    }

    public function search($order = 'title')
    {
        $criteria = new CDbCriteria;
        $criteria->condition = 'owner=' . Yii::app()->user->sudoUser;

        $dataProvider = new CPhonalisaDataProvider($this, array(
            'criteria' => $criteria,
        ));
        return $dataProvider;
    }

    public function beforeDelete()
    {
        $deletedItem = new DeletedItem();
        $deletedItem->model_id = $this->id;
        $deletedItem->model_name = get_Class($this);
        $deletedItem->deleted = time();
        $this->removeVCardFromICloudContactsList();

        if ($deletedItem->save()) {
            return parent::beforeDelete();
        }
        return false;
    }

    public function createCopy($address)
    {
        $vCardCopy = new iCloudData();
        $vCardCopy->fileName = $address->id . '.vcf';
        $vCardCopy->attributes = $this->attributes;
        $vCardCopy->owner = $address->contact_owner;

        return $vCardCopy;
    }

    public function removeVCardFromICloudContactsList()
    {
        $ids = json_decode($this->listId, true);
        if (!empty($ids)) {
            foreach ($ids as $id) {
                $iCloudContactsList = iCloudContactsList::model()->findByPk($id);
                if (isset($iCloudContactsList)) {
                    $iCloudContactsList->updateICloudContactsList('deletedLocalContacts', $this->fileName);
                }
            }
        }
    }

    public function setNewICloudContactListIDs($addressBookId)
    {
        $iCloudContactsLists = iCloudContactsList::model()->findAllByAttributes(array('directoryId' => $addressBookId));
        $newLists = array();
        if (!empty($iCloudContactsLists)) {
            foreach ($iCloudContactsLists as $iCloudContactsList) {
                if (isset($iCloudContactsList)) {
                    $newLists[] = $iCloudContactsList->id;
                }
            }
        }
        $this->listId = json_encode($newLists);
        if (!$this->save()) {
            $this->createErrorLogMessages();
        }
    }

    public function afterDelete()
    {
        $file = Addresses::model()->findByPk($this->fileId);
        if (!$file) {
            $file = CalendarEvent::model()->findByPk($this->fileId);
        }
        
        if ($file) {
            $file->delete();
        }

        return parent::afterDelete();
    }

    private function createErrorLogMessages()
    {
        Log::e("An error has occurred saving vCardData-ID $this->id", 'iCloudSync [iCloudContactsList]');
        foreach ($this->errors as $key => $errorMsg) {
            Log::e("Error in $key ");
            if (is_array($errorMsg)) {
                Log::e("ErrorMessage in " . array_shift($errorMsg));
            }
        }
    }
}
