<?php

class TreeMapHelper
{

    public static function getItemsForKeyMap()
    {
        $modelNames = TreeMapHelper::getNamesForKeyMap();
        $result = array();

        foreach ($modelNames as $key => $modelName) {
            $color = TreeMapHelper::setColorForTreeDataType($modelName);
            $result[$key + 1]['label'] = '<span class="keyMapColor" style="background:' . $color . '">&nbsp;</span> ' . $modelName;
        }
        return $result;
    }

    public static function getNamesForKeyMap()
    {
        return array(
            __('ROUTINGGATEWAYS_HEADER_TITLE'),
            __('ROUTING_INBOUND_MODEL_PATTERN'),
            __('RENDER_TARGET_USER'),
            __('RENDER_TARGET_GROUP'),
            __('RENDER_TARGET_IVR'),
            __('RENDER_TARGET_QUEUE'),
            __('RENDER_TARGET_MANUAL'),
            __('RENDER_TARGET_CONFERENCE'),
            __('ROUTINGINBOUND_MODEL_TARGET_USER_FAX'),
            __('ROUTINGINBOUND_MODEL_TARGET_GROUP_FAX'),
            __('ROUTINGEXTERNALAGENT_HEADER_TITLE'),
            __('CALL_GROUP'),
            __('PHONESETTINGSCALLFORWARD_TARGET_VOICE_MAIL'),
            __('PHONESETTINGSCALLFORWARD_TARGET_ANNONCEMENT')
        );
    }

    public static function setColorForTreeDataType($model)
    {
        $modelName = $model;
        if (!is_string($model)) {
            $modelName = get_class($model);
        }
        switch ($modelName) {
            case __('ROUTINGGATEWAYS_HEADER_TITLE'):
            case 'Gateway':
                return '#adadad';
                break;

            case __('RENDER_TARGET_USER'):
            case 'User':
                return '#003cff';
                break;

            case __('RENDER_TARGET_GROUP'):
            case 'Groups':
                return '#ffff00';
                break;

            case __('ROUTING_INBOUND_MODEL_PATTERN'):
                return '#947600';
                break;

            case 'ForwardPreset':
                return '#9100ff';
                break;

            case __('CALL_GROUP'):
            case 'CallGroup':
                return '#ff00ff';
                break;

            case __('RENDER_TARGET_IVR'):
            case 'IVR':
                return '#b81a79';
                break;
            case 'Queue':
            case __('RENDER_TARGET_QUEUE'):
                return '#999157';
                break;

            case __('RENDER_TARGET_CONFERENCE'):
            case __('CONFERENCE');
            case 'Conference':
                return '#e88e3a';
                break;

            case 'Tiers':
            case __('ROUTINGEXTERNALAGENT_HEADER_TITLE'):
                return '#4DA36A';
                break;

            case __('ROUTINGINBOUND_MODEL_TARGET_USER_FAX'):
            case __('USER_FAX'):
                return '#00ffff';
                break;

            case __('ROUTINGINBOUND_MODEL_TARGET_GROUP_FAX'):
                return '#36b0e0';
                break;

            case __('RENDER_TARGET_MANUAL'):
            case __('MANUAL'):
                return '#00ff0d';
                break;

            case __('PHONESETTINGSCALLFORWARD_TARGET_ANNONCEMENT'):
            case 'announcement':
                return '#166161';
                break;

            case __('PHONESETTINGSCALLFORWARD_TARGET_VOICE_MAIL'):
            case __('USER_VOICE_MAILS'):
            case 'voicemail':
                return '#43465E';
                break;

            case __('CALL_TROUGHT'):
                return '';
                break;

            case 'ERROR':
                return '#ff0000';
                break;

            default:
                return '#000000';
//                return 'red';
                break;
        }
    }

    public static function getModelFromTarget($attribute, $searchInAllModels = false)
    {
        $modelNames = array('User', 'Groups', 'Queue', 'CallGroup', 'Conference', 'IVR', 'Gateway');

        if ($searchInAllModels) {
            $nonTargetModels = array('AudioFile', 'ForwardPreset', 'MusicOnHold', 'UserToCallGroup');
            $modelNames = array_merge($modelNames, $nonTargetModels);
        }

        foreach ($modelNames as $modelName) {
            $model = PhonalisaRecord::model($modelName)->findByPk($attribute);
            if ($model) {
                return $model;
            }
        }
        if (!$model) {
            foreach ($modelNames as $modelName) {
                $model = PhonalisaRecord::model($modelName)->findByAttributes(array('extension' => $attribute));
                if ($model) {
                    return $model;
                }
            }
        }
        return false;
    }

    public static function getTargetTypeForModel($modelName)
    {
        $targetModel = get_class($modelName);
        switch ($targetModel) {
            case 'User':
                return TargetType::User;
                break;
            case 'Groups';
                return TargetType::Group;
                break;
            case 'IVR';
                return TargetType::Ivr;
                break;
            case 'Queue':
                return TargetType::Queue;
                break;
            case 'CallGroup':
                return TargetType::CallGroup;
                break;
            case 'Conference':
                return TargetType::Conference;
                break;
            default:
                return false;
                break;
        }
    }

    public static function checkForForwardLoops($currentTargetModel)
    {
        $models = ForwardPreset::model()->findAll();
        foreach ($models as $model) {
            $configs = $model->configs;
            $forwardRules = TreeMapHelper::getDecodedForwardConfigs($configs);
            foreach ($forwardRules as $forwardRule) {
                $target = $forwardRule->target;
                $targetType = $target->type;
                if ($targetType == 'number') {
                    $targetModel = TreeMapHelper::getModelFromTarget($target->desc);
                    if ($targetModel != NULL) {
                        $checkArray[$model->owner] = $targetModel->id;
                    }
                }
            }
        }
        $arrayFlip = array_flip($checkArray);
        $result = array_intersect_assoc($checkArray, $arrayFlip);

        if (in_array($currentTargetModel->owner, $result)) {
            return $result[$currentTargetModel->owner];
        }
        return false;
    }

    public static function getDecodedForwardConfigs($configs)
    {
        $configsDecodedBase64 = base64_decode($configs);
        $configsDecodedJson = json_decode($configsDecodedBase64);
        $searchFor = $configsDecodedJson->active;
        $forwardRules = array();
        if ($searchFor != "none") {
        	$forwardRules = $configsDecodedJson->$searchFor;
	}
        return $forwardRules;
    }

    public static function getIvrSearchArray($model, $search, $action = 'action')
    {
        $availableKeys = IVR::getAvailableKeys();
        $result = array();
        foreach ($availableKeys as $getAvailableKey) {
            $searchKey = 'key_' . $getAvailableKey . '_';
            if ($model[$searchKey . $action] == $search) {
                $target = $searchKey . 'target';
                $key = $getAvailableKey;
                $result[$key] = $model[$target];
            }
        }
        return $result;
    }

    public static function changeIvrNonNumericKeysToNumericString($key)
    {
        if ($key === 'asterisk') {
            $result = '10';
        } else if ($key === 'pound') {
            $result = '11';
        } else if ($key === 'invalid') {
            $result = '12';
        } else if ($key === 'noInput') {
            $result = '13';
        } else {
            $result = $key;
        }
        return $result;
    }

    public function getModelKeyArrayForToolTipText($modelName)
    {
        $modelKeyArray = array();

        if ($modelName == 'Gateway') {
            $modelKeyArray = array(
                'name',
                'host',
                'current_ip',
                'device_name',
                'expires',
                'typ',
                'active',
                'register',
                'state',
                'maxChannels',
                'inboundBlacklistId',
                'outboundBlacklistId',
                'std_prefix',
                'dialOutToggle',
                'caller_id_in_from',
                'expire_seconds',
                'realm',
                'from_user',
                'from_domain',
                'proxy',
                'register_proxy',
                'register_transport',
                'contact_params',
                'tcp_transport',
                'allowed_codecs',
                'ping',
                'calldeflect_support',
                'answer_on_outbound_dial'
            );
        } else if ($modelName == 'User') {
            $modelKeyArray = array(
                'extension',
                'name',
                'firstname',
                'lastename',
                'email',
                'user_timezone',
                'user_language',
                'user_time_format',
                'user_date_format',
                'zrtp',
                'fax_header_text',
                'fax_header_number',
                'droptarget',
                'group',
                'userMonitor',
                'internMessageBeforeAnswer',
                'externMessageBeforeAnswer',
                'isSystemAdmin',
                'callForwardSet',
                'defaultCallForwardSet',
                'accountBalance'
            );
        } else if ($modelName == 'Groups') {
            $modelKeyArray = array(
                'extension',
                'name',
                'parent',
                'playlistId',
                'provisioningProfile',
                'keyProfile',
                'inboundBlacklistId',
                'outboundBlacklistId',
                'fax_header_text',
                'fax_header_number',
                'droptarget',
                'night_circut',
                'accountingEnabled'
            );
        } else if ($modelName == 'CallGroup') {
            $modelKeyArray = array(
                'extension',
                'name',
                'description',
                'playlistId',
                'target_on_timeout',
                'target_on_busy',
                'target_on_nightcircut',
                'max_hold_time',
                'default_ring_time',
                'night_circut'
            );
        } else if ($modelName == 'IVR') {
            $modelKeyArray = array(
                'extension',
                'title',
                'description',
                'announce_file',
                'answer_time',
                'answer_retry'
            );
        } else if ($modelName == 'Queue') {
            $modelKeyArray = array(
                'extension',
                'name',
                'moh',
                'strategy',
                'abandoned_resume_allowed',
                'announce_file',
                'cid_prefix',
                'discard_abandoned_after',
                'max_wait_time',
                'max_wait_time_with_no_agent',
                'max_wait_time_with_no_agent_time_reached',
                'target_on_no_agents',
                'target_on_no_agents_meta',
                'target_on_timeout',
                'target_on_timeout_meta',
                'tier_rule_no_agent_no_wait',
                'tier_rule_wait_multiply_level',
                'tier_rule_wait_second',
                'tier_rules_apply',
                'time_base_score'
            );
        } else if ($modelName == 'Conference') {
            $modelKeyArray = array(
                'extension',
                'name',
                'announce_file',
                'listen_to_music',
                'mohId',
                'useConfereceMemberName',
            );
        }

        return $modelKeyArray;
    }

    public function getTranslationTagsIfTranslationMode()
    {
        if (Yii::app()->user->translateMode) {
        $tranlsationTagsArray = array(
            'GLOBAL_YES',
            'GLOBAL_NO',
            'ADMIN_CONFIGURATION_OVERVIEW_INCOMMING_CALL',
            'ADMIN_CONFIGURATION_OVERVIEW_TREEMAP_TYPE_NONE_SELECTED'
        );

        foreach ($tranlsationTagsArray as $translationTag) {
            echo "$translationTag => <i data-translation-id='$translationTag'>" . __($translationTag) . "</i><br>";
        }
    }
    }
}
