<?php

require_once(dirname(realpath(__FILE__)) . '/../../../components/callforwarding/class.usercallforward.php');
require_once(dirname(realpath(__FILE__)) . '/../../../components/callforwarding/class.callforwardingfactory.php');

class DefaultController extends Controller
{

    /**
     * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
     * using two-column layout. See 'protected/views/layouts/column2.php'.
     */
    public $layout = '//layouts/column2';

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(
            array('allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('index', 'view', 'script', 'getJson', 'setJson', 'addSystemRules'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array('create', 'update'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('allow', // allow admin user to perform 'admin' and 'delete' actions
                'actions' => array('admin', 'delete'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    /**
     * Updates a particular model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id the ID of the model to be updated
     */
    public function actionView($id)
    {
        $model = ForwardPreset::model()->findByPk($id);

        $this->render('_form', array(
            'model' => $model,
            'write' => false,
        ));
    }

    public function actionUpdate($id)
    {
        $model = ForwardPreset::model()->findByPk($id);


        if (isset($_POST['ForwardPreset'])) {
            $model->attributes = $_POST['ForwardPreset'];
            if ($model->save()) {
                $this->redirect(array('index'));
            }
        }

        $this->render('_form', array(
            'model' => $model,
            'write' => true,
        ));
    }

    public function actionCreate()
    {

        $model = new ForwardPreset();
        $model->type = ForwardPresetType::PrivatePreset;
        $model->owner = app()->user->sudoUserInstance->id;

        if (isset($_POST['ForwardPreset'])) {
            $model->attributes = $_POST['ForwardPreset'];
            if ($model->save()) {
                $this->redirect(array('update', 'id' => $model->id));
            }
        }

        $this->render('_form', array(
            'model' => $model,
            'write' => true,
        ));
    }

    public function actionDelete($id)
    {
        if (Yii::app()->request->isPostRequest) {
            // if AJAX request (triggered by deletion via admin grid view),
            // we should not redirect the browser
            if (isset($_GET['ajax'])) {
                $this->loadModel($id)->delete();
            }
        } else {
            throw new CHttpException(400, 'Invalid request. Please do not repeat this request again.');
        }
    }

    /**
     * Lists all models.
     */
    public function actionIndex()
    {
        $errorMassages = array();

        if (isset($_POST['presets-grid_c0'][0])) {
            $user = app()->user->sudoUserInstance;
            $user->callForwardSet = $_POST['presets-grid_c0'][0];

            if (isset($_POST['presets-grid_c1'][0])) {
                $user->defaultCallForwardSet = $_POST['presets-grid_c1'][0];
            }

            if (!$user->save()) {
                $errorMassages[] = __('CHANGES_NOT_SAVED_CAUSE_OF_AN_ERROR');
            }
        }

        if (isset($_POST['ForwardPreset']['key'])) {
            foreach ($_POST['ForwardPreset']['key'] as $presetId => $key) {
                $model = ForwardPreset::model()->findByPk($presetId);
                if ($model == null) {
                    Log::e("ForwardPreset model $presetId not found");
                }
                //$model->setPhoneKey($key);

                $keyModel = $model->getPhoneKeyModel();

                if ($keyModel == null) {
                    $keyModel = new ForwardPresetPhoneKey();
                    $keyModel->forwardPreset = $model->id;
                }

                $keyModel->key = $key;

                if (!$keyModel->save()) {
                    $errorMassages[] = $keyModel->errors;
                }
            }
        }

        if (isset($_POST['ForwardPreset'])) {
            if (empty($errorMassages)) {
                Yii::app()->user->setFlash('success', __('CHANGES_SAVED_SUCCSESSFULLY'));
            } else {

                $cleanMessages = array();

                foreach ($errorMassages as $errorModel) {
                    foreach ($errorModel as $errorField) {
                        foreach ($errorField as $errorFieldError) {
                            $cleanMessages[] = $errorFieldError;
                        }
                    }
                }

                $errorMessage = "";
                foreach (array_unique($cleanMessages) as $msg) {
                    $errorMessage .= "<p>$msg</p>";
                }

                Yii::app()->user->setFlash('error', $errorMessage);
            }
        }

        $models = app()->user->sudoUserInstance->getForwardingSets();
        $dataProvider = new CArrayDataProvider(
            $models,
            array(
                'pagination' => false,
                'sort' => array(
                    'defaultOrder' => 'created ASC',
                ),
            )
        );

        $this->render('presets', array(
            'dataProvider' => $dataProvider,
            'model' => ForwardPreset::model(),
        ));
    }

    public function actionAddSystemRules()
    {
        $model = new ForwardPreset();
        $model->name = "None";
        $model->description = "All forward rules are disabled";
        $model->type = ForwardPresetType::SytemPreset;
        $model->configs = base64_encode(
            '{"active":"none","advanced":[],"simple":[],"possibleCriterias":{},"possibleTargets":[]}'
        );

        if ($model->save()) {
            echo "done";
        }
    }

    public function actionGetJson($id)
    {
        $user = app()->user->sudoUserInstance;

        $forwardModel = $this->loadModel($id);
        $json = base64_decode($forwardModel->configs);

        $jsonObject = json_decode($json);

        $callforward = CallForwardingFactory::getUserCallForwardFromJSONRepresentation($jsonObject, $user);

        if ($forwardModel->forwardType == ForwardType::Immediately) {
            $callforward->setActive("simple");
        } elseif ($forwardModel->forwardType == ForwardType::Disabled) {
            $callforward->setActive("none");
        } elseif ($forwardModel->forwardType == ForwardType::Custom) {
            $callforward->setActive("advanced");
        }

        $forwardJson = CallForwardingFactory::getJSONRepresentationOfUserCallForward($callforward);
        $this->renderPartial(
            'getJson',
            array(
                'json' => $forwardJson
            )
        );
    }

    public function actionSetJson($id)
    {
        $time = microtime();
        Yii::log(get_class($this) . '.save() start: ' . (microtime() - $time), "error");

        $user = app()->user->sudoUserInstance;

        $jsonData = $_REQUEST['jsonData'];

        $jsonObject = json_decode($jsonData);
        $callforward = CallForwardingFactory::getUserCallForwardFromJSONRepresentation($jsonObject, $user);

        $forwardJson = CallForwardingFactory::getJSONRepresentationOfUserCallForward($callforward);

        $model = $this->loadModel($id);
        $model->configs = base64_encode($forwardJson);
        Yii::log(get_class($this) . '.save() model will start saving at: ' . (microtime() - $time), "error");

        if (!$model->save()) {
            die("cannot save user object!");
        }

        Yii::log(get_class($this) . '.save() model has done saving at: ' . (microtime() - $time), "error");

        $this->renderPartial(
            'getJson',
            array(
                'json' => $forwardJson
            )
        );

        Yii::log(get_class($this) . '.save() end duration: ' . (microtime() - $time), "error");
    }

    public function actionScript()
    {
        $this->renderPartial('script');
    }

    /**
     * Returns the data model based on the primary key given in the GET variable.
     * If the data model is not found, an HTTP exception will be raised.
     * @param integer the ID of the model to be loaded
     */
    public function loadModel($id)
    {
        $model = ForwardPreset::model()->findByPk($id);
        if ($model === null) {
            throw new CHttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

    /**
     * Performs the AJAX validation.
     * @param CModel the model to be validated
     */
    protected function performAjaxValidation($model)
    {
        if (isset($_POST['ajax']) && $_POST['ajax'] === 'phonesettings-user-form') {
            echo CActiveForm::validate($model);
            Yii::app()->end();
        }
    }
}
