<?php

class DefaultController extends Controller
{

    /**
     * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
     * using two-column layout. See 'protected/views/layouts/column2.php'.
     */
    public $layout = '//layouts/column2';

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(
            array('allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('index', 'ajaxPin', 'ajaxPassword'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    /**
     * Lists all models.
     */
    public function actionIndex()
    {
        $userId = Yii::app()->User->sudoUser;
        $model = User::model()->findByPk($userId);
        if (!$model) {
            app()->user->logout();
            app()->user->loginRequired();
        }
        $this->render('index', array(
            'model' => $model,
        ));
    }

    public function actionAjaxPin($id)
    {

        $userId = Yii::app()->User->sudoUser;
        $model = User::model()->findByPk($userId);

        $model->scenario = "changePin";
        // Uncomment the following line if AJAX validation is needed
        $this->performAjaxValidationOnPin($model);
        if (isset($_POST['User'])) {
            $model->attributes = $_POST['User'];
            if ($model->pin == $model->oldPin) {
                if ($model->newPin == $model->confirmPin) {
                    $model->pin = $model->newPin;
                    $model->scenario = "";
                    if (!$model->save()) {
                        throw new CHttpException(500, "Can't save data!");
                    }
                }
            }
        }

        $this->renderPartial('_formPin', array('model' => $model), false, true);
    }

    public function actionAjaxPassword($id)
    {

        $userId = Yii::app()->User->sudoUser;
        $model = User::model()->findByPk($userId);

        $model->scenario = "changePassword";
        // Uncomment the following line if AJAX validation is needed
        $this->performAjaxValidationOnPassword($model);


        if (isset($_POST['User']) && !isset($_POST['ajax'])) {
            $model->attributes = $_POST['User'];
            $model->oldPassword = PasswordGenerator::cryptUserPassword($model, $_POST['User']['oldPassword']);
            if ($model->password == $model->oldPassword) {
                if ($model->newPassword == $model->confirmPassword) {
                    $model->password = $model->newPassword;
                    $model->scenario = "";
                    if (!$model->save()) {
                        throw new CHttpException(500, "Can't save data!");
                    }
                }
            }
        }

        $this->renderPartial('_formPassword', array('model' => $model), false, true);
    }

    /**
     * Returns the data model based on the primary key given in the GET variable.
     * If the data model is not found, an HTTP exception will be raised.
     * @param integer the ID of the model to be loaded
     */
    public function loadModel($id)
    {
        $model = ThirdPartyCredential::model()->findByPk($id);
        if ($model === null) {
            throw new CHttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

    /**
     * Performs the AJAX validation.
     * @param CModel the model to be validated
     */
    protected function performAjaxValidationOnPassword($model)
    {
        if (isset($_POST['ajax']) && $_POST['ajax'] === 'phonesettings-credentials-password-form') {
            echo CActiveForm::validate($model);
            Yii::app()->end();
        }
    }

    protected function performAjaxValidationOnPin($model)
    {
        if (isset($_POST['ajax']) && $_POST['ajax'] === 'phonesettings-credentials-pin-form') {
            echo CActiveForm::validate($model);
            Yii::app()->end();
        }
    }

    public function renderPhoneSettingsCredentials()
    {

        $userSettings = array(
            array(
                'id' => '12',
                'key' => 'pin',
                'short' => $this->getCredentialsGridContent("PHONESETTINGS_USER_MODEL_CURRENT_PIN_SHORT"),
                'value' => '',
                'descr' => $this->getCredentialsGridContent("PHONESETTINGS_USER_MODEL_CURRENT_PIN_DESCRIPTION"),
                'type' => 'text',
            ),
            array(
                'id' => '15',
                'key' => 'password',
                'short' => $this->getCredentialsGridContent("PHONESETTINGS_USER_MODEL_CURRENT_PASSWORD_SHORT"),
                'value' => '',
                'descr' => $this->getCredentialsGridContent("PHONESETTINGS_USER_MODEL_CURRENT_PASSWORD_DESCRIPTION"),
                'type' => 'text',
            ),
        );

        return $userSettings;
    }

    public function getCredentialsGridContent($translationTag)
    {
        return "<p data-translation-id='$translationTag'>" . __($translationTag) . "</p>";
    }

    public function phoneSettingsCredentialsSearch()
    {
        $phonesettings = $this->renderPhoneSettingsCredentials();

        $dataProvider = new CArrayDataProvider($phonesettings, array('pagination' => false));

        return $dataProvider;
    }
}
