<?php

require_once dirname(__FILE__) . '/../../lib/AbstractPhoneModel.php';

class Grandstream extends AbstractPhoneModel
{

    public function __construct($manufactorName = "Grandstream", $phoneModelName = "", $phoneUserAgent = "")
    {
        $this->mManufactorName = $manufactorName;
        $this->mPhoneModelName = $phoneModelName;
        $this->mPhoneUserAgent = $phoneUserAgent;
    }

    public static function getManufactorName()
    {
        return "Grandstream";
    }

    public static function getFunctionPlaceholders()
    {
        return array(
            '0' => 'PHONE_GRANDSTREAM_FUNCTION_EMPTY', // Empty Key
//            ' ' => 'PHONE_GRANDSTREAM_FUNCTION_EMPTY',                        // Empty Key
//            '0' => 'PHONE_GRANDSTREAM_FUNCTION_SPEED_DIAL',                   // Speed Dial
            '1' => 'PHONE_GRANDSTREAM_FUNCTION_BUSY_LAMP_FIELD', // Busy Lamp Field
            '2' => 'PHONE_GRANDSTREAM_FUNCTION_PRESENCE_WATCHER', // Presence Watcher
//            '3' => 'PHONE_GRANDSTREAM_FUNCTION_EVENTLIST_BLF', // eventlist BLF
            '4' => 'PHONE_GRANDSTREAM_FUNCTION_SPEED_DIAL_VIA_ACTIVE_ACCOUNT', // Speed Dial via active account
            'X01' => 'PHONE_GRANDSTREAM_FUNCTION_CALL_FORWARDING', // Call Forwarding
            'X02' => 'PHONE_GRANDSTREAM_FUNCTION_SPEED_DIAL', // Speed Dial
            'X03' => 'PHONE_GRANDSTREAM_FUNCTION_QUEUE_LOGIN', // Queue Login/Logout
        );
    }

    public static function getFunctions($translated = true, $phone = '', $level = '')
    {
        $placeholders = self::getFunctionPlaceholders();

        if ($translated) {
            $functions = array();
            foreach ($placeholders as $key => $value) {
                $functions[$key] = __($value);
            }
            return $functions;
        } else {
            return $placeholders;
        }
    }

    public function getAvailableKeys($level)
    {
        $phones = $this->getSupportedPhoneModels();
        if (!array_key_exists($this->mPhoneModelName, $phones)) {
            return array();
        }

        $phone = $phones[$this->mPhoneModelName];
        $ret = array();
        if ($level == "" || $level == "P") {
            for ($i = 1; $i < $phone->getKeys() + 1; $i++) {
                $button = new PhoneKey();
                $button->name = 'P' . $i;
                $button->phone = $this->mPhoneModelName;
                $button->function = '';
                $button->level = 'P';
                if ($phone->getKeys() == 11) {
                    $button->key_idx = 'P' . ($i * 4 + 353);
                } elseif ($phone->getKeys() == 54) {
                    $button->key_idx = 'P' . ($i + 6001);
                } else {
                    $button->key_idx = 'P' . ($i + 323);
                }
                $button->availableFunctions = $this->getFunctions(true, $button->phone, $button->level);
                $ret[$button->name] = $button;
            }
        }

        $extKeys = $phone->getExtKeys();
        $extModules = $phone->getExtModules();

        for ($i = 1; $i <= $extModules; $i++) {
            if ($level == "" || $level == 'EXT' . $i) {
                for ($j = 1; $j <= $extKeys; $j++) {
                    $button = new PhoneKey();
                    $button->name = 'EXT' . $i . ' P' . $j;
                    $button->phone = $this->mPhoneModelName;
                    $button->function = '';
                    $button->level = 'EXT' . $i;
                    if ($i == '1') {
                        $button->key_idx = 'P' . ($j + 6001);
                    } else {
                        $button->key_idx = 'P' . ($j + 6057);
                    }
                    $button->availableFunctions = $this->getFunctions(true, $button->phone, $button->level);
                    $ret[$button->name] = $button;
                }
            }
        }
        return $ret;
    }

    public function getAvailableKeyLevels()
    {
        $phones = $this->getSupportedPhoneModels();
        if (!array_key_exists($this->mPhoneModelName, $phones)) {
            return array();
        }
        $phone = $phones[$this->mPhoneModelName];
        $keyLevels = array();

        if ($phone->getKeys() != 0) {
            $lineKeyLevel = array();
            $lineKeyLevel["level"] = 'P';
            $lineKeyLevel["name"] = "GRANDSTREAM_KEYLEVEL_MEMORY_KEYS_NAME";
            $lineKeyLevel["descr"] = "GRANDSTREAM_KEYLEVEL_MEMORY_KEYS_DESCR";
            $keyLevels[] = $lineKeyLevel;
        }

        $extModules = $phone->getExtModules();
        if ($phone->getExtKeys() != 0) {
            for ($i = 1; $i <= $extModules; $i++) {
                $ExtKeyLevel = array();
                $ExtKeyLevel["level"] = "EXT" . $i;
                $ExtKeyLevel["name"] = "GRANDSTREAM_KEYLEVEL_EXT" . $i . "_NAME";
                $ExtKeyLevel["descr"] = "GRANDSTREAM_KEYLEVEL_EXT" . $i . "_DESCR";
                $keyLevels[] = $ExtKeyLevel;
            }
        }
        return $keyLevels;
    }

    public static function getSupportedPhoneModels()
    {
        $models = array();
        $tmp = new Grandstream(Grandstream::getManufactorName(), "Grandstream GXP1450", "gxp1450");
        $tmp->setPhoneModelNameHumanReadable("Grandstream GXP1450");
        $tmp->setKeys(0);
        $tmp->setLineKeys(2);
        $models[$tmp->getPhoneModelName()] = $tmp;

        $tmp = new Grandstream(Grandstream::getManufactorName(), "Grandstream GXP2000", "gxp2000");
        $tmp->setPhoneModelNameHumanReadable("Grandstream GXP2000");
        $tmp->setKeys(7);
        $tmp->setLineKeys(4);
        $tmp->setExtKeys(56);
        $tmp->setExtModules(2);
        $models[$tmp->getPhoneModelName()] = $tmp;

        $tmp = new Grandstream(Grandstream::getManufactorName(), "Grandstream GXP2010", "gxp2010");
        $tmp->setPhoneModelNameHumanReadable("Grandstream GXP2010");
        $tmp->setKeys(18);
        $tmp->setLineKeys(4);
        $tmp->setExtKeys(56);
        $tmp->setExtModules(2);
        $models[$tmp->getPhoneModelName()] = $tmp;

        $tmp = new Grandstream(Grandstream::getManufactorName(), "Grandstream GXP2020", "gxp2020");
        $tmp->setPhoneModelNameHumanReadable("Grandstream GXP2020");
        $tmp->setKeys(7);
        $tmp->setLineKeys(6);
        $tmp->setExtKeys(56);
        $tmp->setExtModules(2);
        $models[$tmp->getPhoneModelName()] = $tmp;

        return $models;
    }
}
