<?php
require_once "ConfigItem.php";

class DLSMessage {

	const ActionReadAllItems = "ReadAllItems";
	const ActionWriteItems = "WriteItems";
	const ActionCleanup = "Cleanup";
	const ActionFileDeployment = "FileDeployment";
	const ActionSoftwareDeployment = "SoftwareDeployment";
	
	private $items = array();
	private $nonce = null;
	private $action = null;
	
	
	public function __construct($nonce) {
		
		$this->nonce = $nonce;
		
	}
	
	public function setAction($action) {
		$this->action = $action;
	}
	
	public function getAction() {
		return $action;
	}
	
	public function addItem(ConfigItem $item) {
		if ($item->index != null) {
			$this->items[$item->name."-".$item->index] = $item;
		} else {
			$this->items[$item->name] = $item;
		}
	}
	
	
	public function xml() {
		
		$root =  new SimpleXMLElement("<?xml version=\"1.0\" encoding=\"UTF-8\"?><DLSMessage></DLSMessage>");
		$root->addAttribute("xmlns", "http://www.siemens.com/DLS");
		$root->addAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
		$root->addAttribute("xsi:schemaLocation", "http://www.siemens.com/DLS");
		
		$message = $root->addChild("Message");
		$message->addAttribute("nonce", $this->nonce);
		
		if ($this->action != null) {
			Log::v("[".Yii::app()->session["mac-addr"]."] Writing Action: ".$this->action, "siemens-dls");
			$message->addChild("Action", $this->action);
		}

		if (count($this->items) > 0) {
			$itemList = $message->addChild("ItemList");
			foreach ($this->items as $item) {
				if ($item->index != NULL) {
					Log::d("[".Yii::app()->session["mac-addr"]."] Writing Item: ".$item->name."[".$item->index."] = ".$item->value, "siemens-dls");
				} else {
					Log::d("[".Yii::app()->session["mac-addr"]."] Writing Item: ".$item->name." = ".$item->value, "siemens-dls");
				}
				$item->addXMLtoParent($itemList);
			}
		}
		
		
		
		return $root->asXML();
	}
	
	
}