<?php

include dirname(__FILE__) . "/lib/AbstractTiptel32xxXmlVoicemailApp.php";
require_once "lib/XmlMenuScreen.php";
require_once "lib/XmlFormattedTextScreen.php";

class XmlVoicemailApp extends AbstractTiptel32xxXmlVoicemailApp
{

    private $screen = null;

    private function getScreenAttributes($changeAttribute = array())
    {
        $this->setVoicemailLanguage();
        $screenAttributes = array();
        $screenAttributes["defaultIndex"] = "";
        $screenAttributes["style"] = "none";
        $screenAttributes["Beep"] = "no";
        $screenAttributes["cancelAction"] = "";
        $screenAttributes["wrapList"] = "yes";
        $screenAttributes["Timeout"] = "30";
        $screenAttributes["LockIn"] = "no";

        if (!empty($changeAttribute)) {
            foreach ($changeAttribute as $key => $value) {
                $screenAttributes[$key] = $value;
            }
        }
        return $screenAttributes;
    }

    public function renderVoicemailStartMenu()
    {
        $URI = $this->getVoicemailURI();

        $changedAttributes = array(
            'Selection' => $URI . '?selection=',
        );
        $screenAttributes = $this->getScreenAttributes($changedAttributes);
        $screen = new XmlMenuScreen($screenAttributes);

        $screen->setTitle(__("XML_VOICEMAIL_APP_MENU_START_TITLE") . ": $this->vmTotalCount");

        $screen->addMenuItem(__('XML_VOICEMAIL_APP_NEW_MESSAGES') . ": $this->vmNewCount", $URI, "vmTyp=new");
        $screen->addMenuItem(__('XML_VOICEMAIL_APP_OLD_MESSAGES') . ": $this->vmOldCount", $URI, "vmTyp=old");

        $screen->addButton("1", __('XML_GLOBAL_EXIT_BTN'), 'SoftKey:Cancel');
        $screen->addButton("4", __('XML_GLOBAL_SELECT_BTN'));

        $screen->printScreen();
    }

    public function renderSelectedVoicemailFolder($selectedVoicemails, $voicemailType)
    {
        $URI = $this->getVoicemailURI();
        $currentOffset = $this->offset;
        $currentPage = $this->page;
        if ($currentOffset == 0 || $currentPage == 0) {
            $currentPage = 1;
        }
        $changedAttributes = array(
            'Selection' => $URI . '?selection=',
            'Next' => $this->getNextVoicemails($currentOffset, $currentPage),
            'Previous' => $URI
        );
        $screenAttributes = $this->getScreenAttributes($changedAttributes);

        $screen = new XmlMenuScreen($screenAttributes);

        $title = 'XML_VOICEMAIL_APP_NEW_MESSAGE_TITLE';
        $vmCount = $this->vmNewCount;
        if ($voicemailType === 'old') {
            $title = 'XML_VOICEMAIL_APP_OLD_MESSAGE_TITLE';
            $vmCount = $this->vmOldCount;
        }
        $totalPage = $this->getTotalPage($vmCount);

        $fullTitle = __($title, true, 'messages', array("{count}" => $vmCount));
        $titleWithPage = $fullTitle;

        foreach ($selectedVoicemails as $voicemail) {
            $vmID = $voicemail->id;
            $dateTime = $this->getVoicemailDateTime($vmID);

            $date = PhonalisaDate::userDate($dateTime);
            $time = PhonalisaDate::userTime($dateTime);

            $extension = $voicemail->from;
            $screen->addMenuItem("$date $time $extension", $URI, "vmTyp=$voicemailType&vmID=$vmID", $this->getUCItemDialToListenNumber($vmID));
        }

        if ($currentPage != $totalPage) {
            $titleWithPage = $fullTitle . "   $currentPage/$totalPage";
            $screen->addButton("3", __("-->"), 'SoftKey:Next');
        }

        if ($currentPage > 1) {
            if ($this->offset < $this->vmTotalCount) {
                $titleWithPage = $fullTitle . "   $currentPage/$totalPage";
                $screen->addButton("2", __("<--"), 'SoftKey:Back');
            }
        }
        $screen->addButton("1", __("XML_GLOBAL_BACK_BTN"), 'SoftKey:Previous');
        $screen->addButton("4", __("XML_VOICEMAIL_APP_DETAILS_BTN"));

        $screen->setTitle($titleWithPage);
        $screen->printScreen();
    }

    public function renderVoicemailDetails($voicemail)
    {
        $dialNumber = $this->getUCItemDialToListenNumber();

        $URI = $this->getVoicemailTypURI();
        $vmType = '';
        if (!empty($_REQUEST['vmTyp'])) {
            $vmType = '&vmTyp=' . $_REQUEST['vmTyp'];
        }
        $changedAttributes = array(
            'Next' => $this->getDeleteVoicemailURI($voicemail->id),
            'Previous' => $URI . $vmType
        );
        $screenAttributes = $this->getScreenAttributes($changedAttributes);

        $dateTime = $this->getVoicemailDateTime($voicemail->id);

        $date = PhonalisaDate::userDate($dateTime);
        $time = PhonalisaDate::userTime($dateTime, true);

        $extension = $voicemail->from;

        $dateTranslation = __("XML_VOICEMAIL_APP_DATE");
        $timeTranslation = __("XML_VOICEMAIL_APP_TIME");
        $extensionTranslation = __("XML_VOICEMAIL_APP_EXTENSION");
        $nameTranslation = __('XML_VOICEMAIL_APP_CALLER_NAME');

        $screen = new XmlMenuScreen($screenAttributes);

        $screen->setTitle(__('XML_VOICEMAIL_APP_VOICEMAIL_DETAIL_TITLE'));

        $screen->addMenuItem("$dateTranslation: $date", '', "", $dialNumber);
        $screen->addMenuItem("$timeTranslation: $time", '', "", $dialNumber);
        $screen->addMenuItem("$extensionTranslation: $extension", '', "", $dialNumber);

        if (is_numeric($extension)) {
            $name = $this->getNameFromExtension($extension);
            if ($name != '') {
                $screen->addMenuItem("$nameTranslation: $name", '', "", $dialNumber);
            }
        }

        $screen->addButton("1", __("XML_GLOBAL_BACK_BTN"), 'SoftKey:Previous');
        $screen->addButton("3", __("XML_GLOBAL_DELETE_BTN"), 'SoftKey:Next');
        $screen->addButton("4", __("XML_VOICEMAIL_APP_DIAL_BTN"), 'SoftKey:Dial');

        $screen->printScreen();
    }

    public function renderVoicemailRequestMenu($useCase)
    {
        $URI = $this->getVoicemailDetailURI();
        $changedAttributes = array(
            'Next' => $URI . "&deleteVm=true",
            'Previous' => $URI
        );

        $screenAttributes = $this->getScreenAttributes($changedAttributes);

        $screen = new XmlMenuScreen($screenAttributes);

        $screen->addButton("1", __("XML_VOICEMAIL_APP_NEGATE_REQUEST_BTN"), "SoftKey:Previous");

        if ($useCase === 'delete') {
            $screen->addMenuItem(__("XML_VOICEMAIL_APP_DELETE_CONFIRM_MESSAGE"), '', '');
            $dialCmd = "SoftKey:Next";
        } else {
            $screen->addMenuItem(__("XML_VOICEMAIL_APP_DIAL_CONFIRM_MESSAGE"), '', '', $this->getUCItemDialToListenNumber());
            $dialCmd = "SoftKey:Dial";
        }
        $screen->addButton("4", __("XML_VOICEMAIL_APP_CONFIRM_REQUEST_BTN"), $dialCmd);
        $screen->printScreen();
    }

    public function renderNoEntryFound()
    {
        $changedAttributes = array(
            'Previous' => $this->getVoicemailURI()
        );
        $screenAttributes = $this->getScreenAttributes($changedAttributes);
        $screen = new XmlMenuScreen($screenAttributes);
        $screen->addMenuItem(__("XML_VOICEMAIL_APP_NO_MESSAGES_FOUND"), '', '');
        $screen->addButton("1", __("XML_GLOBAL_BACK_BTN"), 'SoftKey:Previous');
        $screen->printScreen();
    }

    public function renderVoicemailDeleted()
    {
        $changedAttributes = array(
            'Previous' => $this->getVoicemailTypURI()
        );
        $screenAttributes = $this->getScreenAttributes($changedAttributes);
        $screen = new XmlMenuScreen($screenAttributes);
        $screen->addMenuItem(__("XML_VOICEMAIL_APP_MESSAGE_DELETED_INFO"), '', '');
        $screen->addButton("1", __("XML_GLOBAL_BACK_BTN"), 'SoftKey:Previous');

        $screen->printScreen();
    }
}
