<?php

class DefaultController extends Controller
{
        /**
         * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
         * using two-column layout. See 'protected/views/layouts/column2.php'.
         */
        public $layout='//layouts/column2';

        /**
         * @return array action filters
         */
        public function filters()
        {
                return array(
                        'accessControl', // perform access control for CRUD operations
                );
        }

        /**
         * Specifies the access control rules.
         * This method is used by the 'accessControl' filter.
         * @return array access control rules
         */
        public function accessRules()
        {
                return array(               
                        array('allow',  // allow all users to perform 'index' and 'view' actions
                                'actions'=>array('index','view','test'),
                                //'users'=>array('@'),
                        		'expression'=>AccessManager::checkSessionPermission($this->module->name),
                        ),
                        array('allow', // allow authenticated user to perform 'create' and 'update' actions
                                'actions'=>array('create','update'),
                                //'users'=>array('@'),
                        		'expression'=>AccessManager::checkSessionPermission($this->module->name),
                        ),
                        array('allow', // allow admin user to perform 'admin' and 'delete' actions
                                'actions'=>array('delete'),
                                //'users'=>array('@'),
                        		'expression'=>AccessManager::checkSessionPermission($this->module->name),
                        ),
                        array('deny',  // deny all users
                                'users'=>array('*'),
                        ),
                );
        }
        
        public function checkPostedUserIds($post) {
            $postedModerators = array();
            foreach ($post as $key => $value) {
                if (is_array($value)) {
                    $postedModerators[$value['userId']] = $value;
                }
            }
            return $postedModerators;
        }
        
        public function getDefinedModerators($postedModerators,$modelId){
            $definedModerators = array();
            $conferenceModerators = ConferenceModerator::model()->findAllByAttributes(array('conferenceId'=>$modelId));
            foreach ($conferenceModerators as $moderator) {
                $definedModerators[$moderator->id] = $moderator;
            }
            return $definedModerators;
        }
        
        public function saveModerators($modelId) {

        $postedAttributes = array();
        $oldParams = array();
        $postedIds = array();
        foreach ( ConferenceModerator::model()->findAllByAttributes(array('conferenceId'=>$modelId)) as $tier) { 
            $oldParams[$tier->id] = $tier;
        }


        if (isset($_POST['ConferenceModerator'])) {
            
            foreach ($_POST['ConferenceModerator'] as $key => $post) {
             
                if (isset($post['userId'])) {
                    if (!is_array($post) || $post['userId'] == '') {
                        continue;
                    }
                    $postedAttributes[] = $post;
                }
            }
            foreach ($postedAttributes as $attributes) {
                if ($attributes['dataId'] == '') {
                    $tier = new ConferenceModerator();
                } else {
                    $tier = $oldParams[$attributes['dataId']];
                    $postedIds[] =$attributes['dataId']; 
                }
                $tier->attributes = $attributes;
                $tier->conferenceId = $modelId;
                if (!$tier->save()) {
                    dump($tier->errors);
                    die('error, could not save Tier model');
                }
            }
            foreach ($oldParams as $id => $tier) {
                if (!in_Array($tier->id,$postedIds)) {
                    $tier->delete();
                }
            }
        
            
        } else {
            if (!empty($oldParams)){
                foreach ($oldParams as $id => $tier) {
                    $tier->delete();
                }
            }
        }
    }
        public function actionCreate(){
            $model = new Conference();
            $this->performAjaxValidation($model);
            if (isset($_POST['Conference'])) {
                
                $model->attributes = $_POST['Conference'];
                if (!$model->save()) {
                    dump($model->errors);
                } else {
                    if (isset($_POST['ConferenceModerator'])) {
                        $this->saveModerators($model->id,$_POST['ConferenceModerator']);
                    } 
                }
            }
            $this->renderPartial('_form',array('model'=>$model),false,true);
        }

        /**
         * Updates a particular model.
         * If update is successful, the browser will be redirected to the 'view' page.
         * @param integer $id the ID of the model to be updated
         */
        public function actionUpdate($id) {

            if(@$_POST['deleteRequest'] == 'true' && !isset($_POST['ajax'])){
                $this->actionDelete($id);
            }   

            $model = $this->loadModel($id);
            $this->performAjaxValidation($model);
            if (isset($_POST['Conference'])) {
                $model->attributes = $_POST['Conference'];
                if (!$model->save()) {
                    dump($model->errors);
                } else {
                    if (isset($_POST['ConferenceModerator'])) {
                        $this->saveModerators($model->id,$_POST['ConferenceModerator']);
                    }
                }
            }
            $this->renderPartial('_form',array('model'=>$model),false,true);
            
        }

        protected function getModeratorAttributeValues($post) {
            if ($post == '') {
                return array(
                    'moderator'=>null,
                    'moderatorType'=>TargetType::undefined,
                );
            } else {
                $moderatorData = explode('_',$post);
                return array(
                    'moderator'=>$moderatorData[0],
                    'moderatorType'=>$moderatorData[1],
                );
            }
        }

        /**
         * Deletes a particular model.
         * If deletion is successful, the browser will be redirected to the 'admin' page.
         * @param integer $id the ID of the model to be deleted
         */
        public function actionDelete($id) {
// 
            if(Yii::app()->request->isPostRequest)
            {
                // we only allow deletion via POST request
                $this->loadModel($id)->delete();

                // if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
                if(!isset($_GET['ajax'])) {
                    $this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('index'));
                }
            } else {
                throw new CHttpException(400,'Invalid request. Please do not repeat this request again.');
            }
        }

        /**
         * Lists all models.
         */
        public function actionIndex() {      
            
            $model = new Conference('search');        
            if(isset($_GET['Conference']))
                $model->attributes = $_GET['Conference'];
            
            $this->render('index', array('model'=>$model));
        }

     
        /**
         * Returns the data model based on the primary key given in the GET variable.
         * If the data model is not found, an HTTP exception will be raised.
         * @param integer the ID of the model to be loaded
         */
        public function loadModel($id) {
                $model=Conference::model()->findByPk($id);
                if($model===null){
                        throw new CHttpException(404,'The requested page does not exist.');
                }
                return $model;
        }

        /**
         * Performs the AJAX validation.
         * @param CModel the model to be validated
         */
        protected function performAjaxValidation($model)
        {
                if(isset($_POST['ajax']) && $_POST['ajax']==='conference-form')
                {        
                        echo CActiveForm::validate($model);
                        Yii::app()->end();
                }
        }

        public function getTabularFormTabs($form, $model){

            $tabs = array();
        $count = 0;
        $translateKeys = array(
                                'generalData'=>__('CONFERENCE_TAB_MENU_GENERAL_DATA'),
                                'moderatorGrid'=>__('CONFERENCE_TAB_MENU_MODERATORS'),
                );
        foreach ($translateKeys as $view => $tabLabel){
            $tabs[] = array(
                'active'=>$count++ === 0,
                'label'=>$tabLabel,
                'content'=>$this->renderPartialWithContextHelp($view, array('form'=>$form, 'model'=>$model), true),
            );
        }
        return $tabs;

        }

        public function renderModeratorGridInput($data,$row,$column) {
            $this->renderPartial('moderatorGridInput',array('data'=>$data,'column'=>$column,'row'=>$row));
        }
}
