<?php

class DefaultController extends Controller
{

    /**
     * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
     * using two-column layout. See 'protected/views/layouts/column2.php'.
     */
    public $layout = '//layouts/column2';

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(
            array('allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('index', 'view', 'export', 'addTemplateSettings', 'showTemplate'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array('create', 'update'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('allow', // allow admin user to perform 'admin' and 'delete' actions
                'actions' => array('admin', 'delete'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    /**
     * Displays a particular model.
     * @param integer $id the ID of the model to be displayed
     */
    public function actionView($id)
    {
        $this->render('view', array(
            'model' => $this->loadModel($id),
        ));
    }

    /**
     * Creates a new model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     */
    public function actionCreate()
    {
        $model = new Gateway;
        $model->intl_prefix = cfg('DP_INTL_PREFIX');
        // Uncomment the following line if AJAX validation is needed
        $this->performAjaxValidation($model);
        if (isset($_POST['Gateway'])) {
            $model->attributes = $this->getDataForModelAttributes();
            $model = $this->processProviderTemplateToModel($model);
            if (!$model->save()) {
                throw new CHttpException(500, 'Can not save new data.');
            }
        }
        $this->renderPartial('_form', array('model' => $model), false, true);
    }

    protected function getDataForModelAttributes()
    {
        $data = $this->getFormatedModelAttributes($_POST['Gateway']);
        if (PhonalisaLicenseManager::routingBlacklistEnabled()) {
            $inboundListID = $data['inboundBlacklistId'];
            $outboundListID = $data['outboundBlacklistId'];
            $data['inboundBlacklistId'] = '';
            $data['outboundBlacklistId'] = '';
            $data['inboundWhitelistId'] = '';
            $data['outboundWhitelistId'] = '';
            $data = $this->getFormatedBlacklistWhitelistID($data, $inboundListID, 'inbound');
            $data = $this->getFormatedBlacklistWhitelistID($data, $outboundListID, 'outbound');
        }
        return $data;
    }

    protected function getFormatedModelAttributes($attributes)
    {
        $attributesForTrim = array(
            'name',
            'host',
            'proxy',
            'register_proxy',
            'realm',
            'from_user',
            'from_domain'
        );

        foreach ($attributes as $attributeKey => $value) {
            if (in_array($attributeKey, $attributesForTrim)) {
                $attributes[$attributeKey] = trim($value);
            }
        }
        return $attributes;
    }

    protected function getFormatedBlacklistWhitelistID($data, $listID, $listType)
    {
        $listKey = 'whitelist';
        if (strpos($listID, 'blacklist_') !== false) {
            $listKey = 'blacklist';
        }
        $data[$listType . ucfirst($listKey) . 'Id'] = str_replace($listKey . '_', '', $listID);
        return $data;
    }

    public function actionShowTemplate($id)
    {
        $model = $this->loadModel($id);
        if ($model == null) {
            die("model not found.");
        }

        $blacklist = array("csvImported", "hasAdditionalData", "debug",
            "providerTemplate", "dialOutToggle", "state", "active");

        echo "[" . $model->providerTemplate . "]<br>";
        foreach ($model->attributeNames() as $key) {
            if (in_array($key, $blacklist)) {
                continue;
            }
            if ($model->$key != "") {
                echo $key . " = " . CHtml::encode($model->$key) . "<br>";
            }
        }
    }

    protected function processProviderTemplateToModel($model)
    {
        $providerTemplate = $model->providerTemplate;
        if (!empty($providerTemplate)) {
            $templateData = $model->getTemplateSettingsFromProviderIni($providerTemplate);
            foreach ($this->getDataForModelAttributes() as $key => $value) {
                if ($key === 'name' || $key === 'host' || $key === 'typ' || $key === 'providerTemplate') {
                    $model[$key] = $value;
                    continue;
                }
                if (isset($templateData[$key])) {
                    if ($model[$key] !== $value && $value === $templateData[$key]) {
                        $model[$key] = '';
                        continue;
                    }
                    if ($value !== $templateData[$key]) {
                        $model[$key] = $value;
                    }
                }
            }
        } else {
            $model->attributes = $this->getDataForModelAttributes();
        }
        return $model;
    }

    /**
     * Updates a particular model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id the ID of the model to be updated
     */
    public function actionUpdate($id)
    {
        $model = $this->loadModel($id);
        $model = $this->changeBlacklistWhitelistSettings($model);
        // Uncomment the following line if AJAX validation is needed
        $this->performAjaxValidation($model);

        if (isset($_POST['Gateway'])) {
            $model->attributes = $this->getDataForModelAttributes();
            $model = $this->processProviderTemplateToModel($model);
            if (!$model->save()) {
                throw new CHttpException(500, 'Can not save new data.');
            }
        }
        $this->renderPartial('_form', array('model' => $model), false, true);
    }

    protected function changeBlacklistWhitelistSettings($model)
    {
        if ($model->inboundBlacklistId !== '') {
            $model->inboundBlacklistId = 'blacklist_' . $model->inboundBlacklistId;
        }
        if ($model->outboundBlacklistId !== '') {
            $model->outboundBlacklistId = 'blacklist_' . $model->outboundBlacklistId;
        }
        if ($model->inboundWhitelistId !== '') {
            $model->inboundBlacklistId = 'whitelist_' . $model->inboundWhitelistId;
            $model->inboundWhitelistId = '';
        }
        if ($model->outboundWhitelistId !== '') {
            $model->outboundBlacklistId = 'whitelist_' . $model->outboundWhitelistId;
            $model->outboundWhitelistId = '';
        }
        return $model;
    }

    /**
     * Deletes a particular model.
     * If deletion is successful, the browser will be redirected to the 'admin' page.
     * @param integer $id the ID of the model to be deleted
     */
    public function actionDelete($id)
    {
        if (Yii::app()->request->isPostRequest) {
            // we only allow deletion via POST request
            $this->loadModel($id)->delete();
            // if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
            if (!isset($_GET['ajax'])) {
                $this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('admin'));
            }
        } else {
            throw new CHttpException(400, 'Invalid request. Please do not repeat this request again.');
        }
    }

    /**
     * Lists all models.
     */
    public function actionIndex()
    {
        /*
          $dataProvider=new CPhonalisaDataProvider('Gateway');
          $this->render('index',array(
          'dataProvider'=>$dataProvider,
          ));
         */
        $this->actionAdmin();
    }

    /**
     * Manages all models.
     */
    public function actionAdmin()
    {
        $model = new Gateway('search');
        $model->unsetAttributes();  // clear any default values
        if (isset($_GET['Gateway'])) {
            $model->attributes = $_GET['Gateway'];
        }
        $this->render('admin', array(
            'model' => $model,
        ));
    }

    /**
     * Returns the data model based on the primary key given in the GET variable.
     * If the data model is not found, an HTTP exception will be raised.
     * @param integer the ID of the model to be loaded
     */
    public function loadModel($id)
    {
        $model = Gateway::model()->findByPk($id);
        if ($model === null) {
            throw new CHttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

    /**
     * Performs the AJAX validation.
     * @param CModel the model to be validated
     */
    protected function performAjaxValidation($model)
    {
        if (isset($_POST['ajax']) && $_POST['ajax'] === 'gateway-form') {
            echo CActiveForm::validate($model);
            Yii::app()->end();
        }
    }

    public function getTabularFormTabs($form, $model)
    {
        $tabs = array();
        $count = 0;
        $translateKeys = array(
            'generalData' => __('GATEWAY_MODEL_TAB_MENU_GENERAL_DATA'),
            'limitations' => __('GATEWAY_MODEL_TAB_MENU_LIMITATIONS'),
            'additionalData' => __('GATEWAY_MODEL_TAB_MENU_ADDITIONAL_DATA'),
        );

        $tabIndex = 1;
        foreach ($translateKeys as $view => $tabLabel) {
            $tabs[] = array(
                'active' => $count++ === 0,
                'label' => $tabLabel,
                'content' => $this->renderPartialWithContextHelp($view, array('form' => $form, 'model' => $model, "tabIndex" => $tabIndex), true),
            );
            $tabIndex ++;
        }
        return $tabs;
    }

    public function actionExport()
    {
        ExportHelper::exportModelAsCSV('Gateway');
    }

    public function actionAddTemplateSettings($providerTemplate = '')
    {
        $templateData = array();
        if ($providerTemplate) {
            $templateData = Gateway::model()->getTemplateSettingsFromProviderIni($providerTemplate);
            foreach ($templateData as $key => $value) {

                if (strpos($key, "_label")) {
                    $templateData[$key] = $value;
                    $templateData[$key . "_translated"] = __($value);
                }
            }
        }
        
        $data["providerTemplate"] = $providerTemplate;
        $data["data"] = $templateData;
        echo json_encode($data);
        Yii::app()->end();
    }

    public function getBlacklistData()
    {
        $blacklist = array();
        foreach (CHtml::listData(Blacklist::model()->find(array(), 'name'), 'id', 'name') as $blacklistID => $blacklistName) {
            $blacklist['blacklist_' . $blacklistID] = $blacklistName;
        }

        $whitelist = array();
        foreach (CHtml::listData(Whitelist::model()->find(array(), 'name'), 'id', 'name') as $whitelistID => $whitelistName) {
            $whitelist['whitelist_' . $whitelistID] = $whitelistName;
        }

        return array(
            '' => '---',
            __('CHOOSE_BLACKLIST') => $blacklist,
            __('CHOOSE_WHITELIST') => $whitelist
        );
    }

    public function renderProviderCell($data)
    {
        $providerListData = $data->getProviderChoose();
        if (!isset($providerListData[$data->providerTemplate]) || $data->providerTemplate === "default") {
            return $data->host;
        }

        return $providerListData[$data->providerTemplate];
    }
}
