<?php

class DefaultController extends Controller
{

    /**
     * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
     * using two-column layout. See 'protected/views/layouts/column2.php'.
     */
    public $layout = '//layouts/column2';
    private $dateTimeProfileCache = null;
    private $gatewayCache = null;
    private $userGroupsCache = null;

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(
            array('allow', // allow admin user to perform 'admin' and 'delete' actions
                'actions' => array('index', 'delete', 'ajaxValidate', 'select', 'export'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function actionSelect()
    {
        if (isset($_POST['gw_id'])) {
            $this->redirect(array('admin', 'gw_id' => $_POST['gw_id']));
        } else {
            // Redirect user gateway creating page if no gateways exists
            $this->redirect(array('/routingGateways/'));
        }
    }

    public function actionIndex()
    {
        $gateways = $this->getGateways();
        if (empty($gateways)) {
            $this->render('_selectGwform');
        } else {
            $this->actionAdmin();
        }
    }

    private function getGateways()
    {
        if ($this->gatewayCache === null) {
            $this->gatewayCache = Gateway::model()->findAll(array());
        }
        return $this->gatewayCache;
    }

    /**
     * Manages all models.
     */
    public function actionAdmin()
    {
        $model = new RoutingOutbound('search');
        if (isset($_POST['RoutingOutbound'])) {
            foreach ($_POST['RoutingOutbound'] as $id => $post) {
                $this->updateRoutingRules($id, $post);
            }
        }
        $this->render('admin', array('model' => $model));
    }

    public function updateRoutingRules($id, $post)
    {
        $model = RoutingOutbound::model()->findByPk($id);
        if (!$model) {
            $model = new RoutingOutbound();
        }
        $model->attributes = $this->getAttributeValues($id, $post);
        $model->overwriteCallerId = trim($model->overwriteCallerId);

        if ($model->save()) {
            return true;
        } else {
            return false;
        }
    }

    public function getAttributeValues($id, $attributes)
    {
        if (!$this->getTargetValues($attributes)) {
            $attributes['target'] = null; // $attributes['customNumber'];
            $attributes['targetType'] = 2; //TargetType::Manuel;
        }
        if ($attributes['sort'] == 'new') {
            $attributes['sort'] = RoutingOutbound::model()->count();
        }
        return $attributes;
    }

    public function getTargetValues(&$postedTarget)
    {
        $targets = explode('_', $postedTarget['usergroup']);

        if (isset($targets[1])) {
            $postedTarget['targetType'] = $targets[0];
            $postedTarget['usergroup'] = $targets[1];
        } else {
            return false;
        }
        return $postedTarget;
    }

    public function actionDelete($id)
    {
        if (Yii::app()->request->isPostRequest) {
            // we only allow deletion via POST request
            $model = RoutingOutbound::model()->findByPk($id);
            $model->delete();

            // if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
            if (!isset($_GET['ajax'])) {
                $this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('admin'));
            }
        } else {
            throw new CHttpException(400, 'Invalid request. Please do not repeat this request again.');
        }
    }

    public function renderDateTimeProfile(PhonalisaRecord $data, $row, $column)
    {
        $listData = $this->getDateTimeProfileListData();
        $this->renderPartial('dateTimeProfileDropdown', array('listData' => $listData, 'data' => $data));
    }

    private function getDateTimeProfileListData()
    {
        $listData = array('' => '---');
        $listData = array_merge($listData, CHtml::listData($this->getDateTimeProfiles(), 'id', 'title'));
        return $listData;
    }

    private function getDateTimeProfiles()
    {
        if ($this->dateTimeProfileCache === null) {
            $this->dateTimeProfileCache = DateTimeProfile::model()->find(array(), 'title');
        }
        return $this->dateTimeProfileCache;
    }

    public function actionAjaxValidate()
    {
        $model = new RoutingOutbound();
        $model->gw_id = 'validate';
        $errors = array();
        foreach ($_REQUEST['RoutingOutbound'] as $modelId => $attributes) {
            $model->attributes = $this->getAttributeValues($modelId, $attributes);
            $sort = $attributes['sort'];
            if ($modelId == 'new') {
                $model->sort = 0;
            }

            $validateOutput = CActiveForm::validate($model);
            $decodetOutput = (array) json_decode($validateOutput);
            foreach ($decodetOutput as $key => $value) {
                //overwrite inputField Id
                $newKey = '';
                foreach (explode('_', $key) as $index => $part) {
                    if ($index == 0) {
                        $newKey .= $part . '_' . $modelId;
                    } else {
                        $newKey.='_' . $part;
                    }
                }
                $errors[$modelId][$newKey] = $value;
            }
        }

        if (isset($errors['new']['RoutingOutbound_new_pattern'])) {
            unset($errors['new']['RoutingOutbound_new_pattern']);
        }
        if (isset($errors['new']) && empty($errors['new'])) {
            unset($errors['new']);
        }
        if (!empty($errors)) {
            echo json_encode($errors);
        }
        Yii::app()->end();
    }

    public function renderGatewayCell($data, $row, $cell)
    {
        $gateways = $this->getGateways();
        $sortedGateways = $this->sortGateways($gateways);
        $this->renderPartial("gatewayCell", array(
            "data" => $data,
            "row" => $row,
            "cell" => $cell,
            "activeModels" => $sortedGateways["active"],
            "disabledModels" => $sortedGateways["disabled"],
        ));
    }

    private function sortGateways($gateways)
    {
        $sortedGateways["active"] = array();
        $sortedGateways["disabled"] = array();
        foreach ($gateways as $gateway) {
            if ($gateway->active === "1") {
                $sortedGateways["active"][] = $gateway;
            } else {
                $sortedGateways["disabled"][] = $gateway;
            }
        }
        return $sortedGateways;
    }

    public function renderActiveCell($data)
    {
        $this->renderPartial("activeCell", array("data" => $data));
    }

    public function renderUserGroupCell($data)
    {
        $listData = $this->getUserGroups();
        $this->renderPartial(
                "userGroupCell", array(
            "listData" => $listData,
            "data" => $data
                )
        );
    }

    private function getUserGroups()
    {
        if ($this->userGroupsCache === null) {
            $targetHelper = new TargetHelper();
            $listData = array_merge(array('' => '---'), $targetHelper->getTargetList(array(TargetType::User, TargetType::Group)));
            $this->userGroupsCache = $listData;
        }
        return $this->userGroupsCache;
    }

    public function renderPatternCell($data)
    {
        $this->renderPartial("patternCell", array("data" => $data));
    }

    public function renderOverwriteCallerIdCell($data)
    {
        if ($data->overwriteCallerId != '') {
            $style = '';
        } else {
            $style = "display:none";
        }
        $checked = $data->overwriteCallerId !== NULL && $data->overwriteCallerId !== '';
        $this->renderPartial("overwriteCallerIdCell", array("data" => $data, "style" => $style, "checked" => $checked));
    }

    public function renderLcrPrefix(PhonalisaRecord $data)
    {
        $this->renderPartial('lcrPreFixInput', array('data' => $data));
    }

    public function renderhiddenSortCell($data)
    {
        $this->renderPartial("hiddenSortCell", array("data" => $data));
    }

    public function renderDescCell($data)
    {
        $this->renderPartial("descCell", array("data" => $data));
    }

    public function actionExport()
    {
        ExportHelper::exportModelAsCSV('RoutingOutbound');
    }
}
