<?php

class DefaultController extends Controller
{

    /**
     * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
     * using two-column layout. See 'protected/views/layouts/column2.php'.
     */
    public $layout = '//layouts/column2';

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(
            array('allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('index'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array('create', 'update'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('allow', // allow admin user to perform 'admin' and 'delete' actions
                'actions' => array('admin', 'delete'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    /**
     * Creates a new model.
     * If creation is successful, the browser will be redirected to the 'admin' page.
     */
    public function actionCreate()
    {
        $model = new Queue;
        $this->performAjaxValidation($model);
        $tier = new Tiers();

        if (isset($_POST['Queue'])) {
            $model->attributes = $_POST['Queue'];
            $model->attributes = $this->processManualTarget($model);
            TargetHelper::setTargetAttributes('target_type_on_timeout', 'target_on_timeout', $model);
            TargetHelper::setTargetAttributes('target_type_on_no_agents', 'target_on_no_agents', $model);
            TargetHelper::setTargetAttributes('target_type_on_full', 'target_on_full', $model);

            if ($model->save()) {
                $this->saveMembers($model);
                $this->saveExternalAgents($model);
                Yii::app()->end();
            }
        }

        // for footer input elements
        $tier->static_member = '1';

        $this->renderPartial(
                '_form', array(
            'model' => $model,
            'tier' => $tier,
            'availableUsers' => User::model()->findAll(),
            'availableExternalAgents' => $this->getExternalAgents(),
                ), false, true);
    }

    /**
     * Updates a particular model.
     * If update is successful, the browser will be redirected to the 'admin' page.
     * @param integer $id the ID of the model to be updated
     */
    public function actionUpdate($id)
    {
        if (@$_POST['deleteRequest'] == 'true' && !isset($_POST['ajax'])) {
            $this->actionDelete($id);
        }
        $model = $this->loadModel($id);
        $tier = new Tiers();
        $members = $model->getQueueMembers();

        $availableUsers = User::model()->find(array(), 'firstname');
        foreach ($availableUsers as $key => $user) {
            if (array_key_exists($user->id, $members)) {
                unset($availableUsers[$key]);
            }
        }

        //         Uncomment the following line if AJAX validation is needed
        $this->performAjaxValidation($model);

        if (isset($_POST['Queue'])) {
            $model->attributes = $_POST['Queue'];
            $model->attributes = $this->processManualTarget($model);
            TargetHelper::setTargetAttributes('target_type_on_timeout', 'target_on_timeout', $model);
            TargetHelper::setTargetAttributes('target_type_on_no_agents', 'target_on_no_agents', $model);
            TargetHelper::setTargetAttributes('target_type_on_full', 'target_on_full', $model);

            if ($model->save()) {
                $this->saveMembers($model);
                $this->saveExternalAgents($model);
                $this->redirect(array('admin', 'id' => $model->id));
            }
        }
        $availableExternalAgents = $this->getExternalAgents();
        $members = $model->getExternalAgents();

        foreach ($availableExternalAgents as $key => $user) {

            if (array_key_exists($user->id, $members)) {
                unset($availableExternalAgents[$key]);
            }
        }

        // for footer input elements
        $tier->static_member = '1';

        $this->renderPartial(
                '_form', array(
            'model' => $model,
            'tier' => $tier,
            'availableUsers' => $availableUsers,
            'availableExternalAgents' => $availableExternalAgents
                ), false, true);
    }

    protected function getExternalAgents()
    {
        $routingExternalAgentsMax = PhonalisaLicenseManager::routingExternalAgentsMax();
        if ((int) $routingExternalAgentsMax >= 0) {
            return ExternalAgent::model()->find(array(), 'id', $routingExternalAgentsMax);
        }
        return ExternalAgent::model()->findAll();
    }

    /**
     * Deletes a particular model.
     * If deletion is successful, the browser will be redirected to the 'admin' page.
     * @param integer $id the ID of the model to be deleted
     */
    public function actionDelete($id)
    {

        if (Yii::app()->request->isPostRequest) {

            //we only allow deletion via POST request
            if ($this->loadModel($id)->delete()) {

                $permissions = Permissions::model()->findAllByAttributes(array('itemId' => $id));
                while (count($permissions)) {

                    foreach ($permissions as $permission) {

                        $permission->delete();
                    }
                    $permissions = Permissions::model()->findAllByAttributes(array('itemId' => $id));
                }
                $permission = new Permissions();
                $permission->createPermissionsFile();
            }

            //if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
            if (!isset($_GET['ajax'])) {

                $this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('admin'));
            }
        } else {

            throw new CHttpException(400, 'Invalid request. Please do not repeat this request again.');
        }
    }

    /**
     * Lists all models.
     */
    public function actionIndex()
    {
        $this->actionAdmin();
    }

    /**
     * Manages all models.
     */
    public function actionAdmin()
    {

        $model = new Queue('search');
        $model->unsetAttributes();  // clear any default values

        if (isset($_GET['Queue']))
            $model->attributes = $_GET['Queue'];

        $this->render('admin', array(
            'model' => $model,
        ));
    }

    /**
     * Returns the data model based on the primary key given in the GET variable.
     * If the data model is not found, an HTTP exception will be raised.
     * @param integer the ID of the model to be loaded
     */
    public function loadModel($id)
    {
        $model = Queue::model()->findByPk($id);
        if ($model === null) {
            throw new CHttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

    /**
     * Performs the AJAX validation.
     * @param CModel the model to be validated
     */
    protected function performAjaxValidation($model)
    {
        if (isset($_POST['ajax']) && $_POST['ajax'] === 'routing_queue-form') {
            echo CActiveForm::validate($model);
            Yii::app()->end();
        }
    }

    public function getTabularFormTabs($form, $model, $tier, $availableUsers, $availableExternalAgents)
    {
        $tabs = array();
        $count = 0;
        $externalAgents = (int) ExternalAgent::model()->countAllByAttributes(array());

        $tranlateKeys["generalData"] = __('QUEUE_MODEL_TAB_MENU_GENERAL_DATA');
        $tranlateKeys["queueUsers"] = __('QUEUE_MODEL_TAB_MENU_USERS');
        if (PhonalisaLicenseManager::routingExternalAgentEnabled()) {
            $tranlateKeys["externalAgents"] = __('QUEUE_MODEL_TAB_MENU_EXTERNAL_AGENTS');
        }
        $tranlateKeys["additionalData"] = __('QUEUE_MODEL_TAB_MENU_ADDITIONAL_DATA');

        foreach ($tranlateKeys as $view => $tabLabel) {
            if ($view == 'queueUsers' && $model->isNewRecord) {
                $content = $this->renderPartial('notFound', array('text' => 'QUEUE_MODEL_TAB_QUEUE_MEMBERS_CAN_BE_ADDED_LATER'), true);
            } else if ($view == 'externalAgents' && $model->isNewRecord) {
                $content = $this->renderPartial('notFound', array('text' => 'QUEUE_MODEL_TAB_EXTERNAL_AGENTS_CAN_BE_ADDED_LATER'), true);
            } else if ($view == 'externalAgents' && $externalAgents == 0) {
                $content = $this->renderPartial('notFound', array('text' => 'QUEUE_NO_EXTERNAL_AGENT_FOUND'), true);
            } else {
                $content = $this->renderPartialWithContextHelp(
                        $view, array(
                    'form' => $form,
                    'tier' => $tier,
                    'model' => $model,
                    'availableUsers' => $availableUsers,
                    'availableExternalAgents' => $availableExternalAgents
                        ), true);
            }

            $tabs[] = array(
                'active' => $count++ === 0,
                'label' => $tabLabel,
                'content' => $content,
            );
        }
        return $tabs;
    }

    public function renderUserDropDown($data, $row, $column)
    {
        $availableUsers = User::model()->findAll();
        $this->renderPartial('queueUserInput', array('data' => $data, 'row' => $row, 'column' => $column, 'availableUsers' => $availableUsers));
    }

    public function renderExternalAgentsDropdown($data, $row, $column)
    {

        $availableUsers = ExternalAgent::model()->findAll();
        $this->renderPartial('externalAgentInput', array('data' => $data, 'row' => $row, 'column' => $column, 'availableUsers' => $availableUsers));
    }

    protected function saveExternalAgents($queue)
    {
        if (isset($_POST['ExternalAgent']['footer'])) {
            unset($_POST['ExternalAgent']['footer']);
            if (!count($_POST['ExternalAgent'])) {
                unset($_POST['ExternalAgent']);
            }
        }
        $this->saveTiers(@$_POST['ExternalAgent'], $queue, TiersType::external);
    }

    protected function saveMembers($queue)
    {
        $this->saveTiers(@$_POST['Tiers'], $queue, TiersType::user);
    }

    protected function getOldParams($queue, $tiersType)
    {
        $oldParams = Tiers::model()->findAllByAttributes(array('type' => $tiersType, 'queueId' => $queue->id));
        $oldParamsArray = array();
        foreach ($oldParams as $param) {
            if ($tiersType === TiersType::external) {
                $oldParamsArray[$param->id] = $param;
            } else {
                $oldParamsArray[$param->userId] = $param;
            }
        }
        return $oldParamsArray;
    }

    protected function saveTiers($postData, $queue, $tiersType)
    {
        $oldParams = $this->getOldParams($queue, $tiersType);
        if (isset($postData)) {
            foreach ($postData as $post) {
                if (!isset($post['userId'])) {
                    continue;
                }
                if (array_key_exists($post['userId'], $oldParams)) {
                    $tier = $oldParams[$post['userId']];
                    unset($oldParams[$post['userId']]);
                } else {
                    $tier = new Tiers();
                }
                $tier->attributes = $post;
                $tier->type = $tiersType;
                $tier->queueId = $queue->id;
                $tier->queueExtension = $queue->extension;
                if ($tiersType == TiersType::external) {
                    $tier->static_member = '1';
                }
                $tier->save();
            }
            foreach ($oldParams as $param) {
                $param->delete();
            }
        } else {
            foreach ($oldParams as $param) {
                $param->delete();
            }
        }
    }

    protected function processManualTarget($model)
    {
        $post = $_POST['Queue'];
        if ($post['target_on_timeout'] == TargetType::Manual) {
            $model->target_on_timeout = TargetType::Manual . '_' . $post['target_on_timeout-manual'];
        }
        if ($post['target_on_no_agents'] == TargetType::Manual) {
            $model->target_on_no_agents = TargetType::Manual . '_' . $post['target_on_no_agents-manual'];
        }
        if ($post['target_on_full'] == TargetType::Manual) {
            $model->target_on_full = TargetType::Manual . '_' . $post['target_on_full-manual'];
        }
        return $model;
    }
}
