<?php
class DefaultController extends Controller
{

    /**
     * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
     * using two-column layout. See 'protected/views/layouts/column2.php'.
     */
    public $layout = '//layouts/column2';

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(
            array('allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('index', 'view', 'create', 'update', 'admin', 'delete'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array(
                    'indexPlaylist',
                    'viewPlaylist',
                    'playlist',
                    'createPlaylist',
                    'updatePlaylist',
                    'adminPlaylist',
                    'deletePlaylist',
                    "downloadFile"
                ),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('deny', // deny all users
                'users' => array('*'),
            ),
        );
    }
    /**
     * Displays a particular model.
     * @param integer $id the ID of the model to be displayed
     */
    /*
      public function actionView($id)
      {
      $this->render('view',array(
      'model'=>$this->loadModel($id),
      ));
      }
     */

    /**
     * Creates a new model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     */
    public function actionCreate()
    {
        $model = new MusicOnHold;
        $this->performAjaxValidation($model);

        if (isset($_POST['MusicOnHold'])) {
            $model->attributes = $_POST['MusicOnHold'];
            if ($model->save()) {
                Yii::app()->end();
            }
        }

        $this->renderPartial('_form_moh', array('model' => $model), false, true);
    }

    public function actionCreatePlaylist()
    {
        $model = new AudioFile('create');
        $playlistID = @$_REQUEST['playlistID'];

        $this->performAjaxValidation($model);

        if (isset($_POST['AudioFile'])) {
            $model->attributes = $_POST['AudioFile'];
            $model->validate();
            if ($model->errors) {
                if ($model->errors['data']) {
                    $errorMessage = '';
                    foreach ($model->errors['data'] as $value) {
                        $errorMessage.= $value . "\n";
                    }
                    throw new CHttpException(500, $errorMessage);
                }
            }
            $model->type = AudioFileType::musicOnHold;

            if ($model->save()) {
                Yii::app()->end();
            }

            //$playlistID = $_POST['Playlist']["playlistID"];
        }
        $this->renderPartialWithContextHelp('_form_playlist', array('model' => $model, 'playlistID' => $playlistID), false, true);
    }

    /**
     * Updates a particular model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id the ID of the model to be updated
     */
    public function actionUpdate($id)
    {
        $model = $this->loadModel($id);

//         Uncomment the following line if AJAX validation is needed
        $this->performAjaxValidation($model);

        if (isset($_POST['MusicOnHold'])) {
            $model->attributes = $_POST['MusicOnHold'];
            if ($model->save()) {
                $this->redirect(array('index', 'id' => $model->id));
            }
        }
        $this->renderPartial('_form_moh', array('model' => $model), false, true);
    }

    public function actionUpdatePlaylist($id)
    {
        //audioFile Model
        $model = $this->loadModelPlaylist($id);
        $playlist = MusicOnHold::model()->findByPk($_REQUEST['playlistId']);
        $playlistID = $model->playlistID;

        //         Uncomment the following line if AJAX validation is needed
        $this->performAjaxValidation($model);

        if (isset($_POST['AudioFile'])) {
            $model->attributes = $_POST['AudioFile'];
            if ($model->save()) {
                $this->redirect(array('adminPlaylist', 'playlistId' => $playlist->id));
            }
        }
        $this->renderPartialWithContextHelp('_form_playlist', array('model' => $model, 'playlistID' => $playlistID), false, true);
    }

    public function actionPlaylist()
    {
        $this->actionIndexPlaylist();
    }

    /**
     * Deletes a particular model.
     * If deletion is successful, the browser will be redirected to the 'admin' page.
     * @param integer $id the ID of the model to be deleted
     */
    public function actionDelete($id)
    {
//        dump(isset($_GET['ajax']));
//        die('foo');
        if (Yii::app()->request->isPostRequest) {
            //we only allow deletion via POST request
            $this->loadModel($id)->delete();

            //if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
            if (!isset($_GET['ajax'])) {
                $this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('admin'));
            }
        } else {
            throw new CHttpException(400, 'Invalid request. Please do not repeat this request again.');
        }
    }

    public function actionDeletePlaylist($id)
    {
        if (Yii::app()->request->isPostRequest) {
            //we only allow deletion via POST request
            $this->loadModelPlaylist($id)->delete();

            //if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
            if (!isset($_GET['ajax'])) {
                $this->redirect(url(
                    'systemMusicOnHold/default/adminPlaylist',
                    array(
                        'playlistId' => $_REQUEST['playlistId']
                    )
                ));
            }
        } else {
            throw new CHttpException(400, 'Invalid request. Please do not repeat this request again.');
        }
    }

    /**
     * Lists all models.
     */
    public function actionIndex()
    {
        $this->actionAdmin();
    }

    public function actionIndexPlaylist()
    {
        $this->actionAdminPlaylist();
    }

    /**
     * Manages all models.
     */
    public function actionAdmin()
    {
        $model = new MusicOnHold('search');
        $model->unsetAttributes();  // clear any default values
        app()->user->setFlash('info', '<strong>' . __('MUSIC_ON_HOLD_DELETE_MOH_INFO') . '</strong>');
        if (isset($_GET['MusicOnHold'])) {
            $model->attributes = $_GET['MusicOnHold'];
        }
        $this->render('admin_moh', array(
            'model' => $model,
        ));
    }

    /**
     * Manages all models.
     */
    public function actionAdminPlaylist()
    {
        $model = new AudioFile('search');
        $playlistID = @$_REQUEST['playlistId'];
        $playlist = MusicOnHold::model()->findByPk($playlistID);

        //$model->unsetAttributes();  // clear any default values
        if (isset($_GET['AudioFile'])) {
            $model->attributes = $_GET['AudioFile'];
        }

        $this->render('admin_playlist', array(
            'model' => $model, 'playlist' => $playlist,
        ));
    }
    
    public function actionDownloadFile($id)
    {
        $model = AudioFile::model()->findByPk($id);
        if ($model) {
            $length = 0;
            $data = "";
            $binaryField = "data";
            $model->getBinaryField($binaryField, $data, $length);
            $this->renderPartial('_downloadFile', array(
                'data' => $data,
                'type' => $_GET['type'],
                "name" => $_GET['name']
            ));
        } else {
            throw new CHttpException(404, 'File not found');
        }
    }

    /**
     * Returns the data model based on the primary key given in the GET variable.
     * If the data model is not found, an HTTP exception will be raised.
     * @param integer the ID of the model to be loaded
     */
    public function loadModel($id)
    {
        $model = MusicOnHold::model()->findByPk($id);
        if ($model === null) {
            throw new CHttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

    public function loadModelPlaylist($id)
    {
        $model = AudioFile::model()->findByPk($id);
        if ($model === null) {
            throw new CHttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

    /**
     * Performs the AJAX validation.
     * @param CModel the model to be validated
     */
    protected function performAjaxValidation($model)
    {
        if (isset($_POST['ajax']) && ($_POST['ajax'] === 'music-on-hold-form' || $_POST['ajax'] === 'playlist-form')) {
            echo CActiveForm::validate($model);
            Yii::app()->end();
        }
    }
}
