<?php

abstract class WizardStep
{
    protected $wizardController;

    private $stepIsDone = false;

    const LanguageAndStyle = 0;
    //const Welcome = 1;
    const TOC = 1;
    const NetworkSettings = 2;
    const ProxySettings = 3;
    const AdminUser = 4;
    const LicenseActivation = 5;
    
    const LastStep = self::LicenseActivation;

    public static function getStepImplementation($step)
    {
        switch ($step) {
            case WizardStep::LanguageAndStyle: 
                return new WizardStepLanguageAndStyle();
            /*    
            case WizardStep::Welcome: 
                return new WizardStepWelcome();
            */    
            case WizardStep::TOC: 
                return new WizardStepTOC();
                
            case WizardStep::NetworkSettings: 
                if (app()->params->applianceMode) {
                    return new WizardStepApplianceNetworkSettings();
                } else {
                    return new WizardStepNetworkSettings();
                }
            case WizardStep::ProxySettings: 
                return new WizardStepProxySettings();
                
            case WizardStep::AdminUser: 
                return new WizardStepAdminUser();
                
            case WizardStep::LicenseActivation: 
                return new WizardStepLicenseActivation();
                
            default: throw new CHttpException(500, 'Internal Wizard Error no such step found; '.$step.'!');
        }
    }

    public function executeStep($wizardController)
    {
        
        $this->wizardController = $wizardController;
        
        if ($this->skipStep()) {
            return true;
        }
        
        if ($this->backToPreviosStep()) {
            $this->wizardController->actionIndex();
            return;
        }
        
        $this->run();
        return $this->stepIsDone;
    }

    protected function setStepIsDone($isDone) {
        $this->stepIsDone = $isDone;
    }
    
    protected function skipStep()
    {
        if (isset($_POST["WizardForm"]["skip"])) {
            unset($_POST["WizardForm"]["skip"]);
            return true;
        }
        return false;
    }
    

    protected function backToPreviosStep() 
    {
        if (isset($_POST["WizardForm"]["prev"])) {
            $wizardModel = WizardHelper::getWizardModel();
            $wizardModel->decrementStep();
            $wizardModel->save();
            unset($_POST["WizardForm"]["prev"]);
            return true;
        }
        return false;
    }

    protected function performAjaxValidation($model, $form)
    {
        if (isset($_POST['ajax']) && $_POST['ajax'] === $form)
        {
            echo CActiveForm::validate($model);
            Yii::app()->end();
        }
    }
    

    abstract protected function run();

}