<?php

class AddressbookAddressesTest extends WebTestCase {
    
    private $testbook1;
    private $testbook2;
    private $testAddresses = array();
    
    protected function onNotSuccessfulTest(Exception $e)
    {
        $this->clearTestEnv();
        parent::onNotSuccessfulTest($e);
    }
    
    private function clearTestEnv()
    {
        $this->removeTestAddresses();
        $this->removePermissions();
        $this->deleteTestbooks();
    }
    
    private function removeTestAddresses()
    {
        foreach ($this->testAddresses as $address) {
            $address->delete();
        }
    }
    
    private function removePermissions()
    {
        $this->writePermissionForTestbook1->delete();
        $this->writePermissionForTestbook2->delete();
        $temPermissions = new Permissions();
        $temPermissions->createPermissionsFile();
    }
     
    private function deleteTestbooks()
    {
        $this->testbook1->delete();
        $this->testbook2->delete();
    }
    
    private function prepareTests()
    {
        app()->language = "de_de";
        date_default_timezone_set("Europe/Berlin");
        $this->createTestBooks();
        $this->setTestPermissions();
        $this->createTestAddresses();
        $this->goToModulePage();
    }
    
    private function createTestBooks()
    {
        $addressbook1 = new Addressbook();
        $addressbook2 = new Addressbook();
        $attributes1 = array(
            "title"=>"TestBook1",
            "type"=>  AddressbookType::group,
            "icon"=>"addressbookAddresses.png"
        );
        $attributes2 = array(
            "title"=>"TestBook2",
            "type"=>AddressbookType::group,
            "icon"=>"addressbookGeneral.png"
        );
        $addressbook1->attributes = $attributes1;
        $addressbook2->attributes = $attributes2;
        
        $addressbook1->save();
        $addressbook2->save();
        
        $this->testbook1 = $addressbook1;
        $this->testbook2 = $addressbook2;
    }
    
    private function setTestPermissions()
    {
        $allUsersGroups = Groups::model()->find(array("type"=>"allUsers"),"id");
        $allUsersGroup = $allUsersGroups[0];                
        $this->writePermissionForTestbook1 = $this->createPermission($this->testbook1, $allUsersGroup->id);
        $this->writePermissionForTestbook2 = $this->createPermission($this->testbook2, $allUsersGroup->id);
        $this->writePermissionForTestbook2->createPermissionsFile();
    }
    
    private function createPermission($addressbook, $allUsersGroupId)
    {
        $permission = new Permissions();
        $permission->attributes = array(
            'groupId' => $allUsersGroupId,
            'type' => 'write',
            'itemId' => $addressbook->id,
        );
        if ($permission->save()) { 
            return $permission;
        } else {
            throw new CHttpException(500, "Can not create permissions!");
        }
    }
    
    private function createTestAddresses()
    {
        //$owners = User::model()->find(array("name"=>"admin"),"name");
        //$owner = $owners[0];
        
        
        for ($i = 0; $i<10; $i++) {
            $newTestAddress = new Addresses();
            $attributes = array(
                'contact_tid'=>$this->testbook1->id,
                'n_family'=>'Test'.$i,
                'n_given'=>'TestAddress'.$i,
            );
                        
            $newTestAddress->attributes = $attributes;
            $newTestAddress->contact_owner = "intern";//$owner->id;
            
            if ( $newTestAddress->save()) {
                $this->testAddresses[] = $newTestAddress;
            } else {
                print_r($newTestAddress->attributes);
                print_r($newTestAddress->errors);
                die();
            }
        }
    }
    
    private function goToModulePage()
    {
        $this->open('?r=addressbookAddresses/default/index');
        $this->assertElementPresent('name=LoginForm[username]');
        $this->type('name=LoginForm[username]', 'admin');
        $this->type('name=LoginForm[password]', 'demo');
        $this->clickAndWait("//input[@value='Login']");
        $this->open('?r=addressbookAddresses/default/index');
    }
    
    private function goToAddressbook($addressbook)
    {
        $bookLinkXpath = "//a[text()='".$addressbook->title."']";
        $this->clickAndWait($bookLinkXpath);
    }
    
    private function checkEditableGeneralTab()
    {
        $visibleIds = array_merge($this->getHeaderInputIds(), $this->getGeneralTabIds());
        $editableIds = array(
            "Addresses_tel_work",
            "Addresses_tel_cell",
            "Addresses_tel_fax",
            "Addresses_contact_email",
            "Addresses_contact_type_drop",
            "Addresses_tel_home",
            "Addresses_tel_cell_private",
            "Addresses_tel_fax_home",
            "Addresses_contact_email_home",
            "Addresses_contact_url_home",
        );
        $notVisibleIds = array_merge($this->getAddressesTabIds(), $this->getCompanyTabIds(),$this->getSocialNetworkTabIds(), $this->getLastContactTabIds());

        foreach ($visibleIds as $id ) {
            $this->assertVisible("//*[@id='".$id."']");
        }
        foreach ($editableIds as $id ) {
            $this->assertEditable("//*[@id='".$id."']");
        }
        foreach ($notVisibleIds as $id ) {
            $this->assertNotVisible("//*[@id='".$id."']");
        }
        
        $this->assertNotVisible("//*[@id='Addresses_contact_type']");
        $this->assertNotEditable("//*[@id='Addresses_contact_bday']");
        $this->checkCrudActionsVisibleInEditMode();
    }
    
    private function getHeaderInputIds()
    {
        return array(
            "Addresses_n_prefix",
            "Addresses_n_given",
            "Addresses_n_middle",
            "Addresses_n_family",
            "Addresses_org_name",
        );
    }
    
    private function getGeneralTabIds()
    {
        return array(
            "organisationPhones",
            "Addresses_tel_work",
            "Addresses_tel_cell",
            "Addresses_tel_fax",
            "Addresses_contact_email",
            "Addresses_contact_type_drop",
            "privatePhones",
            "Addresses_tel_home",
            "Addresses_tel_cell_private",
            "Addresses_tel_fax_home",
            "Addresses_contact_email_home",
            "Addresses_contact_url_home",
            "Addresses_contact_bday",
            "addressSource",
            "Addresses_source"
        );
    }
    
    private function getAddressesTabIds()
    {
        return array(
            "organisationAddress",
            "Addresses_adr_one_street",
            "Addresses_adr_one_postalcode",
            "Addresses_adr_one_locality",
            "Addresses_adr_one_countryname",
            "privateAddress",
            "Addresses_adr_two_street",
            "Addresses_adr_two_postalcode",
            "Addresses_adr_two_locality",
            "Addresses_adr_two_countryname",
        );
    }
    
    private function getCompanyTabIds()
    {
        return array(
            "Addresses_org_unit",
            "Addresses_contact_role",
            "Addresses_contact_room",
            "Addresses_tel_assistent",
            "Addresses_contact_url",
            "Addresses_contact_freebusy_uri",
            "Addresses_contact_calendar_uri",
            "Addresses_contact_pubkey",
            "Addresses_contact_label",
            "Addresses_contact_assistent",
            "Addresses_contact_note",
        );
    }
    
    private function getSocialNetworkTabIds()
    {
        return array(
            "Addresses_facebook_profile_url",
            "Addresses_twitter_profile_url",
            "Addresses_google_profile_url",
            "Addresses_xing_profile_url",
            "Addresses_linkedin_profile_url",
        );
    }
    
    private function getLastContactTabIds()
    {
        return array(
            "lastContactTab"
        );
    }
    
    private function checkNotEditableGeneralTab()
    {
        $book2LinkXpath = "//a[text()='TestBook2']"; //FIXME : use $this->testbook2->title
        $this->clickAndWait($book2LinkXpath);
        $this->click("//*[@id='editToggle']");
        $this->checkCrudActionsVisibleInReadEditModeWithWritePermissions();
        $notEditableIds = array_merge(
            $this->getAddressesTabIds(),
            $this->getCompanyTabIds(),
            $this->getGeneralTabIds(),
            $this->getHeaderInputIds(),
            $this->getLastContactTabIds(),
            $this->getSocialNetworkTabIds()
        );
        foreach ($notEditableIds as $id) {
            if (substr($id,0,7) == "Address") {
                if ($id !== "Addresses_contact_type_drop") { // FIXME : contact type dropdown should be disabled
                    $this->assertNotEditable("//*[@id='".$id."']");
                }
            }
        }
        $notVisibleIds = array_merge(
            $this->getAddressesTabIds(),
            $this->getCompanyTabIds(),
            $this->getGeneralTabIds(),
            $this->getLastContactTabIds(),
            $this->getSocialNetworkTabIds()
        );
        $visibleIds = array("Addresses_source", "addressSource");
        foreach ($notVisibleIds as $id ) {
            if (!in_array($id, $visibleIds)) {
                $this->assertNotVisible("//*[@id='".$id."']");
            }
        }
    }
    
        
    private function checkCrudActionsVisibleInReadEditModeWithWritePermissions()
    {
        $this->assertNotVisible("//*[@id='saveAlias']");
        $this->assertVisible("//*[@id='editToggle']");
        $this->assertVisible("//*[@id='deleteButton']");
    }
    
    private function checkEditableAddressesTab()
    {
        $addressesTabLinkXpath = '//*[@id="yw7"]/li[2]/a';
        $this->click($addressesTabLinkXpath);
        sleep(1); // FIXME : wait for visible 
        $visibleIds = array_merge($this->getHeaderInputIds(),$this->getAddressesTabIds());
        foreach ($visibleIds as $id ) {
            $this->assertVisible("//*[@id='".$id."']");
        }
        
        $editableIds = $this->getAddressesTabIds();
        foreach ($editableIds as $id ) {
            
            if (substr($id,0,7) == "Address") {
                $this->assertEditable("//*[@id='".$id."']");
            }
        }
        $notVisibleIds = array_merge($this->getGeneralTabIds(),$this->getCompanyTabIds(), $this->getLastContactTabIds(),$this->getSocialNetworkTabIds());
        foreach ($notVisibleIds as $id ) {
            $this->assertNotVisible("//*[@id='".$id."']");
        }
        $this->assertNotVisible("//*[@id='Addresses_contact_type']");
        $this->checkCrudActionsVisibleInEditMode();
    }

    private function checkNotEditableAddressesTab()
    {
        $lastContactTabLinkXpath = '//*[@id="yw7"]/li[2]/a';
        $this->click($lastContactTabLinkXpath);
        sleep(1); // FIXME : wait for visible
        $notEditableIds = array_merge(
            $this->getAddressesTabIds(),
            $this->getCompanyTabIds(),
            $this->getGeneralTabIds(),
            $this->getLastContactTabIds(),
            $this->getSocialNetworkTabIds()
        );
        foreach (array_merge($notEditableIds,$this->getHeaderInputIds()) as $id) {
            if (substr($id,0,7) == "Address") {
                if ($id !== "Addresses_contact_type_drop") { // FIXME : contact type dropdown should be disabled
                    $this->assertNotEditable("//*[@id='".$id."']");
                }
            }
        }
        foreach ($notEditableIds as $id) {
            $this->assertNotVisible("//*[@id='".$id."']");
        }
    }
    
    private function checkEditableCompanyTab()
    {
        $companyTabLinkXpath = '//*[@id="yw7"]/li[3]/a';
        $this->click($companyTabLinkXpath);
        sleep(1); // FIXME : wait for visible 
        $visibleIds = array_merge($this->getHeaderInputIds(),$this->getCompanyTabIds());
        foreach ($visibleIds as $id ) {
            $this->assertVisible("//*[@id='".$id."']");
        }
        
        $editableIds = $this->getAddressesTabIds();
        foreach ($editableIds as $id ) {
            
            if (substr($id,0,7) == "Address") {
                $this->assertEditable("//*[@id='".$id."']");
            }
        }
        $notVisibleIds = array_merge($this->getGeneralTabIds(),$this->getAddressesTabIds(), $this->getLastContactTabIds(),$this->getSocialNetworkTabIds());
        foreach ($notVisibleIds as $id ) {
            $this->assertNotVisible("//*[@id='".$id."']");
        }
        $this->assertNotVisible("//*[@id='Addresses_contact_type']");
        $this->checkCrudActionsVisibleInEditMode();
    }
    
    private function checkNotEditableCompanyTab()
    {
        $lastContactTabLinkXpath = '//*[@id="yw7"]/li[3]/a';
        $this->click($lastContactTabLinkXpath);
        sleep(1); // FIXME : wait for visible
        $notEditableIds = array_merge(
            $this->getAddressesTabIds(),
            $this->getCompanyTabIds(),
            $this->getGeneralTabIds(),
            $this->getLastContactTabIds(),
            $this->getSocialNetworkTabIds()
        );
        foreach (array_merge($notEditableIds,$this->getHeaderInputIds()) as $id) {
            if (substr($id,0,7) == "Address") {
                if ($id !== "Addresses_contact_type_drop") { // FIXME : contact type dropdown should be disabled
                    $this->assertNotEditable("//*[@id='".$id."']");
                }
            }
        }
        foreach ($notEditableIds as $id) {
            $this->assertNotVisible("//*[@id='".$id."']");
        }
    }
    
    private function checkEditableSocialNetworkTab()
    {
        $socialNetworkTabLinkXpath = '//*[@id="yw7"]/li[4]/a';
        $this->click($socialNetworkTabLinkXpath);
        sleep(1); // FIXME : wait for visible 
        $visibleIds = array_merge($this->getHeaderInputIds(),$this->getSocialNetworkTabIds());
        foreach ($visibleIds as $id ) {
            $this->assertVisible("//*[@id='".$id."']");
        }
        
        $editableIds = $this->getAddressesTabIds();
        foreach ($editableIds as $id ) {
            
            if (substr($id,0,7) == "Address") {
                $this->assertEditable("//*[@id='".$id."']");
            }
        }
        $notVisibleIds = array_merge($this->getGeneralTabIds(),$this->getAddressesTabIds(), $this->getLastContactTabIds(),$this->getCompanyTabIds());
        foreach ($notVisibleIds as $id ) {
            $this->assertNotVisible("//*[@id='".$id."']");
        }
        $this->assertNotVisible("//*[@id='Addresses_contact_type']");
        $this->checkCrudActionsVisibleInEditMode();
    }
    
    private function checkNotEditableSocialNetworkTab()
    {
        $lastContactTabLinkXpath = '//*[@id="yw7"]/li[4]/a';
        $this->click($lastContactTabLinkXpath);
        sleep(1); // FIXME : wait for visible
        $notEditableIds = array_merge(
            $this->getAddressesTabIds(),
            $this->getCompanyTabIds(),
            $this->getGeneralTabIds(),
            $this->getLastContactTabIds(),
            $this->getSocialNetworkTabIds()
        );
        foreach (array_merge($notEditableIds,$this->getHeaderInputIds()) as $id) {
            if (substr($id,0,7) == "Address") {
                if ($id !== "Addresses_contact_type_drop") { // FIXME : contact type dropdown should be disabled
                    $this->assertNotEditable("//*[@id='".$id."']");
                }
            }
        }
        foreach ($notEditableIds as $id) {
            $this->assertNotVisible("//*[@id='".$id."']");
        }
    }    

    private function checkAllRows()
    {
        for ($i = 1 ; $i<11; $i++) {
            $xpathToGivenCell = '//*[@id="addresses-grid"]/table/tbody/tr['.$i.']/td[2]';
            $this->assertElementContainsText($xpathToGivenCell, "TestAddress".($i-1));
            $xpathToGivenRow = '//*[@id="addresses-grid"]/table/tbody/tr['.$i.']';
            $xpathToCheckbox = '//div[@id="addresses-grid"]/table/tbody/tr['.$i.']/td[1]/input';
            $this->assertNotChecked($xpathToCheckbox);
            $this->assertEditable($xpathToCheckbox);
            $this->checkAddressDetailViewHasChanged($xpathToGivenRow, $i);
        }
    }
    
    
    private function processAjaxAction($action)
    {
        $this->chooseOkOnNextConfirmation();
        $this->runScript('$("#function").val("'.$action.'").trigger("change")');
        sleep(1);
        
        $confirmation = $this->getConfirmation();
        $this->compareConfirmMessage($action,$confirmation);
        sleep(1);
        $this->waitForNotVisible("//div[@id='addresses-grid_Modal']");
    }
    
    private function compareConfirmMessage($action, $confirmation)
    {
        $expectedMessage = "";
        if ($action == "copy_".$this->testbook2->id) {
            $expectedMessage = "Kopiere ausgewählte Adresse(n) nach \"".$this->testbook2->title."\" Adressbuch?";
        } else if ($action == "delete") {
            $expectedMessage = "Lösche ausgewählte Adresse(n)?";
        } else if ($action == 'move_'.$this->testbook2->id) {
            $expectedMessage = "Verschiebe ausgewählte Adresse(n) nach  \"".$this->testbook2->title."\" Adressbuch?";
        } else if ($action == "copy") {
            $expectedMessage = "Kopiere ausgewählte Adresse(n)?";
        }
        $this->assertEquals($confirmation, $expectedMessage);
    }

    private function checkCrudActionsVisibleInEditMode()
    {
        $this->assertVisible("//*[@id='saveAlias']");
        $this->assertVisible("//*[@id='editToggle']");
        $this->assertVisible("//*[@id='deleteButton']");
    }
     
    private function checkEditableLastContactTab() 
    {
        $lastContactTabLinkXpath = '//*[@id="yw7"]/li[5]/a';
        $this->click($lastContactTabLinkXpath);
        sleep(1); // FIXME : wait for visible 
        $visibleIds = array_merge($this->getHeaderInputIds(),$this->getLastContactTabIds());
        foreach ($visibleIds as $id ) {
            $this->assertVisible("//*[@id='".$id."']");
        }
        
        $editableIds = $this->getAddressesTabIds();
        foreach ($editableIds as $id ) {
            
            if (substr($id,0,7) == "Address") {
                $this->assertEditable("//*[@id='".$id."']");
            }
        }
        $notVisibleIds = array_merge($this->getGeneralTabIds(),$this->getAddressesTabIds(), $this->getSocialNetworkTabIds(),$this->getCompanyTabIds());
        foreach ($notVisibleIds as $id ) {
            $this->assertNotVisible("//*[@id='".$id."']");
        }
        $this->assertNotVisible("//*[@id='Addresses_contact_type']");
        $this->checkCrudActionsVisibleInEditMode();
    }
    
    
    private function checkAddressDetailViewHasChanged($xpathToGivenRow,$i)
    {
        $this->click($xpathToGivenRow);
        $this->waitForCondition("var textFieldValue = selenium.getValue(\"//input[@id='Addresses_n_given']\"); \"TestAddress".($i-1)."\" == textFieldValue");
        $given = $this->getValue("//input[@id='Addresses_n_given']");
        $this->assertTrue($given == "TestAddress".($i-1), "Given name in detail view is not the same as the selected one!");
    }

    private function checkCreateAddress()
    {
        $this->goToAddressbook($this->testbook2);
        $this->checkCrudActionsVisibleInEditMode();
        $keyValueForForm = $this->getKeyValueForForm("aa");
        
        foreach ($keyValueForForm as $key => $value) {
            $this->type("//*[@id='".$key."']", $value);
        }
        sleep(1);
        $this->assertVisible('//*[@id="Addresses_contact_email_em_"]');
        $this->assertVisible('//*[@id="Addresses_contact_email_home_em_"]');
        //FIXME : "errorList" disapears in selenium ...
        $this->type("//*[@id='Addresses_contact_email']", "a@a.a");
        $this->type("//*[@id='Addresses_contact_email_home']", "a@a.a");
        
        $this->clickAndWait('//*[@id="saveAlias"]');
        $this->checkCrudActionsVisibleInReadEditModeWithWritePermissions();
        $keyValueForForm["Addresses_contact_email"] = "a@a.a";
        $keyValueForForm["Addresses_contact_email_home"] = "a@a.a";
        
        foreach ($keyValueForForm as $key => $value) {
            $xpath = "//*[@id='".$key."']";
            //$this->assertVisible($xpath);
            $this->assertNotEditable($xpath);
            $this->assertElementValueEquals($xpath, $value);
        }
    }
    
    private function getKeyValueForForm($defaultValue)
    {
        $keyValueForForm = array(
            //Header Input fields
            "Addresses_n_prefix" =>"$defaultValue",
            "Addresses_n_given" =>"$defaultValue",
            "Addresses_n_middle" =>"$defaultValue",
            "Addresses_n_family" =>"$defaultValue",
            "Addresses_org_name" =>"$defaultValue",
            //GeneralTab
            "Addresses_tel_work" => "$defaultValue",
            "Addresses_tel_cell" => "$defaultValue",
            "Addresses_tel_fax" => "$defaultValue",
            "Addresses_contact_email" => "$defaultValue",
            "Addresses_tel_home" => "$defaultValue",
            "Addresses_tel_cell_private" => "$defaultValue",
            "Addresses_tel_fax_home" => "$defaultValue",
            "Addresses_contact_email_home" => "$defaultValue",
            "Addresses_contact_url_home" => "$defaultValue",
            //AddressesTab
            "Addresses_adr_one_street" => "$defaultValue",
            "Addresses_adr_one_postalcode" => "$defaultValue",
            "Addresses_adr_one_locality" => "$defaultValue",
            "Addresses_adr_one_countryname" => "$defaultValue",
            "Addresses_adr_two_street" => "$defaultValue",
            "Addresses_adr_two_postalcode" => "$defaultValue",
            "Addresses_adr_two_locality" => "$defaultValue",
            "Addresses_adr_two_countryname" => "$defaultValue",
            //CompanyTab
            "Addresses_org_unit" => "$defaultValue",
            "Addresses_contact_role" => "$defaultValue",
            "Addresses_contact_room" => "$defaultValue",
            "Addresses_tel_assistent" => "$defaultValue",
            "Addresses_contact_url" => "$defaultValue",
            "Addresses_contact_freebusy_uri" => "$defaultValue",
            "Addresses_contact_calendar_uri" => "$defaultValue",
            "Addresses_contact_pubkey" => "$defaultValue",
            "Addresses_contact_label" => "$defaultValue",
            "Addresses_contact_assistent" => "$defaultValue",
            "Addresses_contact_note" => "$defaultValue",
            //SocialNetworkTab
            "Addresses_facebook_profile_url" => "$defaultValue",
            "Addresses_twitter_profile_url" => "$defaultValue",
            "Addresses_google_profile_url" => "$defaultValue",
            "Addresses_xing_profile_url" => "$defaultValue",
            "Addresses_linkedin_profile_url" => "$defaultValue",
        );
        return $keyValueForForm;
    }
    
    private function checkUpdateAddress()
    {
        $this->assertElementNotPresent("//tr[@class='even']");
        $this->assertElementPresent("//li[@id='editToggle']");
        $this->assertElementNotPresent("//li[@id='editToggle' and @class ='active']");
        $this->click("//li[@id='editToggle']");
        $this->assertElementPresent("//li[@id='editToggle' and @class ='active']");
        
        $keyValueForForm = $this->getKeyValueForForm("bb");
        foreach ($keyValueForForm as $key => $value) {
            $this->type("//*[@id='".$key."']", $value);
        }
        $this->type("//*[@id='Addresses_contact_email']", "b@b.b");
        $this->type("//*[@id='Addresses_contact_email_home']", "b@b.b");
        $this->clickAndWait('//*[@id="saveAlias"]');

        $this->checkCrudActionsVisibleInReadEditModeWithWritePermissions();
        $keyValueForForm["Addresses_contact_email"] = "b@b.b";
        $keyValueForForm["Addresses_contact_email_home"] = "b@b.b";
        
        foreach ($keyValueForForm as $key => $value) {
            $xpath = "//*[@id='".$key."']";
            //$this->assertVisible($xpath);
            $this->assertNotEditable($xpath);
            $this->assertElementValueEquals($xpath, $value);
        }
    }
    
    private function checkDeleteAddress() 
    {
        $this->goToAddressbook($this->testbook2);
        $this->checkCrudActionsVisibleInReadEditModeWithWritePermissions();
        $this->click('//*[@id="deleteButton"]');
    }

    ##################TESTS##################
    
    public function testGeneralTab()
    {
       $this->prepareTests();
       $this->goToAddressbook($this->testbook2);
       $this->checkEditableGeneralTab();
       $this->checkNotEditableGeneralTab();
       $this->clearTestEnv();
    }
    
    public function testAddressesTab()
    {
        $this->prepareTests();
        $this->goToAddressbook($this->testbook2);
        $this->checkEditableAddressesTab();
        $this->goToAddressbook($this->testbook1);
        $this->checkNotEditableAddressesTab();
        $this->clearTestEnv();
    }
    
    public function testCompanyTab()
    {
        $this->prepareTests();
        $this->goToAddressbook($this->testbook2);
        $this->checkEditableCompanyTab();
        $this->goToAddressbook($this->testbook1);
        $this->checkNotEditableCompanyTab();
        $this->clearTestEnv();
    }
    
    public function testSocialNetworkTab()
    {
        $this->prepareTests();
        $this->goToAddressbook($this->testbook2);
        $this->checkEditableSocialNetworkTab();
        $this->goToAddressbook($this->testbook1);
        $this->checkNotEditableSocialNetworkTab();
        $this->clearTestEnv();
    }
    
    public function testEmptyAddressbook() {
        $this->prepareTests();
        //validate empty addressbook
        $this->goToAddressbook($this->testbook2);
        $this->assertElementPresent("//div[@id='addresses-grid']");
        $xpathToNoEntryText = '//*[@id="addresses-grid"]/table/tbody/tr/td/span';
        $this->assertElementContainsText($xpathToNoEntryText,"Keine Ergebnisse gefunden.");
        
        $this->assertElementPresent("//*[@id='Addresses_n_given']");
        $this->assertEditable("//*[@id='Addresses_n_given']");
        
        $this->clearTestEnv();
    } 
    
    public function testAddressesCRUDOperations()
    {
        $this->prepareTests();
        $this->checkCreateAddress();
        $this->checkUpdateAddress();
        $this->checkDeleteAddress();
        $this->clearTestEnv();
    }
   
    public function testAddressesGrid()
    {
        $this->prepareTests();
        $this->goToAddressbook($this->testbook1);
        $this->checkAllRows();
        
        $this->click("//*[@id='selectAll']");
        for ($i = 1 ; $i<11; $i++) {
            $xpathToCheckbox = '//div[@id="addresses-grid"]/table/tbody/tr['.$i.']/td[1]/input';
            $this->assertChecked($xpathToCheckbox);
            $this->assertNotEditable($xpathToCheckbox);
        }
       
        $this->clearTestEnv();
    }
    
    public function testCopyAllToAddressBook()
    {
        $this->prepareTests();
        
        $this->goToAddressbook($this->testbook1);
        $this->click("//input[@id='selectAll']");
        $this->processAjaxAction("copy_".$this->testbook2->id);
        $this->goToAddressbook($this->testbook2);
        
        $this->assertValue("Addresses_n_given", "TestAddress0");
        $this->checkAllRows();
        $this->clearTestEnv();
    }
    
    public function testCopySelectedAddressesToAddressbook()
    {
        $this->prepareTests();
        
        $this->goToAddressbook($this->testbook1);
        $this->check("//input[@id='addresses-grid_c0_1']");
        $this->check("//input[@id='addresses-grid_c0_2']");
        $this->check("//input[@id='addresses-grid_c0_3']");

        $this->processAjaxAction("copy_".$this->testbook2->id);
        $this->goToAddressbook($this->testbook2);
       
        $this->assertValue("Addresses_n_given", "TestAddress1");
        
        for ($i = 1 ; $i<4; $i++) {
            $xpathToGivenCell = '//*[@id="addresses-grid"]/table/tbody/tr['.$i.']/td[2]';
            $this->assertElementContainsText($xpathToGivenCell, "TestAddress".($i));
            $xpathToGivenRow = '//*[@id="addresses-grid"]/table/tbody/tr['.$i.']';
            $xpathToCheckbox = '//div[@id="addresses-grid"]/table/tbody/tr['.$i.']/td[1]/input';
            $this->assertNotChecked($xpathToCheckbox);
            $this->assertEditable($xpathToCheckbox);
            $this->checkAddressDetailViewHasChanged($xpathToGivenRow, ($i+1));
        }
        $this->clearTestEnv();
    }
    
    public function testCopyNoAddresses()
    {
        $this->prepareTests();
        $this->goToAddressbook($this->testbook1);
        $this->processAjaxAction("copy_".$this->testbook2->id);
        $this->goToAddressbook($this->testbook2);
        
        $xpathToGivenCell = "//*[@id='addresses-grid']/table/tbody/tr[1]/td/span";
        $this->assertElementContainsText($xpathToGivenCell, "Keine Ergebnisse gefunden.");
        $this->clearTestEnv();
    }
    
    public function testDeleteAllAddresses()
    {
        $this->prepareTests();
        
        $this->goToAddressbook($this->testbook1);
        $this->click("//input[@id='selectAll']");
        $this->processAjaxAction("delete");
        
        $xpathToGivenCell = "//*[@id='addresses-grid']/table/tbody/tr[1]/td/span";
        $this->assertElementContainsText($xpathToGivenCell, "Keine Ergebnisse gefunden.");
        
        $this->clearTestEnv();
    }
    
    public function testDeleteSelectedAddresses()
    {
        $this->prepareTests();
        
        $this->goToAddressbook($this->testbook1);
        
        $this->check("//input[@id='addresses-grid_c0_1']");
        $this->check("//input[@id='addresses-grid_c0_2']");
        $this->check("//input[@id='addresses-grid_c0_3']");
        
        $this->processAjaxAction("delete");
        
        $this->assertElementContainsText("//input[@id='addresses-grid_c0_0']/../../td[2]", "TestAddress0");
        $this->assertElementContainsText("//input[@id='addresses-grid_c0_1']/../../td[2]", "TestAddress4");
        
        $this->clearTestEnv();
    }
    
    public function testDeleteNoAddresses()
    {
        $this->prepareTests();
        $this->goToAddressbook($this->testbook1);
        $this->processAjaxAction("delete");
        
        for ($i = 1 ; $i<4; $i++) {
            $xpathToGivenCell = '//*[@id="addresses-grid"]/table/tbody/tr['.$i.']/td[2]';
            $this->assertElementContainsText($xpathToGivenCell, "TestAddress".($i-1));
            $xpathToGivenRow = '//*[@id="addresses-grid"]/table/tbody/tr['.$i.']';
            $xpathToCheckbox = '//div[@id="addresses-grid"]/table/tbody/tr['.$i.']/td[1]/input';
            $this->assertNotChecked($xpathToCheckbox);
            $this->assertEditable($xpathToCheckbox);
            $this->checkAddressDetailViewHasChanged($xpathToGivenRow, $i);
        }
        
        $this->clearTestEnv();
    }
    
    public function testMoveAllAddressesToAddressbook()
    {
        $this->prepareTests();

        $this->goToAddressbook($this->testbook1);
        $this->click("//input[@id='selectAll']");
        
        $this->processAjaxAction('move_'.$this->testbook2->id);

        $xpathToGivenCell = "//*[@id='addresses-grid']/table/tbody/tr[1]/td/span";
        $this->assertElementContainsText($xpathToGivenCell, "Keine Ergebnisse gefunden.");
        
        $this->goToAddressbook($this->testbook2);
        $this->checkAllRows();
        $this->clearTestEnv();
    }
    
    public function testMoveSelectedAddressesToAddressbook()
    {
        $this->prepareTests();
        
        $this->goToAddressbook($this->testbook1);
        $this->check("//input[@id='addresses-grid_c0_1']");
        $this->check("//input[@id='addresses-grid_c0_2']");
        $this->check("//input[@id='addresses-grid_c0_3']");
        
        $this->processAjaxAction('move_'.$this->testbook2->id);
        
        $this->goToAddressbook($this->testbook2);
        $this->assertElementContainsText("//input[@id='addresses-grid_c0_0']/../../td[2]", "TestAddress1");
        $this->assertElementContainsText("//input[@id='addresses-grid_c0_1']/../../td[2]", "TestAddress2");
        $this->assertElementContainsText("//input[@id='addresses-grid_c0_2']/../../td[2]", "TestAddress3");
        $this->clearTestEnv();
    }
    
    public function testMoveNoAddressesToAddressbook()
    {
        $this->prepareTests();
        
        $this->goToAddressbook($this->testbook1);
        $this->processAjaxAction('move_'.$this->testbook2->id);
        $this->checkAllRows();
        $this->goToAddressbook($this->testbook2);
        $xpathToNoEntryText = '//*[@id="addresses-grid"]/table/tbody/tr/td/span';
        $this->assertElementContainsText($xpathToNoEntryText,"Keine Ergebnisse gefunden.");
        $this->clearTestEnv();
    }
    
    public function testCopyAllAddressesToCurrentAddressbook()
    {
        $this->prepareTests();
        $this->goToAddressbook($this->testbook1);
        
        $this->click("//*[@id='selectAll']");
        $this->processAjaxAction("copy");
        $this->waitForPageToLoad();
        //sleep(2); // wait for grid reload
        
        for ($i=0;$i<=14;$i=$i+2) {
            
            $this->assertEquals(
                $this->getText("//input[@id='addresses-grid_c0_".$i."']/../../td[2]"),
                $this->getText("//input[@id='addresses-grid_c0_".($i+1)."']/../../td[2]")
            );
        }
        $value1 = $this->getText("//input[@id='addresses-grid_c0_16']/../../td[2]");
        $this->clickAndWait('//*[@id="yw4"]/li[4]/a');
        //$this->runScript('$(".pagination").find(".next").find("a").click()');
        //$this->waitForCondition("selenium.browserbot.getCurrentWindow().$('#addresses-grid').hasClass('grid-view-loading') === false");
        
        $value2 = $this->getText("//input[@id='addresses-grid_c0_0']/../../td[2]");
        $this->assertEquals($value1, $value2);

        for ($i=1;$i<=2;$i=$i+2) {
            $this->assertEquals(
                $this->getText("//input[@id='addresses-grid_c0_".$i."']/../../td[2]"),
                $this->getText("//input[@id='addresses-grid_c0_".($i+1)."']/../../td[2]")
            );
        }
        $this->assertElementNotPresent("//input[@id='addresses-grid_c0_3']/../../td[2]");
        
        $this->clearTestEnv();
    }
    
    public function testCopySelectedAddressesToCurrentAddressbook()
    {
        $this->prepareTests();
        
        $this->goToAddressbook($this->testbook1);
        
        $this->check("//input[@id='addresses-grid_c0_1']");
        $this->check("//input[@id='addresses-grid_c0_2']");
        $this->check("//input[@id='addresses-grid_c0_3']");
        
        $this->processAjaxAction("copy");
        
        $this->assertEquals($this->getText("//input[@id='addresses-grid_c0_1']/../../td[2]"), $this->getText("//input[@id='addresses-grid_c0_2']/../../td[2]"));
        $this->assertEquals($this->getText("//input[@id='addresses-grid_c0_3']/../../td[2]"), $this->getText("//input[@id='addresses-grid_c0_4']/../../td[2]"));
        $this->assertEquals($this->getText("//input[@id='addresses-grid_c0_5']/../../td[2]"), $this->getText("//input[@id='addresses-grid_c0_6']/../../td[2]"));
        
        
        $this->clearTestEnv();
    }
    
    public function testCopyNoAddressesToCurrentAddressbook()
    {
        $this->prepareTests();
        
        $this->goToAddressbook($this->testbook1);
        $this->processAjaxAction("copy");
        $this->checkAllRows();
        for ($i = 1 ; $i<11; $i++) {
            $xpathToGivenCell = '//*[@id="addresses-grid"]/table/tbody/tr['.$i.']/td[2]';
            $this->assertElementContainsText($xpathToGivenCell, "TestAddress".($i-1));
            $xpathToGivenRow = '//*[@id="addresses-grid"]/table/tbody/tr['.$i.']';
            $xpathToCheckbox = '//div[@id="addresses-grid"]/table/tbody/tr['.$i.']/td[1]/input';
            $this->assertNotChecked($xpathToCheckbox);
            $this->assertEditable($xpathToCheckbox);
            $this->checkAddressDetailViewHasChanged($xpathToGivenRow, $i);
        }
        
        $this->clearTestEnv();
    }
    
}